#ifndef _iso_base_tools_h_
/** \file iso_base_tools.h
 * \class iso::base::Tools singleton class providing a basic set basic utility functions
 * \brief singleton class providing a basic set basic utility functions
 *  
 *  
 *  Created by Daniel Bisig on 7/22/07.
 */

#define _iso_base_tools_h_

#include "iso_base_types.h"
#include <iostream>

namespace iso
{

namespace base
{

class Tools
{
public:
	/**
	\brief get singleton instance
	\return singleton instance
	*/
	static Tools& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();
	
	/**
	\brief convert byte buffer into int
	\param pBuffer byte buffer
	\return int
	*/
	template<class Type> void buffer2Type( const char* pBuffer, Type& pValue );

	/**
	\brief convert byte buffer into int (little endian version)
	\param pBuffer byte buffer
	\return int
	*/
	template<class Type> void buffer2TypeLE( const char* pBuffer, Type& pValue );

	/**
	\brief convert byte buffer into int (big endian version)
	\param pBuffer byte buffer
	\return int
	*/
	template<class Type> void buffer2TypeBE( const char* pBuffer, Type& pValue );

	/**
	\brief convert type into buffer
	\param pValue type value
	\param pBuffer target buffer
	\remarks beforms byte swap if necessary
	*/
	template<class Type> void type2Buffer( const Type& pValue, char* pBuffer );

	/**
	\brief convert type into buffer (iittle endian version)
	\param pValue type value
	\param pBuffer target buffer
	\remarks beforms byte swap if necessary
	*/
	template<class Type> void type2BufferLE( const Type& pValue, char* pBuffer );

	/**
	\brief convert type into buffer (big endian version)
	\param pValue type value
	\param pBuffer target buffer
	\remarks beforms byte swap if necessary
	*/
	template<class Type> void type2BufferBE( const Type& pValue, char* pBuffer );

	/**
	\brief convert type array into buffer
	\param pValues type array
	\param pValueCount number of type values
	\param pBuffer target buffer
	\remarks beforms byte swap if necessary
	*/
	template<class Type> void type2Buffer( const Type* pValues, unsigned int pValueCount, char* pBuffer );

	/**
	\brief convert type array into buffer (little endian version)
	\param pValues type array
	\param pValueCount number of type values
	\param pBuffer target buffer
	\remarks beforms byte swap if necessary
	*/
	template<class Type> void type2BufferLE( const Type* pValues, unsigned int pValueCount, char* pBuffer );
	
	/**
	\brief convert type array into buffer (big endian version)
	\param pValues type array
	\param pValueCount number of type values
	\param pBuffer target buffer
	\remarks beforms byte swap if necessary
	*/
	template<class Type> void type2BufferBE( const Type* pValues, unsigned int pValueCount, char* pBuffer );	

	/**
	\brief convert buffer into buffer
	\param pSource source buffer
	\param pValueSize size of value type stored in source buffer
	\param pValueCount number of values stored in source buffer
	\param pTarget target buffer
	\remarks beforms byte swap if necessary
	*/
	void buffer2Buffer( const char* pSource, unsigned int pValueSize, unsigned int pValueCount, char* pTarget );

	/**
	\brief copy byte buffer into byte buffer
	\param pSource source byte array
	\param pTarget target byte array
	\param pArraySize size of array
	\remarks performs byte swap if necessary
	*/
	void buffer2Buffer( char* pSource, char* pTarget, unsigned int pArraySize );

	/**
	\brief byteswap from source byte array to target byte array
	\param pSource source byte array
	\param pTarget target byte array
	\param pArraySize size of array
	
	convert from big endian to little endian and vice versa
	*/
	void byteswap(char* pSource, char* pTarget, unsigned int pArraySize);
	
	/**
	\brief byteswap for float values
	\param pValue input value
	\return byteswapped value
	
	convert from big endian to little endian and vice versa
	*/
	float byteswap(float pValue);

	/**
	\brief byteswap for double values
	\param pValue input value
	\return byteswapped value
	
	convert from big endian to little endian and vice versa
	*/
	double byteswap(double pValue);
	
	/**
	\brief return absolute time (in micro secs)
	\return absolute time (in micro secs)
	*/
	int64 time();

	/**
	\brief return absolute time (in mili secs)
	\return absolute time (in mili secs)
	*/
	double milisecs();
	
	/**
	\brief return absolute time (in micro secs)
	\return absolute time (in micro secs)
	*/	
	int64 absTime();
	
	/**
	\brief return relative time since the first instantiation of this class (in micro secs)
	\return relative time since the first instantiation of this class (in micro secs)
	*/
	int64 relTime();

protected:
	/**
	\brief singleton instance
	*/
	static Tools* sTools;

	/**
	\brief start time
	*/
	Time mStartTime;

	/**
	\brief current time
	*/
	Time mCurrentTime;
	
	/**
	\brief time zone
	*/
	TimeZone mTimeZone;

	/**
	\brief default constructor
	*/
	Tools();
	
	/**
	\brief destructor
	*/	
	~Tools();
};

template<class Type> 
void 
Tools::buffer2Type( const char* pBuffer, Type& pValue )
{
	int typeSize = sizeof(pValue);
	char* typeBuf = (char*)( &pValue );
	
#ifdef _iso_ppc_

	for(int tI=0, bI=typeSize - 1; tI < typeSize; ++tI, --bI) typeBuf[tI] = pBuffer[bI];

#else

	memcpy( typeBuf, pBuffer, typeSize );
	
#endif

}

template<class Type> 
void 
Tools::buffer2TypeLE( const char* pBuffer, Type& pValue )
{
	int typeSize = sizeof(pValue);
	char* typeBuf = (char*)( &pValue );

	for(int tI=0, bI=typeSize - 1; tI < typeSize; ++tI, --bI) typeBuf[tI] = pBuffer[bI];
}

template<class Type> 
void 
Tools::buffer2TypeBE( const char* pBuffer, Type& pValue )
{
	int typeSize = sizeof(pValue);
	char* typeBuf = (char*)( &pValue );
	
	memcpy( typeBuf, pBuffer, typeSize );
}

template<class Type> 
void 
Tools::type2Buffer( const Type& pValue, char* pBuffer )
{
	int typeSize = sizeof(pValue);
	char* typeBuf = (char*)( &pValue );
	
#ifdef _iso_ppc_

	for(int tI=typeSize - 1, bI=0; bI < typeSize; --tI, ++bI) pBuffer[bI] = typeBuf[tI];

#else

	memcpy( pBuffer, typeBuf, typeSize );
	
#endif

}

template<class Type> 
void 
Tools::type2BufferLE( const Type& pValue, char* pBuffer )
{
	int typeSize = sizeof(pValue);
	char* typeBuf = (char*)( &pValue );
	
	for(int tI=typeSize - 1, bI=0; bI < typeSize; --tI, ++bI) pBuffer[bI] = typeBuf[tI];
}

template<class Type> 
void 
Tools::type2BufferBE( const Type& pValue, char* pBuffer )
{
	int typeSize = sizeof(pValue);
	char* typeBuf = (char*)( &pValue );

	memcpy( pBuffer, typeBuf, typeSize );
}

template<class Type> 
void 
Tools::type2Buffer( const Type* pValues, unsigned int pValueCount, char* pBuffer )
{
	int typeSize = sizeof(Type);
	int byteCount = pValueCount * typeSize;
	char* typeBuf = (char*)( pValues );
	
	#ifdef _iso_ppc_
	
	int typeSize_1 = typeSize - 1;
	
	for(int vI=0, bI=0; vI < byteCount; vI += typeSize)
	{
		for(int tI=typeSize_1; tI >= 0; --tI, ++bI)
		{
			pBuffer[bI] = typeBuf[vI + tI];
		}
	}
	
	#else
	
	memcpy( pBuffer, typeBuf, byteCount );
	
	#endif
}

template<class Type> 
void 
Tools::type2BufferLE( const Type* pValues, unsigned int pValueCount, char* pBuffer )
{
	int typeSize = sizeof(Type);
	int byteCount = pValueCount * typeSize;
	char* typeBuf = (char*)( pValues );
	int typeSize_1 = typeSize - 1;
	
	for(int vI=0, bI=0; vI < byteCount; vI += typeSize)
	{
		for(int tI=typeSize_1; tI >= 0; --tI, ++bI)
		{
			pBuffer[bI] = typeBuf[vI + tI];
		}
	}
}

template<class Type> 
void 
Tools::type2BufferBE( const Type* pValues, unsigned int pValueCount, char* pBuffer )
{
	int typeSize = sizeof(Type);
	char* typeBuf = (char*)( pValues );
	
	memcpy( pBuffer, typeBuf, typeSize *  pValueCount );
}

};

};

#endif