/** \file iso_com_messenger.h
 *
 *  Created by Daniel Bisig on 6/2/09.
 */

#ifndef _iso_com_messenger_h_
#define _iso_com_messenger_h_

#include <iso_base/iso_base_types.h>
#include "iso_com_exception.h"
#include "iso_com_message.h"
#include <QHash>
#include <QMultiHash>

namespace iso 
{

namespace com
{

class Sender;
class Receiver;
class MessageListener;

class Messenger
{
public:

//	enum CommunicationProtocol
//	{
//		UDP,
//		TCP
//	};
//
//	enum CommunicationFormat
//	{
//		Default,
//		OSC
//	};

	/**
	\brief create messenger
	*/
	Messenger();
	
	/**
	\brief destructor
	*/
	virtual ~Messenger();

	/**
	 \brief return sender names
	 \return sender names
	 */
	QList< base::String > senderNames() const;
	
	/**
	\brief check sender
	\param pSenderName name of sender
	\return true if sender is found, false otherwise
	*/	
	bool checkSender( const base::String& pSenderName ) const;
		
	/**
	\brief retrieve sender
	\param pSenderName name of sender
	\return sender
	\exception ComException sender not found
	*/
	Sender* sender( const base::String& pSenderName ) throw (ComException);
	
	/**
	\brief return sender format
	\param pSender sender
	\returns sender format
	*/
	Format senderFormat( Sender* pSender );

	/**
	\brief return sender format
	\param pSenderName name of sender
	\returns sender format
	\exception ComException sender not found
	*/
	Format senderFormat( const base::String& pSenderName ) const throw (ComException);

	/**
	 \brief return sender protocol
	 \param pSenderName name of sender
	 \returns sender protocol
	 \exception ComException sender not found
	 */
	Protocol senderProtocol( const base::String& pSenderName ) const throw (ComException);

	/**
	 \brief return sender ip
	 \param pSenderName name of sender
	 \returns sender ip
	 \exception ComException sender not found
	 */
	base::String senderIP( const base::String& pSenderName ) const throw (ComException);	

	/**
	 \brief return sender ip
	 \param pSenderName name of sender
	 \returns sender ip
	 \exception ComException sender not found
	 */
	unsigned int senderPort( const base::String& pSenderName ) const throw (ComException);	
	
	/**
	\brief add sender
	\param pSender sender
	\exception ComException Exception sender with identical name already registered
	*/
	void addSender(Sender* pSender) throw (ComException);

	/**
	\brief create direct sender
	\param pSenderName name of direct sender
	\param pReceiverName name of direct receiver
	\remarks a receiver that posssesses the supplied name must already have been stored previously
	\exception ComException failed to create direct sender
	*/
	void createSender(const base::String& pSenderName, const base::String& pReceiverName) throw (ComException);

	/**
	\brief create network sender
	\param pSenderName name of network sender
	\param pReceiverIP IP address of receiver
	\param pReceiverPort port of receiver
	\param pComProt communication protocol (UDP / TCP)
	\param pComForm communication format (Default / OSC)
	\exception ComException failed to create network sender
	*/
	void createSender(const base::String& pSenderName, const base::String& pReceiverIP, unsigned int pReceiverPort, Protocol pComProt, Format pComForm) throw (ComException);

	/**
	\brief remove sender
	\param pSenderName name of sender
	\exception failed to remove sender
	*/
	virtual void removeSender(const base::String& pSenderName) throw (ComException);
	
	/**
	 \brief remove all senders
	 */
	virtual void removeSenders();

	/**
	 \brief return receiver names
	 \return receiver names
	 */
	QList< base::String > receiverNames() const;	
	
	/**
	\brief check receiver
	\param pReceiverName name of receiver
	\return true if receiver is found, false otherwise
	*/	
	bool checkReceiver( const base::String& pReceiverName ) const;

	/**
	\brief retrieve receiver
	\param pReceiverName name of receiver
	\return receiver
	\exception ComException receiver not found
	*/
	Receiver* receiver( const base::String& pReceiverName ) throw (ComException);

	/**
	 \brief return receiver format
	 \param pSenderName name of receiver
	 \returns receiver format
	 \exception ComException receiver not found
	 */
	Format receiverFormat( const base::String& pReceiverName ) const throw (ComException);
	
	/**
	 \brief return receiver protocol
	 \param pSenderName name of receiver
	 \returns receiver protocol
	 \exception ComException receiver not found
	 */
	Protocol receiverProtocol( const base::String& pReceiverName ) const throw (ComException);
	
	/**
	 \brief return receiver port
	 \param pReceiverName name of receiver
	 \returns receiver port
	 \exception ComException receiver not found
	 */
	unsigned int receiverPort( const base::String& pReceiverName ) const throw (ComException);	

	
	/**
	\brief add receiver
	\param pReceiver receiver
	\exception ComException Exception receiver with identical name already registered
	*/
	void addReceiver(Receiver* pReceiver) throw (ComException);	

	/**
	\brief create direct receiver
	\param pReceiverName name of direct receiver
	\exception ComException failed to create direct receiver
	*/
	void createReceiver(const base::String& pReceiverName) throw (ComException);

	/**
	\brief create network receiver
	\param pReceiverName name of network receiver
	\param pListeningPort receiver listening port
	\param pComProt communication protocol (UDP / TCP)
	\param pComForm communication format (Default / OSC)
	\exception ComException failed to create network receiver
	*/
	void createReceiver(const base::String& pReceiverName, unsigned int pListeningPort, Protocol pComProt, Format pComForm) throw (ComException);

	/**
	\brief remove receiver
	\param pReceiverName name of receiver
	\exception failed to remove receiver
	*/
	virtual void removeReceiver(const base::String& pReceiverName) throw (ComException);
	
	/**
	 \brief remove all receivers
	 */
	virtual void removeReceivers();

	/**
	\brief register message listener
	\param pReceiverName name of receiver
	\param pMessageListener message listener
	\exception ComException receiver not found
	*/
	void registerMessageListener( const base::String& pReceiverName, MessageListener& pMessageListener ) throw (ComException);
	
	/**
	\brief send message
	\param pMessage message
	\remarks sends the same message via all senders
	\remarks failure in sending produces an error message in the programs verbose output
	*/
	void send( const Message& pMessage);

	/**
	\brief send message
	\param pSenderName name of sender
	\param pMessage message
	\exception ComException sender not found
	\remarks sends message via single specified sender
	\remarks failure in sending produces an error message in the programs verbose output
	*/
	void send( const base::String& pSenderName, const Message& pMessage) throw (ComException);
	
	/**
	\brief calls receive on all receivers
	*/
	void receive();
	
	/**
	\brief check if receiver has available message
	\param pReceiverName name of receiver
	\return true if message is available, false otherwise
	\exception ComException receiver not found
	*/
	bool messageAvailable( const base::String& pReceiverName ) const throw (ComException);

	/**
	\brief retrieve message from receiver
	\param pReceiverName receiver name
	\param pTimeOut maximum time in milisecs this function will block before returning ( < 0: no timeout, funcion blocks till message is complete)
	\return message, message will be empty if timeout is exceeded
	\exception ComException failed to retrieve message
	*/
	Message message( const base::String& pReceiverName, int pTimeOut ) throw (ComException);
	
	/**
	\brief remove available message
	\param pReceiverName name of receiver
	\exception ComException receiver not found
	*/
	void clearMessage( const base::String& pReceiverName ) const throw (ComException);

	/**
	\brief update messenger
	\remarks calls receive method on all receivers
	*/
	virtual void update();
	
protected:
	QHash<base::String, Sender*> mSenders; ///\brief message sender repository
	QHash<base::String, Receiver*> mReceivers;  ///\brief message receiver repository
	QMultiHash< Receiver*, MessageListener* > mMessageListeners; ///\brief message listeners
	
	void updateMessageListeners();
};

};
	
};

#endif