/**	\file iso_data_array.h
 *	\class iso::data::Array array class
 *	\brief array class
 *
 *  one dimensional resizable array class that implements boundary checks
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_data_array_h_
#define _iso_data_array_h_

#include <math.h>
#include <stdlib.h>
#include <iostream>

#include "iso_data_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace data
{

template <typename Type>
class Array
{
public:	
	/**
	\brief array elements
	*/
	Type* e;

	/**
	\brief default constructor
	*/
	Array();
	
	/**
	\brief parameter based constructor
	\param pCount number of array elements
	
	sets value of all array elements to 0.0
	*/
	Array(unsigned int pCount);
	
	/**
	\brief parameter based constructor
	\param pCount number of array elements
	\param pValue value of all array components
	
	sets value of all array elements to pValue
	*/
    Array( unsigned int pCount, Type pValue );

	/**
	\brief parameter based constructor
	\param pCount number of array elements
	\param pValues array of values of vector components
	*/
    Array( unsigned int pCount, Type* pValues );

	/**
	\brief parameter based constructor
	\param pCount number of array elements
	\param pValue1 first value of a series of values
	\param pValue2 second value of a series of values
	*/
    Array( unsigned int pCount, double pValue1, double pValue2, ... );
	
	/**
	\brief copy constructor
	\param pArray vector
	*/
	Array(const Array<Type>& pArray);

	/**
	\brief destructor
	*/
	virtual ~Array();

	/**
	\brief return element count
	\return element count
	*/
	inline unsigned int dataCount() const;
	
	/**
	\brief return array values
	\return array values
	*/
	Type* data(); 

	/**
	\brief return array values
	\return array values
	*/
	const Type* data() const; 
	
	/**
	\brief access element
	\param pIndex index of element
	\return component value
	\exception Exception index out of range
	*/
	inline Type& operator[] (unsigned int pIndex) throw (base::Exception);

	/**
	\brief access element
	\param pIndex index of element
	\return component value
	\exception Exception index out of range
	*/
	inline const Type& operator[] (unsigned int pIndex) const throw (base::Exception);
   
	/**
	\brief set all array elements to 0
	*/
	inline void clear();
	
	/**
	\brief change size of array
	\param pCount new array size
	
	original array values are preserved as far as possible.
	when array size increases, new array elements contain zero
	when array size decreases, previous values at positions that are beyond new array size are lost
	*/
	void resize(unsigned int pCount);
	
	/**
	\brief set all array elements to pValue
	\param pValue value
	*/
	inline void set( Type pValue );

	/**
	\brief resize array and set all array elements to pValue
	\param pCount new array size
	\param pValue value
	*/
	inline void set( Type pValue, unsigned int pCount );
	
	/**
	\brief set all array elements to the values in the pValues
	\param pValues values
	
	make sure that the correct number of values is provided
	*/
	inline void set( Type* pValues );
	
	/**
	\brief copy all array elements from pArray
	\param pArray array
	\param pResize resize array if size differs
	\exception Exception arrays not of same size
	*/
	inline void set( const Array<Type>& pArray, bool pResize = false ) throw (base::Exception);
	
	/**
	\brief set all array elements to value 
	\param pValue value
	\return array
	*/
	inline const Array<Type>& operator=(Type pValue);

	/**
	\brief copy all array elements from pArray
	\param pArray array
	\return array
	\exception Exception arrays not of same size
	*/
	inline const Array<Type>& operator=(const Array<Type>& pArray) throw (base::Exception);
	
	/**
	\brief array == array
	\param pArray comparison array
	\return true if array sizes and all array elements are identical, false otherwise
	*/     
	inline bool operator==( const Array<Type>& pArray ) const;
	
	/**
	\brief array != array
	\param pArray comparison array
	\return true if array sizes and all array elements are not identical, false otherwise
	*/    
	inline bool operator!=( const Array<Type>& pArray ) const;
			
	/**
	\brief print textual array information 
	*/
	base::String info() const;
		
	/**
	\brief print array information
	\param pOstream output stream
	\param pArray array
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const Array<Type>& pArray)
	{
		pOstream << pArray.info().toStdString(); 
		
		return pOstream;
	}

protected:
	/**
	\brief number of array elements
	*/
	unsigned int mCount;

};

typedef Array<int>    ArrayI;
typedef Array<float>  ArrayF;
typedef Array<double> ArrayD;
typedef Array<real> ArrayR;

template <typename Type>
Array<Type>::Array()
	: mCount(0)
	, e(NULL)
{}

template <typename Type>
Array<Type>::Array(unsigned int pCount) 
	: mCount(pCount)
{
	e = new Type[mCount];
	clear();
	
	for(unsigned int i=0; i<pCount; ++i) std::cout << e[i] << "\n";
}

template <typename Type>
Array<Type>::Array( unsigned int pCount, Type pValue ) 
	: mCount(pCount)
{
	e = new Type[mCount];
	set(pValue);
}

template <typename Type>
Array<Type>::Array( unsigned int pCount, Type* pValues ) 
	: mCount(pCount)
{ 
	e = new Type[mCount];
	set(pValues);	
}

template <typename Type>
Array<Type>::Array( unsigned int pCount, double pValue1, double pValue2, ... )
	: mCount(pCount)
{
	e = new Type[mCount];

	e[0] = static_cast<Type>( pValue1 );
	e[1] = static_cast<Type>( pValue2 );

	va_list ap;
	va_start(ap, pValue2);
	
    for(unsigned int i=2; i < mCount; i++)
	{
		e[i] = static_cast<Type>( va_arg(ap, double) );
	}
	va_end(ap);		
}

template <typename Type>
Array<Type>::Array(const Array<Type>& pArray) 
	: mCount(pArray.mCount)
{ 
	e = new Type[mCount];	
	set(pArray);
}

template <typename Type>
Array<Type>::~Array()
{
	delete [] e;
}

template <typename Type>
unsigned int 
Array<Type>::dataCount() const
{
	return mCount;
}

template <typename Type>
Type* 
Array<Type>::data()
{
	return e;
}

template <typename Type>
const Type* 
Array<Type>::data() const
{
	return e;
}

template <typename Type>
Type& 
Array<Type>::operator[] (unsigned int pIndex) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mCount) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("array elementCount %1 index %2").arg(mCount).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	return e[pIndex];
}

template <typename Type>
const Type& 
Array<Type>::operator[] (unsigned int pIndex) const throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mCount) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("array elementCount %1 index %2").arg(mCount).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	return e[pIndex];
}

template <typename Type>
void 
Array<Type>::clear() 
{
	//memset(e, 0, mCount * sizeof(Type));
}

template <typename Type>
void 
Array<Type>::resize(unsigned int pCount)
{
	if(mCount == pCount) return;
	
	Type* newE = new Type[pCount];
	unsigned minCount = mCount > pCount ? pCount : mCount;
	unsigned int i=0;
	
	for(; i<minCount; ++i)
	{
		newE[i] = e[i];
	}
	for(; i<pCount; ++i)
	{
		newE[i] = 0.0;
	}
	
	delete [] e;
	e = newE;
	mCount = pCount;
}

template <typename Type>
void 
Array<Type>::set( Type pValue ) 
{ 
	for( unsigned int i=0; i<mCount; ++i ) e[i] = pValue;
}

template <typename Type>
void 
Array<Type>::set( Type pValue, unsigned int pCount ) 
{ 
	if(mCount != pCount) resize(pCount);
	mCount = pCount;

	for( unsigned int i=0; i<mCount; ++i ) e[i] = pValue;
}

template <typename Type>
void 
Array<Type>::set( Type* pValues )
{
	for(int i=0; i<mCount; ++i) e[i] = pValues[i];

	//memcpy(e, pValues, mCount * sizeof(Type));
}

template <typename Type>
void 
Array<Type>::set( const Array<Type>& pArray, bool pResize ) throw (base::Exception)
{ 
	#ifdef _iso_debug_
	if(mCount != pArray.mCount && pResize == false) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("array1 elementCount %1 array2 elementCount %2").arg(mCount).arg(pArray.mCount), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	if(mCount != pArray.mCount) resize(pArray.mCount);
	
	for(int i=0; i<mCount; ++i) e[i] = pArray.e[i];
	
	//memcpy(e, pArray.e, mCount * sizeof(Type));
}

template <typename Type>
const Array<Type>& 
Array<Type>::operator=(Type pValue) 
{ 
	set(pValue);
	
	return (*this);
}

template <typename Type>
const Array<Type>& 
Array<Type>::operator=(const Array<Type>& pArray) throw (base::Exception)
{
	if( mCount != pArray.dataCount() ) resize( pArray.dataCount() );

	set(pArray);
	
	return (*this);
}

template <typename Type>
bool 
Array<Type>::operator==( const Array<Type>& pArray ) const
{ 
	if(mCount != pArray.mCount) return false;		
	
	for(unsigned int i=0; i<mCount; ++i)
	{
		if(e[i] != pArray.e[i]) return false;
	}
	
	return true;
}

template <typename Type>
bool 
Array<Type>::operator!=( const Array<Type>& pArray ) const 
{ 
	if(mCount != pArray.mCount) return true;
	
	for(unsigned int i=0; i<mCount; ++i)
	{
		if (e[i] == pArray.e[i]) return false;
	}
	
	return true; 
}

template <typename Type>
base::String 
Array<Type>::info() const
{
	base::String infoString = base::String("count %1\n")
	.arg(mCount);
	
	infoString.append("    e: ");
	for(unsigned int i=0; i<mCount; ++i) infoString.append(base::String("%1 ").arg(e[i]));
	infoString.append("\n");
	
	return infoString;
}

};

};

#endif

