/** \brief iso_data_function_manager.h
 *
 *  Created by Daniel Bisig on 3/12/09.
 */

#ifndef _iso_data_function_manager_h_
#define _iso_data_function_manager_h_

#include "iso_data_value_manager.h"
#include "iso_data_value_function.h"

namespace iso
{

namespace data
{

#pragma mark FunctionManager (definition)

class FunctionManager
{
public:
	static FunctionManager& get();
	static void destroy();

	template <class FunctionType, class ValueType> void registerFunction() throw(base::Exception);	
	template<class FunctionType>void applyFunction( Values& pValues1 ) throw(base::Exception);	
	template<class FunctionType>void applyFunction( Values& pValues1, const Values& pValues2 ) throw(base::Exception);	
	template<class FunctionType>void applyFunction( Values& pValues1, const Values& pValues2, Values& pValues3 ) throw(base::Exception);	
	template<class FunctionType>void applyFunction( Values& pValues1, const VoidValue& pValue2 ) throw(base::Exception);	
	template<class FunctionType>void applyFunction( Values& pValues1, const VoidValue& pValue2, Values& pValues3 ) throw(base::Exception);	
	
protected:
	FunctionManager();
	~FunctionManager();

	static FunctionManager* sInstance;

	QHash< base::String, ValueFunction* > mFunctionMap;
};

#pragma mark FunctionManager (implementation)

template <class FunctionType, class ValueType>
void
FunctionManager::registerFunction() throw(base::Exception)
{
	const base::String& functionName = FunctionType::sName;
	ValueType valueType;
	
	if( mFunctionMap.contains(functionName) == false ) mFunctionMap[functionName] = new FunctionType();
	
	FunctionType* function = dynamic_cast<FunctionType*>( mFunctionMap[functionName] );
	
	if(function == NULL) throw( base::Exception( "ERR_FUNCTION_MISMATCH", base::String("function %1 doesn't correspond to correct function type").arg(functionName), __FILE__, __FUNCTION__, __LINE__ ) );
	
	if(function->checkValueType( valueType ) == false)
	{
		function->registerValueType( valueType );
	}
}

template<class FunctionType>
void 
FunctionManager::applyFunction( Values& pValues1 ) throw(base::Exception)
{
	const base::String& functionName = FunctionType::sName;
	
	if( mFunctionMap.contains(functionName) == false ) throw( base::Exception( "ERR_FUNCTION_UNKNOWN", base::String("function %1 has not been registered").arg(functionName), __FILE__, __FUNCTION__, __LINE__ ) );
	
	ValueFunction* function = mFunctionMap[functionName];
	
	unsigned int valueCount = pValues1.valueCount();
	
	VoidValue** values1 = pValues1.values();
	VoidValue* value1;
	
	for(unsigned int valueNr=0; valueNr < valueCount; ++valueNr)
	{
		value1 = values1[valueNr];
		
		Values* subValues1 = dynamic_cast<Values*>(value1);
		
		if(subValues1 != NULL)
		{
			applyFunction<FunctionType>( *subValues1 );
		}
		else if(subValues1 == NULL )
		{
			function->apply( *value1 );
		}
		else
		{
			throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied data types don't match"), __FILE__, __FUNCTION__, __LINE__ ) );
		}
	}
}


template<class FunctionType>
void 
FunctionManager::applyFunction( Values& pValues1, const Values& pValues2 ) throw(base::Exception)
{
	const base::String& functionName = FunctionType::sName;
	
	if( mFunctionMap.contains(functionName) == false ) throw( base::Exception( "ERR_FUNCTION_UNKNOWN", base::String("function %1 has not been registered").arg(functionName), __FILE__, __FUNCTION__, __LINE__ ) );
	
	ValueFunction* function = mFunctionMap[functionName];
	
	unsigned int valueCount = pValues1.valueCount();
	
	if( pValues2.valueCount() != valueCount ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("number of supplied values doesn't match: parameter 1 : %1 != parameter 2: %2").arg(valueCount).arg(pValues2.valueCount()), __FILE__, __FUNCTION__, __LINE__ ) );
	
	VoidValue** values1 = pValues1.values();
	VoidValue* value1;

	const VoidValue** values2 = pValues2.values();
	const VoidValue* value2;
	
	for(unsigned int valueNr=0; valueNr < valueCount; ++valueNr)
	{
		value1 = values1[valueNr];
		value2 = values2[valueNr];
		
		Values* subValues1 = dynamic_cast<Values*>(value1);
		const Values* subValues2 = dynamic_cast<const Values*>(value2);
		
		if(subValues1 != NULL && subValues2 != NULL)
		{
			applyFunction<FunctionType>( *subValues1, *subValues2 );
		}
		else if(subValues1 == NULL && subValues2 == NULL )
		{
			function->apply( *value1, *value2 );
		}
		else
		{
			throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied data types don't match"), __FILE__, __FUNCTION__, __LINE__ ) );
		}
	}
}

template<class FunctionType>
void 
FunctionManager::applyFunction( Values& pValues1, const VoidValue& pValue2 ) throw(base::Exception)
{
	const base::String& functionName = FunctionType::sName;
	
	if( mFunctionMap.contains(functionName) == false ) throw( base::Exception( "ERR_FUNCTION_UNKNOWN", base::String("function %1 has not been registered").arg(functionName), __FILE__, __FUNCTION__, __LINE__ ) );
	
	ValueFunction* function = mFunctionMap[functionName];
	
	unsigned int valueCount = pValues1.valueCount();
	
	VoidValue** values1 = pValues1.values();
	VoidValue* value1;
	
	for(unsigned int valueNr=0; valueNr < valueCount; ++valueNr)
	{
		value1 = values1[valueNr];
		
		Values* subValues1 = dynamic_cast<Values*>(value1);
		
		if(subValues1 != NULL )
		{
			applyFunction<FunctionType>( *subValues1, pValue2 );
		}
		else
		{
			function->apply( *value1, pValue2 );
		}
	}
}



template<class FunctionType>
void 
FunctionManager::applyFunction( Values& pValues1, const Values& pValues2, Values& pValues3 ) throw(base::Exception)
{
	const base::String& functionName = FunctionType::sName;
	
	if( mFunctionMap.contains(functionName) == false ) throw( base::Exception( "ERR_FUNCTION_UNKNOWN", base::String("function %1 has not been registered").arg(functionName), __FILE__, __FUNCTION__, __LINE__ ) );
	
	ValueFunction* function = mFunctionMap[functionName];

	unsigned int valueCount = pValues1.valueCount();

	if( pValues2.valueCount() != valueCount ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("number of supplied values doesn't match: parameter 1 : %1 != parameter 2: %2").arg(valueCount).arg(pValues2.valueCount()), __FILE__, __FUNCTION__, __LINE__ ) );
	if( pValues3.valueCount() != valueCount ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("number of supplied values doesn't match: parameter 1 : %1 != parameter 3: %2").arg(valueCount).arg(pValues3.valueCount()), __FILE__, __FUNCTION__, __LINE__ ) );
	
	VoidValue** values1 = pValues1.values();
	VoidValue* value1;

	const VoidValue** values2 = pValues2.values();
	const VoidValue* value2;

	VoidValue** values3 = pValues3.values();
	VoidValue* value3;
	
	for(unsigned int valueNr=0; valueNr < valueCount; ++valueNr)
	{
		value1 = values1[valueNr];
		value2 = values2[valueNr];
		value3 = values3[valueNr];
		
		Values* subValues1 = dynamic_cast<Values*>(value1);
		const Values* subValues2 = dynamic_cast<const Values*>(value2);
		Values* subValues3 = dynamic_cast<Values*>(value3);
		
		if(subValues1 != NULL && subValues2 != NULL && subValues3 != NULL)
		{
			applyFunction<FunctionType>( *subValues1, *subValues2, *subValues3 );
		}
		else if(subValues1 == NULL && subValues2 == NULL && subValues3 == NULL)
		{
			function->apply( *value1, *value2, *value3 );
		}
		else
		{
			throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied data types don't match"), __FILE__, __FUNCTION__, __LINE__ ) );
		}
	}
}

template<class FunctionType>
void 
FunctionManager::applyFunction( Values& pValues1, const VoidValue& pValue2, Values& pValues3 ) throw(base::Exception)
{
	const base::String& functionName = FunctionType::sName;
	
	if( mFunctionMap.contains(functionName) == false ) throw( base::Exception( "ERR_FUNCTION_UNKNOWN", base::String("function %1 has not been registered").arg(functionName), __FILE__, __FUNCTION__, __LINE__ ) );
	
	//FunctionType* function = dynamic_cast<FunctionType*>( mFunctionMap[functionName] );
	//if(function == NULL) throw( base::Exception( "ERR_FUNCTION_MISMATCH", base::String("function %1 doesn't correspond to correct function type").arg(functionName), __FILE__, __FUNCTION__, __LINE__ ) );

	ValueFunction* function = mFunctionMap[functionName];

	unsigned int valueCount = pValues1.valueCount();
	
	if( pValues3.valueCount() != valueCount ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("valuecount mismatch"), __FILE__, __FUNCTION__, __LINE__ ) );
	
	VoidValue** values1 = pValues1.values();
	VoidValue* value1;

	VoidValue** values3 = pValues3.values();
	VoidValue* value3;
	
	for(unsigned int valueNr=0; valueNr < valueCount; ++valueNr)
	{
		value1 = values1[valueNr];
		value3 = values3[valueNr];
		
		Values* subValues1 = dynamic_cast<Values*>(value1);
		Values* subValues3 = dynamic_cast<Values*>(value3);
		
		if(subValues1 != NULL && subValues3 != NULL)
		{
			applyFunction<FunctionType>( *subValues1, pValue2, *subValues3 );
		}
		else if(subValues1 == NULL && subValues3 == NULL)
		{
			function->apply( *value1, pValue2, *value3 );
		}
		else
		{
			throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied data types don't match"), __FILE__, __FUNCTION__, __LINE__ ) );
		}
	}
}

};

};

#endif