/** \file iso_data_indexhash.h
 *	\class iso::data::IndexHash a dual data structure that allows to retrieve data both by key and index
 *	\brief a dual data structure that allows to retrieve data both by key and index
 *
 *  data structure stores pointers only. It manages data both via a hash table and vector data structure. \n
 *  the data structure is relatively slow in removing / replacing elements since it has to find the element in both the hash table and vector\n
 *
 *  Created by Daniel Bisig on 4/14/07.
 */

#ifndef _iso_data_indexhash_h_
#define _iso_data_indexhash_h_

#include <QVector>
#include <QHash>

#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace data
{

template <typename KeyType, typename ValueType>
class IndexHash
{
public:
	/**
	\brief default constructor
	*/
	IndexHash();
	
	/**
	\brief destructor
	*/
	~IndexHash();
	
	/**
	\brief return vector of data
	\return vector of data
	*/
	QVector<ValueType*>& vector();
	
	/**
	\brief return hash table of data
	\return hash table of data
	*/
	QHash<KeyType, ValueType*>& hash();
	
	/**
	\brief returns number of elements stored
	\return number of elements stored
	*/
	unsigned int size() const;
	
	/**
	\brief check whether hash table contains key
	\param pKey key to look for
	\return true if key is found, false otherwise
	*/
	bool contains(KeyType pKey) const;
	
	/**
	\brief return index for key
	\param pKey key to look for
	\return index
	\exception base::Exception key not found
	*/
	unsigned int index(KeyType pKey) const throw (base::Exception);

	/**
	\brief return key
	\param pIndex index of element
	\return key
	\exception base::Exception index out of range
	*/
	inline const KeyType& key(unsigned int pIndex) const throw (base::Exception);

	/**
	\brief return value
	\param pIndex index of element
	\return value
	\exception base::Exception index out of range
	*/
	inline const ValueType& value(unsigned int pIndex) const throw (base::Exception);

	/**
	\brief return value
	\param pIndex index of element
	\return value
	\exception base::Exception index out of range
	*/
	inline ValueType& value(unsigned int pIndex) throw (base::Exception);

	/**
	\brief access element
	\param pIndex index of element
	\return element
	\exception base::Exception index out of range
	*/
	inline ValueType& operator[] (unsigned int pIndex) throw (base::Exception);

	/**
	\brief access element
	\param pIndex index of element
	\return element
	\exception base::Exception index out of range
	*/
	inline const ValueType& operator[] (unsigned int pIndex) const throw (base::Exception);	

	/**
	\brief access element
	\param pKey key of element
	\return element
	\exception base::Exception key not found
	*/
	inline ValueType& operator[] (const KeyType& pKey) throw (base::Exception);

	/**
	\brief access element
	\param pKey key of element
	\return element
	\exception base::Exception key not found
	*/
	inline const ValueType& operator[] (const KeyType& pKey) const throw (base::Exception);	

	/**
	\brief add element
	\param pKey storage key
	\param pValue element value
	\exception base::Exception key alreay stored
	*/
	void add(const KeyType& pKey, ValueType& pValue) throw (base::Exception);

	/**
	\brief insert element
	\param pKey storage key
	\param pValue element value
	\exception base::Exception key alreay stored
	
	same as add
	*/
	void insert(const KeyType& pKey, ValueType& pValue) throw (base::Exception);

	/**
	 \brief insert element
	 \param pKey storage key
	 \param pValue element value
	 \param pIndex insert position index
	 \exception base::Exception key alreay stored
	 */
	void insert(const KeyType& pKey, ValueType& pValue, unsigned int pIndex ) throw (base::Exception);	

	/**
	 \brief move element
	 \param pFromIndex from index position
	 \param pToIndex to indey position
	 \exception base::Exception index out of bounds
	 */
	void move( unsigned int pFromIndex, unsigned int pToIndex ) throw (base::Exception);		
	
	/**
	\brief replace element
	\param pKey storage key
	\param pValue element value
	\exception base::Exception key not found
	*/
	void replace(const KeyType& pKey, ValueType& pValue) throw (base::Exception);

	/**
	\brief replace element
	\param pIndex index of element
	\param pValue element value
	\exception base::Exception index out of range
	*/
	void replace(unsigned int pIndex, ValueType& pValue) throw (base::Exception);

	/**
	\brief set element
	\param pKey storage key
	\param pValue element value
	
	if key is found: element is replaced
	if key is not found: element is added
	*/
	void set(const KeyType& pKey, ValueType& pValue);

	/**
	\brief set element
	\param pIndex index of element
	\param pValue element value
	\exception base::Exception index out of range
	*/
	void set(unsigned int pIndex, ValueType& pValue) throw (base::Exception);

	/**
	\brief remove element
	\param pKey storage key
	\exception base::Exception key not found
	*/
	void remove(const KeyType& pKey) throw (base::Exception);

	/**
	\brief remove element
	\param pIndex index of element
	\exception base::Exception index out of range
	*/
	void remove(unsigned int pIndex) throw (base::Exception);
	
	/**
	\brief remove all elements
	*/
	void clear();
	
protected:
	/**
	\brief vector of data
	*/
	QVector<ValueType*> mValueVector;

	/**
	\brief vector of keys
	*/
	QVector<KeyType> mKeyVector;
	
	/**
	\brief hash table of data
	*/
	QHash<KeyType, ValueType*> mHash;

};

template <typename KeyType, typename ValueType>
IndexHash<KeyType, ValueType>::IndexHash()
{}

template <typename KeyType, typename ValueType>
IndexHash<KeyType, ValueType>::~IndexHash()
{
	mValueVector.clear();
	mKeyVector.clear();
	mHash.clear();
}

template <typename KeyType, typename ValueType>
QVector<ValueType*>&
IndexHash<KeyType, ValueType>::vector()
{
	return mValueVector;
}

template <typename KeyType, typename ValueType>
QHash<KeyType, ValueType*>&
IndexHash<KeyType, ValueType>::hash()
{
	return mHash;
}

template <typename KeyType, typename ValueType>
unsigned int
IndexHash<KeyType, ValueType>::size() const
{
	return mValueVector.size();
}

template <typename KeyType, typename ValueType>
bool 
IndexHash<KeyType, ValueType>::contains(KeyType pKey) const
{
	return mHash.contains(pKey);
}

template <typename KeyType, typename ValueType>
unsigned int 
IndexHash<KeyType, ValueType>::index(KeyType pKey) const throw (base::Exception)
{
	int index = mKeyVector.indexOf(pKey);
	
	if(index < 0) throw(base::Exception( "ERR_NAME_UNKNOWN", base::String("hash key not found %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	return static_cast<unsigned int>( index );
}

template <typename KeyType, typename ValueType>
const KeyType& 
IndexHash<KeyType, ValueType>::key(unsigned int pIndex) const throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mKeyVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mKeyVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	return mKeyVector[pIndex];
}

template <typename KeyType, typename ValueType>
const ValueType& 
IndexHash<KeyType, ValueType>::value(unsigned int pIndex) const throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mValueVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mValueVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	return *(mValueVector[pIndex]);
}

template <typename KeyType, typename ValueType>
ValueType& 
IndexHash<KeyType, ValueType>::value(unsigned int pIndex) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mValueVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mValueVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	return *(mValueVector[pIndex]);
}

template <typename KeyType, typename ValueType>
ValueType& 
IndexHash<KeyType, ValueType>::operator[] (unsigned int pIndex) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mValueVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mValueVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	return *(mValueVector[pIndex]);
}

template <typename KeyType, typename ValueType>
const ValueType& 
IndexHash<KeyType, ValueType>::operator[] (unsigned int pIndex) const throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mValueVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mValueVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	return *(mValueVector[pIndex]);
}

template <typename KeyType, typename ValueType>
ValueType& 
IndexHash<KeyType, ValueType>::operator[] (const KeyType& pKey) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mHash.contains(pKey) == false) throw(base::Exception( "ERR_NAME_UNKNOWN", base::String("hash key not found %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	return *(mHash[pKey]);
}

template <typename KeyType, typename ValueType>
const ValueType& 
IndexHash<KeyType, ValueType>::operator[] (const KeyType& pKey) const throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mHash.contains(pKey) == false)  throw(base::Exception( "ERR_NAME_UNKNOWN", base::String("hash key not found %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	return *(mHash[pKey]);
}

template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::add(const KeyType& pKey, ValueType& pValue) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mHash.contains(pKey) == true)  throw(base::Exception( "ERR_NAME_CONFLICT", base::String("hash key already stored %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	mValueVector.append(&pValue);
	mKeyVector.append(pKey);
	mHash.insert(pKey, &pValue);
}

template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::insert(const KeyType& pKey, ValueType& pValue) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mHash.contains(pKey) == true)  throw(base::Exception( "ERR_NAME_CONFLICT", base::String("hash key already stored %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	mValueVector.append(&pValue);
	mKeyVector.append(pKey);
	mHash.insert(pKey, &pValue);
}
	
template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::insert(const KeyType& pKey, ValueType& pValue, unsigned int pIndex ) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mHash.contains(pKey) == true)  throw(base::Exception( "ERR_NAME_CONFLICT", base::String("hash key already stored %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	if(mValueVector.size() < pIndex ) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("index %1 exceeds size of indexhash %2").arg(pIndex).arg( mValueVector.size() ), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
		
	mValueVector.insert(pIndex, &pValue);
	mKeyVector.insert(pIndex, pKey);
	mHash.insert(pKey, &pValue);
}

template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::move(unsigned int pFromIndex, unsigned int pToIndex) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mValueVector.size() < pFromIndex ) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("fromIndex %1 exceeds size of indexhash %2").arg(pFromIndex).arg( mValueVector.size() ), __FILE__, __FUNCTION__, __LINE__ ));
	if(mValueVector.size() < pToIndex ) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("toIndex %1 exceeds size of indexhash %2").arg(pToIndex).arg( mValueVector.size() ), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	ValueType* _value = mValueVector[pFromIndex];
	KeyType _key = mKeyVector[pFromIndex];
	
	mValueVector.remove(pFromIndex);
	mKeyVector.remove(pFromIndex);
	
	mValueVector.insert(pToIndex, _value);
	mKeyVector.insert(pToIndex, _key);
}
	
template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::replace(const KeyType& pKey, ValueType& pValue) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mHash.contains(pKey) == false)  throw(base::Exception( "ERR_NAME_UNKNOWN", base::String("hash key not found %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	ValueType* origValue = mHash[pKey];
	
	mHash[pKey] = &pValue;
	
	unsigned int elementCount = mValueVector.size();
	for(unsigned int i=0; i<elementCount; ++i) 
	{
		if(mValueVector[i] == origValue)
		{
			mValueVector[i] = &pValue;
			mKeyVector[i] = pKey;
			return;
		}
	}
}

template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::replace(unsigned int pIndex, ValueType& pValue) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mValueVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mValueVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	ValueType* origValue = mValueVector[pIndex];
	
	mValueVector[pIndex] = &pValue;
		
	QHashIterator<KeyType, ValueType*> hashIterator(mHash);

	while (hashIterator.hasNext()) 
	{
		hashIterator.next();
		
		if(hashIterator.value() == origValue) 
		{
			mHash[hashIterator.key()] =  &pValue;
			return;
		}
	}
}


template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::set(const KeyType& pKey, ValueType& pValue)
{
	if(mHash.contains(pKey)) replace(pKey, pValue);
	else add(pKey, pValue);
}

template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::set(unsigned int pIndex, ValueType& pValue) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mValueVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mValueVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	replace(pIndex, pValue);
}

template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::remove(const KeyType& pKey) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(mHash.contains(pKey) == false) throw(base::Exception( "ERR_NAME_UNKNOWN", base::String("hash key not found %1").arg(pKey), __FILE__, __FUNCTION__, __LINE__ ));
	#endif

	ValueType* origValue = mHash[pKey];

	mHash.remove(pKey);
	
	unsigned int elementCount = mValueVector.size();
	for(unsigned int i=0; i<elementCount; ++i)
	{
		if(mValueVector[i] == origValue)
		{
			mValueVector.remove(i);
			mKeyVector.remove(i);
			return;
		}
	}
}

template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::remove(unsigned int pIndex) throw (base::Exception)
{
	#ifdef _iso_debug_
	if(pIndex >= mValueVector.size()) throw(base::Exception( "ERR_OUT_OF_BOUNDS", base::String("vector size %1 index %2").arg(mValueVector.size()).arg(pIndex), __FILE__, __FUNCTION__, __LINE__ ));
	#endif
	
	ValueType* origValue = mValueVector[pIndex];
	
	mValueVector.remove(pIndex);
	mKeyVector.remove(pIndex);

	QHashIterator<KeyType, ValueType*> hashIterator(mHash);

	while (hashIterator.hasNext()) 
	{
		hashIterator.next();
		
		if(hashIterator.value() == origValue) 
		{
			mHash.remove(hashIterator.key());
			return;
		}
	}	
}
	
template <typename KeyType, typename ValueType>
void 
IndexHash<KeyType, ValueType>::clear()
{
	mValueVector.clear();
	mKeyVector.clear();
	mHash.clear();
}

};

};

#endif
