/**	\file iso_data_pool.h
 *	\class iso::data::Pool class
 *	\brief pool class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_data_pool_h_
#define _iso_data_pool_h_

#include <iso_base/iso_base_exception.h>
#include <QVector>

namespace iso
{

namespace data
{

template <typename Type>
class Pool
{
public:
	/**
	\brief default constructor
	
	pool elements will be created via the elements default constructor
	*/
	Pool();
	
	/**
	\brief parameter based constructor
	\param pStartPoolSize number of events that are initially in the pool
	\param pPoolSizeIncrement size by which the pool increases whenever it runs out of events
	
	pool elements will be created via the elements default constructor
	*/
	Pool(unsigned int pStartPoolSize, unsigned int pPoolSizeIncrement = sPoolSizeIncrement);

	/**
	\brief proto element based constructor
	\param pProtoElement proto element
	
	create pool elements based on proto element (copy constructor)
	*/
	Pool(const Type& pProtoElement);
	
	/**
	\brief proto element based constructor
	\param pProtoElement proto element
	\param pStartPoolSize number of events that are initially in the pool
	\param pPoolSizeIncrement size by which the pool increases whenever it runs out of events

	create pool elements based on proto element (copy constructor)
	*/
	Pool(const Type& pProtoElement, unsigned int pStartPoolSize, unsigned int pPoolSizeIncrement = sPoolSizeIncrement);
	
	/**
	\brief destructor
	*/
	virtual ~Pool();
	
	/**
	\brief retrieve object from pool
	*/
	inline Type* retrieve();
	
	/**
	\brief release object into pool
	\param pObject object to be released
	*/
	inline void release(Type* pObject);
	
	/**
	\brief obtain textual event pool information
	\return base::String containing event pool information
	*/
	base::String info() const;

	/**
	\brief print pool information
	\param pOstream output stream
	\param pPool pool
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const Pool& pPool)
	{
		pOstream << pPool.info().toStdString(); 
		
		return pOstream;
	}
	
protected:
	/**
	\brief default initial pool size
	*/
	static unsigned int sStartPoolSize;

	/**
	\brief default pool size increment
	*/
	static unsigned int sPoolSizeIncrement;
	
	/**
	\brief use proto element to create pool elements
	*/
	Type* mProtoElement;

	/**
	\brief vector of unused events
	*/
	QVector<Type*> mObjects;
	
	/**
	\brief size by which the pool increases whenever it runs out of events
	*/
	unsigned int mPoolSizeIncrement;
};	

template <typename Type> unsigned int Pool<Type>::sStartPoolSize = 1000;
template <typename Type> unsigned int Pool<Type>::sPoolSizeIncrement = 100;

template <typename Type>
Pool<Type>::Pool()
	: mPoolSizeIncrement(sPoolSizeIncrement)
	, mProtoElement(NULL)
{
	for(unsigned int i=0; i<sStartPoolSize; ++i)
	{
		Type* object = new Type();
		mObjects.push_back(object);
	}	
}

template <typename Type>
Pool<Type>::Pool(unsigned int pStartPoolSize, unsigned int pPoolSizeIncrement)
	: mPoolSizeIncrement(pPoolSizeIncrement)
	, mProtoElement(NULL)
{
	for(unsigned int i=0; i<pStartPoolSize; ++i)
	{
		Type* object = new Type();
		mObjects.push_back(object);
	}
}

template <typename Type>
Pool<Type>::Pool(const Type& pProtoElement)
	: mPoolSizeIncrement(sPoolSizeIncrement)
{
	mProtoElement = new Type(pProtoElement);

	for(unsigned int i=0; i<sStartPoolSize; ++i)
	{
		Type* object = new Type(*mProtoElement);
		mObjects.push_back(object);
	}	
}

template <typename Type>
Pool<Type>::Pool(const Type& pProtoElement, unsigned int pStartPoolSize, unsigned int pPoolSizeIncrement)
	: mPoolSizeIncrement(pPoolSizeIncrement)
{
	mProtoElement = new Type(pProtoElement);

	for(unsigned int i=0; i<pStartPoolSize; ++i)
	{
		Type* object = new Type(*mProtoElement);
		mObjects.push_back(object);
	}
}

template <typename Type>	
Pool<Type>::~Pool()
{
	unsigned int poolSize = mObjects.size();
	
	for(unsigned int i=0; i<poolSize; ++i)
	{
		Type* object = mObjects[i];
		delete object;
	}
	
	mObjects.clear();
	
	if(mProtoElement != NULL) delete mProtoElement;
}

template <typename Type>	
Type* 
Pool<Type>::retrieve()
{
	Type* object;

	if(mObjects.size() == 0) // create more objects
	{
		if(mProtoElement == NULL) for(unsigned int i=0; i<mPoolSizeIncrement; ++i) mObjects.push_back(new Type());
		else for(unsigned int i=0; i<mPoolSizeIncrement; ++i) mObjects.push_back(new Type(*mProtoElement));
	}
	
	object = mObjects[0];
	mObjects.pop_front();
	return object;
}

template <typename Type>
void 
Pool<Type>::release(Type* pObject)
{
	mObjects.push_back(pObject);
}

template <typename Type>
base::String 
Pool<Type>::info() const
{
	base::String infoString = base::String("Pool:\n    poolSize %1\n    poolSizeIncrement: %2\n")
		.arg(mObjects.size())
		.arg(mPoolSizeIncrement);
	
	return infoString;
}

};

};

#endif

