/** \file iso_data_rc_pointer.h
 *  \class iso::data::RCPtr refernce counted pointer
 *  \brief reference counted pointer 
 *
 *
 *  Created by Daniel Bisig on 8/3/10.
 *
 */

#ifndef _iso_data_rc_pointer_h_
#define _iso_data_rc_pointer_h_

#include <QHash>

namespace iso
{
	
	namespace data
	{
#pragma mark RCPtr Definition
		
		template <typename DataType>
		class RCPtr
		{
		public:
			RCPtr();
			RCPtr( DataType* pPtr );
			RCPtr( const RCPtr<DataType>& pRCPtr );
			~RCPtr( );
			
			RCPtr<DataType>& operator=( const RCPtr<DataType>& pRCPtr );
			
			const DataType& operator*( ) const;
			const DataType* operator&( ) const;
			const DataType* operator->( ) const;
			DataType& operator*( );
			DataType* operator&( );
			DataType* operator->( );
			operator void*() const;
			
			// debug
			unsigned int refCount() const;
			// debug end
			
		protected:
			DataType* mPtr;
			static QHash<DataType*, int> sRefCountMap;
		};

#pragma mark RCPtr Implementation
				
		template <typename DataType>
		QHash<DataType*, int> 
		RCPtr<DataType>::sRefCountMap;
		
		template <typename DataType>	
		RCPtr<DataType>::RCPtr()
		: mPtr( NULL )
		{}
		
		template <typename DataType>	
		RCPtr<DataType>::RCPtr( DataType* pPtr )
		: mPtr( pPtr )
		{
			if( mPtr == NULL ) return;
			
			if( sRefCountMap.contains(mPtr) ) sRefCountMap[mPtr]++;
			else sRefCountMap[mPtr] = 1;
		}

		template <typename DataType>	
		RCPtr<DataType>::RCPtr( const RCPtr<DataType>& pRCPtr )
		: mPtr( pRCPtr.mPtr )
		{
			if( mPtr == NULL ) return;
			
			if( sRefCountMap.contains(mPtr) ) sRefCountMap[mPtr]++;
			else sRefCountMap[mPtr] = 1;
		}
		
		template <typename DataType>	
		RCPtr<DataType>::~RCPtr( )
		{
			if( mPtr == NULL ) return;
			
			//std::cout << "Ptr: " << mPtr << "\n";
			
			//std::cout << "~RCPtr oldRefCount " << sRefCountMap[mPtr] << " ";
			
			sRefCountMap[mPtr]--;
			
			//std::cout << "newRefCount " << sRefCountMap[mPtr] << "\n";
			
			if (sRefCountMap[mPtr] == 0) 
			{
				//std::cout << "smart ptr delete\n";
				
				sRefCountMap.remove(mPtr);
				
				//std::cout << "ptr " << mPtr << "\n";
				
				delete mPtr;
				
				//std::cout << "done\n";
			}
		}

		template <typename DataType>	
		const DataType& 
		RCPtr<DataType>::operator*( ) const
		{
			return *mPtr;
		}
		
		template <typename DataType>	
		const DataType* 
		RCPtr<DataType>::operator&( ) const
		{
			return mPtr;
		}
		
		template <typename DataType>
		const DataType* 
		RCPtr<DataType>::operator->( ) const
		{
			return mPtr;
		}
		
		template <typename DataType>	
		DataType& 
		RCPtr<DataType>::operator*( )
		{
			return *mPtr;
		}
		
		template <typename DataType>	
		DataType* 
		RCPtr<DataType>::operator&( )
		{
			return mPtr;
		}
		
		template <typename DataType>
		DataType* 
		RCPtr<DataType>::operator->( )
		{
			return mPtr;
		}
		
		template <typename DataType>
		RCPtr<DataType>::operator void*() const
		{ 
			return mPtr; 
		}
		
		template <typename DataType>
		RCPtr<DataType>& 
		RCPtr<DataType>::operator=( const RCPtr<DataType>& pRCPtr )
		{
			if( mPtr != NULL )
			{
				sRefCountMap[mPtr]--;
				
				if (sRefCountMap[mPtr] == 0) 
				{
					sRefCountMap.remove(mPtr);
					
					delete mPtr;
				}
			}
			
			mPtr = pRCPtr.mPtr;
			
			if( mPtr != NULL )
			{
				if( sRefCountMap.contains(mPtr) ) sRefCountMap[mPtr]++;
				else sRefCountMap[mPtr] = 1;
			}
			
			return *this;
		}
		
		template <typename DataType>
		unsigned int 
		RCPtr<DataType>::refCount() const
		{
			if( sRefCountMap.contains(mPtr) ) return sRefCountMap[mPtr];
			return 0;
		}
	};
	
};

#endif

