/**	\file iso_event_manager.h
 *  \class iso::event::EventManager manages events
 *	\brief manages events
 *
 *  Created by Daniel Bisig on 7/14/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_event_manager_h_
#define _iso_event_manager_h_

#include "iso_event.h"
#include "iso_event_listener.h"
#include "iso_event_exception.h"
#include <QHash>

namespace iso
{

namespace event
{

class EventScheduler;

typedef void (*EventHandleFunction)(Event*);

class EventManager
{
public:
	/**
	\brief default name
	*/
	static const base::String sClassName;

	/**
	\brief default constructor
	*/
	EventManager();

	/**
	\brief parameter based constructor
	\param pSchedulerUpdateInterval event scheduler update interval ( > 0.0: automated interval in milisecs, <= 0.0 : scheduler needs to be updated manually )
	*/
	EventManager(real pSchedulerUpdateInterval);
	
	/**
	\brief destructor
	*/
	~EventManager();

	/**
	\brief return name of event manager
	\return name of event manager
	*/
	const base::String& name() const;
	
	/**
	\brief return event scheduler
	\return event scheduler
	*/
	EventScheduler& scheduler();

	/**
	\brief set event scheduler update interval
	\param pSchedulerInterval update interval (in microsecs)
	*/
	void setSchedulerInterval(unsigned int pSchedulerInterval);

	/**
	\brief manual update scheduler
	*/
	void update();

	/**
	\brief terminate event manager
	*/
	void terminate();
	
	/**
	\brief add event
	\param pEvent event
	*/
	void addEvent( Event* pEvent );

	/**
	\brief remove events
	*/
	void removeEvents();

	/**
	\brief register event listener
	\param pEvent event type
	\param pEventListener event listener
	*/
	void addEventListener( Event* pEvent, EventListener* pEventListener );

	/**
	\brief register event listener
	\param pEvent event type
	\param pEventListener event listener
	*/
	void removeEventListener( Event* pEvent, EventListener* pEventListener );

	/**
	\brief inform event listeners about a newly created event
	\param pEvent event
	*/
	void notifyEventListeners(Event* pEvent);
		
	/**
	\brief obtain textual event manager information
	\param pPropagate print information for all events in the event scheduler
	\return base::String containing event manager information
	*/
	base::String info(bool pPropagate) const;

	/**
	\brief print event pool information
	\param pOstream output stream
	\param pEventManager event manager
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const EventManager& pEventManager)
	{
		pOstream << pEventManager.info(false).toStdString(); 
		
		return pOstream;
	}
		
protected:	
	/**
	\brief number of event managers
	*/
	static unsigned int sInstanceCount;

	/**
	\brief event manager name
	*/
	base::String mName;
	
	/**
	\brief event scheduler
	*/
	EventScheduler* mScheduler;
	
	/**
	\brief map from event type to event handler functions
	*/
	QHash<const std::type_info*, EventListener*> mEventListeners;
};

};

};

#endif