/** \file iso_geom_line_tools.h
 *
 *  Created by Daniel Bisig on 9/17/07.
 */

#ifndef _iso_geom_line_tools_h_
#define _iso_geom_line_tools_h_

#include "iso_geom_types.h"
#include <iso_math/iso_math_vector.h>
#include <iso_math/iso_math_vector2.h>
#include <iso_math/iso_math_vector3.h>

namespace iso
{

namespace geom
{

class LineTools
{
public:
	static LineTools& get();
	static void destroy();

	/**
	\brief calculcate closest point on line segment passing through pV1 and pV2
	\param pSP search point
	\param pV1 line vertex 1
	\param pV2 line vertex 2
	\param pRP resulting closest point
	\param pU resulting interplation parameter
	*/
	void closestPoint(const math::Vector<real>& pSP, const math::Vector<real>& pV1, const math::Vector<real>& pV2, math::Vector<real>& pRP, real& pU);

	/**
	\brief calculcate closest point on line segment passing through pV1 and pV2
	\param pSP search point
	\param pV1 line vertex 1
	\param pV2 line vertex 2
	\param pRP resulting closest point
	\param pU resulting interplation parameter
	*/
	void closestPointLine(const math::Vector<real>& pSP, const math::Vector<real>& pV1, const math::Vector<real>& pV2, math::Vector<real>& pRP, real& pU);
	
	/**
	\brief calculcate closest point on line segment passing through pV1 and pV2
	\param pSP search point
	\param pV1 line vertex 1
	\param pV2 line vertex 2
	\param pRP resulting closest point
	\param pU resulting interplation parameter
	*/
	void closestPointRay(const math::Vector<real>& pSP, const math::Vector<real>& pOrigin, const math::Vector<real>& pDirection, math::Vector<real>& pRP, real& pU);	

	/**
	\brief calculcate closest point on polyline
	\param pSP search point
	\param pVs polyline vertices
	\param pV2 line vertex 2
	\param pRP resulting closest point
	\param pI1 starting index & resulting index of begin of line segment
	\param pI2 ending index & resulting index of end of line segment
	\param pU resulting interplation parameter
	*/
	void closestPoint(const math::Vector<real>& pSP, const QVector< math::Vector<real> >& pVs, math::Vector<real>& pRP, int& pI1, int& pI2, real& pU );
	
	/**
	\brief calculate intersection between two lines
	\param pL1V1 start point of first line
	\param pL1V2 end point of first line
	\param pL2V1 start point of second line
	\param pL2V2 end point of second line
	\param pRP result point (if intersection exists)
	\return true, if there is an intersection, false otherwise
	*/
	bool intersect( const math::Vector<real>& pL1V1, const math::Vector<real>& pL1V2, const math::Vector<real>& pL2V1, const math::Vector<real>& pL2V2, math::Vector<real>& pRP );

	/**
	\brief calculate intersection between two lines
	\param pL1V1 start point of first line
	\param pL1V2 end point of first line
	\param pL2V1 start point of second line
	\param pL2V2 end point of second line
	\param pRP result point (if intersection exists)
	\return true, if there is an intersection, false otherwise
	*/
	bool intersectLineLine( const math::Vector<real>& pL1V1, const math::Vector<real>& pL1V2, const math::Vector<real>& pL2V1, const math::Vector<real>& pL2V2, math::Vector<real>& pRP );

	/**
	\brief calculate intersection between ray and line
	\param pRS start point of ray
	\param pRD direction of ray
	\param pLV1 start point of line
	\param pLV2 end point of line
	\param pRP result point (if intersection exists)
	\return true, if there is an intersection, false otherwise
	*/
	bool intersectRayLine( const math::Vector<real>& pRS, const math::Vector<real>& pRD, const math::Vector<real>& pLV1, const math::Vector<real>& pLV2, math::Vector<real>& pRP );

	/**
	\brief calculate intersection between two rays
	\param pR1S start point of first ray
	\param pR1D direction of first ray
	\param pR2S start point of second ray
	\param pR2D direction of second ray
	\param pRP result point (if intersection exists)
	\return true, if there is an intersection, false otherwise
	*/
	bool intersectRayRay( const math::Vector<real>& pR1S, const math::Vector<real>& pR1D, const math::Vector<real>& pR2S, const math::Vector<real>& pR2D, math::Vector<real>& pRP );

	/**
	\brief calculate intersection(s) between two lines
	\param pLV1 start point of line
	\param pLV2 end point of line
	\param pVs polyline vertices
	\param pRPs result points
	\return true, if there is at least one intersection
	*/
	bool intersectLinePolyLine( const math::Vector<real>& pLV1, const math::Vector<real>& pLV2, QVector< math::Vector<float> >& pVs, QVector< math::Vector<float> >& pRPs );

	/**
	\brief calculate intersection(s) between two lines
	\param pLV1 start point of line
	\param pLV2 end point of line
	\param pVs polyline vertices
	\param pRPs result points
	\param pIs stating indices of intersected line segments
	\return true, if there is at least one intersection
	*/
	bool intersectLinePolyLine( const math::Vector<real>& pLV1, const math::Vector<real>& pLV2, QVector< math::Vector<float> >& pVs, QVector< math::Vector<float> >& pRPs, QVector< int >& pIs );

	/**
	\brief calculate intersection between ray and line
	\param pRS start point of ray
	\param pRD direction of ray
	\param pVs polyline vertices
	\param pRPs result points
	\return true, if there is at least one intersection, false otherwise
	*/
	bool intersectRayPolyLine( const math::Vector<real>& pRS, const math::Vector<real>& pRD, QVector< math::Vector<float> >& pVs, QVector< math::Vector<float> >& pRPs );

	/**
	\brief calculate intersection between ray and line
	\param pRS start point of ray
	\param pRD direction of ray
	\param pVs polyline vertices
	\param pRPs result points
	\param pIs stating indices of intersected line segments
	\return true, if there is at least one intersection, false otherwise
	*/
	bool intersectRayPolyLine( const math::Vector<real>& pRS, const math::Vector<real>& pRD, QVector< math::Vector<float> >& pVs, QVector< math::Vector<float> >& pRPs, QVector< int >& pIs );

protected:
	LineTools();
	~LineTools();

	static LineTools* sLineTools;

};


};

};

#endif