/** \file iso_geom_mesh.h
 *
 *  Created by Daniel Bisig on 8/8/07.
 */

#ifndef _iso_geom_mesh_h_
#define _iso_geom_mesh_h_

#include "iso_geom_mesh_types.h"
#include "iso_geom_geometry.h"
#include "iso_geom_triangle.h"

namespace iso
{

namespace geom
{

class Mesh : public Geometry, public TriMesh
{
public:
	friend class MeshTools;
	
	/**
	\brief default constructor
	*/
	Mesh();
	
	/**
	\brief destructor
	*/
	~Mesh();
	
	QVector< Triangle > faces();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	/**
	\brief remove all vertices / edges / faces
	*/
	void clear();
	
	/**
	\brief access vertex
	\param pVertexHandle vertex handle
	\return vertex
	*/
	inline const Vertex& vertex(const VertexHandle& pVertexHandle) const;

	/**
	\brief access edge
	\param pEdgeHandle edge handle
	\return edge
	*/
	inline const Edge& edge(const EdgeHandle& pEdgeHandle) const;

	/**
	\brief face vertex
	\param pFaceHandle face handle
	\return face
	*/
	inline const Face& face(const FaceHandle& pFaceHandle) const;


	void enumVertices(VertexIter& pVertexIter);

	/**
	\brief begin vertex enumeration
	\return vertex iterator
	
	begin enumeration of all mesh vertices
	*/
	VertexIter enumVertices();
	
	/**
	\brief begin edge enumeration
	\return edge iterator
	
	begin enumeration of all mesh edges
	*/
	EdgeIter enumEdges();
	
	/**
	\brief begin face enumeration
	\return face iterator
	
	begin enumeration of all mesh faces
	*/
	FaceIter enumFaces();
	
	/**
	\brief begin vertex enumeration
	\param pVertexHandle vertex handle
	\return vertex-vertex iterator
	
	begin enumeration of all neighboring vertices
	*/
	VertexVertexIter enumVertexVertices(const VertexHandle& pVertexHandle);
	
	/**
	\brief begin edge enumeration
	\param pVertexHandle vertex handle
	\return vertex-edge iterator
	
	begin enumeration of all incident edges
	*/
	VertexEdgeIter enumVertexEdges(const VertexHandle& pVertexHandle);
	
	/**
	\brief begin face enumeration
	\param pVertexHandle vertex handle
	\return vertex-face iterator
	
	begin enumeration of all adjacent faces
	*/
	VertexFaceIter enumVertexFaces(const VertexHandle& pVertexHandle);
	
	/**
	\brief begin vertex enumeration
	\param pFaceHandle face handle
	\return face-vertex iterator
	
	begin enumeration over the face's vertices
	*/
	FaceVertexIter enumFaceVertices(const FaceHandle& pFaceHandle);

	/**
	\brief begin edge enumeration
	\param pFaceHandle face handle
	\return face-edge iterator
	
	begin enumeration over the face's edges
	*/
	FaceEdgeIter enumFaceEdges(const FaceHandle& pFaceHandle);

	/**
	\brief begin face enumeration
	\param pFaceHandle face handle
	\return face-face iterator
	
	begin enumeration over all edge neighboring faces
	*/
	FaceFaceIter enumFaceFaces(const FaceHandle& pFaceHandle);
	
	/**
	\brief continue vertex enumeration
	\param pVertexIter vertex iterator
	\param pVertexHandle returned vertex handle
	\return true if vertex available, false otherwise
	
	continue enumeration over all mesh vertices.\n
	*/
	inline bool nextVertexHandle(VertexIter& pVertexIter, VertexHandle& pVertexHandle);

	/**
	\brief continue edge enumeration
	\param pVertexIter edge iterator
	\param pEdgeHandle returned edge handle
	\return true if edge available, false otherwise
	
	continue enumeration over all mesh edges.\n
	*/
	inline bool nextEdgeHandle(EdgeIter& pEdgeIter, EdgeHandle& pEdgeHandle);

	/**
	\brief continue face enumeration
	\param pFaceIter face iterator
	\param pFaceHandle returned face handle
	\return true if face available, false otherwise
	
	continue enumeration over all mesh faces.\n
	*/
	inline bool nextFaceHandle(FaceIter& pFaceIter, FaceHandle& pFaceHandle);

	/**
	\brief continue vertex enumeration
	\param pVertexVertexIter vertex-vertex iterator
	\param pVertexHandle returned vertex handle
	\return true if vertex available, false otherwise
	
	continue enumeration of all neighboring vertices.\n
	*/
	inline bool nextVertexVertexHandle(VertexVertexIter& pVertexVertexIter, VertexHandle& pVertexHandle);

	/**
	\brief continue edge enumeration
	\param pVertexEdgeIter vertex-edge iterator
	\param pEdgeHandle returned edge handle
	\return true if edge available, false otherwise
	
	continue enumeration of all incident edges.\n
	*/
	inline bool nextVertexEdgeHandle(VertexEdgeIter& pVertexEdgeIter, EdgeHandle& pEdgeHandle);

	/**
	\brief continue face enumeration
	\param pVertexFaceIter vertex-face iterator
	\param pFaceHandle returned face handle
	\return true if face available, false otherwise
	
	continue enumeration of all adjacent faces.\n
	*/
	inline bool nextVertexFaceHandle(VertexFaceIter& pVertexFaceIter, FaceHandle& pFaceHandle);

	/**
	\brief continue vertex enumeration
	\param pFaceVertexIter face-vertex iterator
	\param pVertexHandle returned vertex handle
	\return true if vertex available, false otherwise
	
	continue enumeration over the face's vertices.\n
	*/
	inline bool nextFaceVertexHandle(FaceVertexIter& pFaceVertexIter, VertexHandle& pVertexHandle);

	/**
	\brief continue edge enumeration
	\param pFaceEdgeIter face-edge iterator
	\param pEdgeHandle returned edge handle
	\return true if edge available, false otherwise
	
	continue enumeration over the face's edges.\n
	sets edge handle to NULL when enumeration is completed
	*/
	inline bool nextFaceEdgeHandle(FaceEdgeIter& pFaceEdgeIter, EdgeHandle& pEdgeHandle);

	/**
	\brief continue face enumeration
	\param pFaceFaceIter face-face iterator
	\param pFaceHandle returned face handle
	\return true if face available, false otherwise
	
	continue enumeration over all edge neighboring faces.\n
	*/
	inline bool nextFaceFaceHandle(FaceFaceIter& pFaceFaceIter, FaceHandle& pFaceHandle);

	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);
	
protected:

	VertexIter mVertexIterEnd; /// \brief internal iterator that indicate end of vertex iteration
	EdgeIter mEdgeIterEnd; /// \brief internal iterator that indicates end of edge iteration
	FaceIter mFaceIterEnd; /// \brief internal iterator that indicates end of face iteration
};

const Vertex&
Mesh::vertex(const VertexHandle& pVertexHandle) const
{
	return DefaultMesh::vertex( pVertexHandle );
}

const Edge&
Mesh::edge(const EdgeHandle& pEdgeHandle) const
{
	return DefaultMesh::edge( pEdgeHandle );
}

const Face&
Mesh::face(const FaceHandle& pFaceHandle) const
{
	return DefaultMesh::face( pFaceHandle );
}

bool
Mesh::nextVertexHandle(VertexIter& pVertexIter, VertexHandle& pVertexHandle)
{
	if( pVertexIter != mVertexIterEnd ) 
	{
		pVertexHandle = pVertexIter.handle();	
		++pVertexIter;
		return true;
	}
	else return false;
}

bool
Mesh::nextEdgeHandle(EdgeIter& pEdgeIter, EdgeHandle& pEdgeHandle)
{
	if( pEdgeIter != mEdgeIterEnd ) 
	{
		pEdgeHandle = pEdgeIter.handle();	
		++pEdgeIter;
		return true;
	}
	else return false;
}

bool 
Mesh::nextFaceHandle(FaceIter& pFaceIter, FaceHandle& pFaceHandle)
{
	if( pFaceIter != mFaceIterEnd ) 
	{
		pFaceHandle = pFaceIter.handle();	
		++pFaceIter;
		return true;
	}
	else return false;
}

bool
Mesh::nextVertexVertexHandle(VertexVertexIter& pVertexVertexIter, VertexHandle& pVertexHandle)
{
	if(pVertexVertexIter) 
	{
		pVertexHandle = pVertexVertexIter.handle();
		++pVertexVertexIter;
		return true;
	}
	else return false;
}

bool
Mesh::nextVertexEdgeHandle(VertexEdgeIter& pVertexEdgeIter, EdgeHandle& pEdgeHandle)
{
	if(pVertexEdgeIter) 
	{
		pEdgeHandle = pVertexEdgeIter.handle();
		++pVertexEdgeIter;
		return true;
	}
	else return false;
}

bool
Mesh::nextVertexFaceHandle(VertexFaceIter& pVertexFaceIter, FaceHandle& pFaceHandle)
{
	if(pVertexFaceIter) 
	{
		pFaceHandle = pVertexFaceIter.handle();
		++pVertexFaceIter;
		return true;
	}
	else return false;
}

bool
Mesh::nextFaceVertexHandle(FaceVertexIter& pFaceVertexIter, VertexHandle& pVertexHandle)
{
	if(pFaceVertexIter) 
	{
		pVertexHandle = pFaceVertexIter.handle();
		++pFaceVertexIter;
		return true;
	}
	else return false;
}

bool
Mesh::nextFaceEdgeHandle(FaceEdgeIter& pFaceEdgeIter, EdgeHandle& pEdgeHandle)
{
	if(pFaceEdgeIter) 
	{
		pEdgeHandle = pFaceEdgeIter.handle();
		++pFaceEdgeIter;
		return true;
	}
	else return false;
}

bool
Mesh::nextFaceFaceHandle(FaceFaceIter& pFaceFaceIter, FaceHandle& pFaceHandle)
{
	if(pFaceFaceIter) 
	{
		pFaceHandle = pFaceFaceIter.handle();
		++pFaceFaceIter;
		return true;
	}
	else return false;
}

};

};


#endif