/** \file iso_geom_spline.h
 *
 *  Created by Daniel Bisig on 4/1/07.
 */

#ifndef _iso_geom_spline_h_
#define _iso_geom_spline_h_

#include "iso_geom_types.h"
#include "iso_geom_geometry.h"
#include <iso_math/iso_math_vector.h>
#include <QVector>

namespace iso
{

namespace geom
{

class SplineAlg;

class Spline : public Geometry
{
public:
	/**
	\brief create spline
	\param p_dimension dimension of spline
	
	contains zero control points
	*/
	Spline(unsigned int p_dimension);
	
	/**
	\brief create spline
	\param p_controlPoints spline control points
	*/
	Spline(QVector< math::Vector<real> >& p_controlPoints);

	/**
	\brief copy constructor
	\param pSpline spline to copy
	*/
	Spline(const Spline& pSpline);
	
	/**
	\brief destructor
	*/
	~Spline();

	/**
	\brief spline = spline
	\param pSpline spline to copy values from
	\return spline
	\remark fails if dimensions don't match
	*/
    const Spline& operator=( const Spline& pSpline ); 
	
	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();	
	
	/**
	\brief return subdivision count
	\return subdivision count
	*/
	unsigned int subdivisionCount() const;
	
	/**
	\brief set subdivision count
	\param p_subdivisionCount subdivision count
	*/
	void setSubdivisionCount(unsigned int p_subdivisionCount);

	/**
	\brief return closed
	\return closed
	*/
	bool closed() const;
	
	/**
	\brief set closed
	\param p_closed closed
	*/
	void setClosed(bool p_closed);
	
	/**
	\brief return spline algorithm
	*/
	SplineAlg* splineAlg();
	
	/**
	\brief set spline algorithm
	\param p_splineAlg spline algorithm
	\remarks fails if dimension of spline and spline algorithm differ
	*/
	void setSplineAlg(SplineAlg* p_splineAlg);

	/**
	\brief return control points
	\return control points
	*/
	QVector< math::Vector<real> >& controlPoints();
	
	/**
	\brief return spline points
	\return spline points
	*/
	QVector< math::Vector<real> >& splinePoints();
	
	/**
	\brief add control point
	\param p_controlPoint control point
	\remarks fails if dimension of spline and control point differ
	*/
	void addControlPoint(math::Vector<real>& p_controlPoint);
	
	/**
	\brief add control points
	\param p_controlPoints vector of control points
	\remarks fails if dimension of spline and control points differ
	*/
	void addControlPoints(QVector< math::Vector<real> >& p_controlPoints);
	
	/**
	\brief remove all control points
	*/
	void removeControlPoints();
	
	/**
	\brief update spline points
	\remarks spline employs internal spline algorithm if available
	*/
	void update();

	/**
	\brief update spline points
	\param pSplineAlg spline algorithm
	\remarks employs external spline algorithm
	*/
	void update(SplineAlg* pSplineAlg);

	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

protected:
	/**
	\brief default constructor
	*/
	Spline();

	/**
	\brief spline control points
	*/
	QVector< math::Vector<real> > m_controlPoints;
	
	/**
	\brief spline points
	*/
	QVector< math::Vector<real> > m_splinePoints;
	
	/**
	\brief spline algorithm
	
	calculates spline points from control points
	*/
	SplineAlg* m_splineAlg;
	
	/**
	\brief spline closed or open
	*/
	bool m_closed;
};

};

};

#endif