/** \file iso_gl_texture.h
 *  \class iso::gl::Texture
 *
 *  Created by Daniel Bisig on 12/4/07.
 *
 */

#ifndef _iso_gl_texture_h_
#define _iso_gl_texture_h_

#include "iso_gl.h"
#include "iso_gl_types.h"
#include "iso_gl_exception.h"
#include <iso_data/iso_data_includes.h>

namespace iso
{

namespace gl
{

class Texture
{
public:
	/**
	\brief create texture
	\param pSize texture size
	\param pTarget texture target
	\param pLevel mid mapping level
	\param pInternalFormat internal texture format
	\param pFormat pixel data format
	\param pType pixel data type
	*/
	Texture( const math::Vector<unsigned int>& pSize, GLenum pTarget, GLint pLevel, GLint pInternalFormat, GLenum pFormat, GLenum pType ) throw (GLException);

	/**
	 \brief create texture
	 \param pDim texture dimension
	 \param pSize size of texture
	 \param pTarget texture target
	 \param pLevel mid mapping level
	 \param pInternalFormat internal texture format
	 \param pFormat pixel data format
	 \param pType pixel data type
	 */
	Texture( unsigned int pDim, unsigned int* pSize, GLenum pTarget, GLint pLevel, GLint pInternalFormat, GLenum pFormat, GLenum pType ) throw (GLException);	

	/**
	 \brief create texture
	 \param pId texture id
	 \param pDim texture dimension
	 \param pSize size of texture
	 \param pTarget texture target
	 \param pLevel mid mapping level
	 \param pInternalFormat internal texture format
	 \param pFormat pixel data format
	 \param pType pixel data type
	 */
	Texture( GLuint pId, unsigned int pDim, unsigned int* pSize, GLenum pTarget, GLint pLevel, GLint pInternalFormat, GLenum pFormat, GLenum pType );	
	
	/**
	\brief destructor
	*/
	virtual ~Texture();

	/**
	\brief returns true if texture has been created, false otherwise
	*/
	inline bool created() const;

	/**
	\brief create texture
	*/
	virtual void create() throw (GLException);

	/**
	\brief create texture from data
	\param pData texture data
	\remark fails if texture data has wrong format
	*/
	template<typename DataType> void create( DataType* pData ) throw (GLException);

	/**
	\brief update texture based on internally stored buffered
	
	has no meaning in this base class
	*/
	virtual void update() throw (GLException);

	/**
	\brief update texture from data
	\param pData texture data
	\remark fails if texture data has wrong format
	*/
	template<typename DataType> void update( DataType* pData ) throw (GLException);

	/**
	\brief return texture id
	\return texture id
	*/
	int id() const;
	
	/**
	\brief return target
	\return target
	*/
	GLenum target() const;
	
	/**
	 \brief return texture dimension
	 \returns texture dimension
	 */
	unsigned int dim() const;
	
	/**
	 \brief return texture size
	 \param pSize size array
	 */
	void size( unsigned int* pSize ) const;

	/**
	\brief return texture size
	\return texture size
	 deprecated
	*/
	math::Vector<int> size() const;
	
	/**
	\brief return number of color channels
	\return number of color channels
	*/
	int channelCount() const;

	/**
	\brief return internal format
	\return internal format
	*/
	GLint internalFormat() const;
	
	/**
	\brief return pixel data format
	return pixel data format
	*/
	GLenum format() const;

	/**
	\brief return pixel data type
	return pixel data type
	*/
	GLenum type() const;

	/**
	\brief bind texture
	*/
	void bind();

	/**
	\brief release texture
	*/
	void release();
	
	/**
	\brief destroy texture
	*/
	void destroyTexture();

	/**
	\brief print texture information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const Texture& pTexture)
	{
		pOstream << pTexture.info().toStdString();
		
		return pOstream;
	};
	
	/**
	\brief obtain textual texture information
	\return String containing texture information
	*/
	virtual base::String info() const;
	
protected:
	/**
	\brief default constructor
	*/
	Texture();

	GLuint mId; ///\brief texture id
	unsigned int mDim; ///\brief texture dimension
	unsigned int* mSize; /**> \brief texture size */
	GLenum  mTarget; ///\brief texture target 
 	GLint  	mLevel; ///\brief mid mapping level
 	GLint  	mInternalFormat; ///\brief internal texture format
 	GLenum  mFormat; ///\brief pixel data format
 	GLenum  mType; ///\brief pixel data type
	bool mCreated;
	bool mDoNotDispose;
};

bool 
Texture::created() const
{
	return mCreated;
}

template<typename DataType> 
void 
Texture::create( DataType* pData ) throw (GLException)
{
	GL::get().clearError();

    glBindTexture(mTarget, mId);   
  
 	glTexParameteri(mTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(mTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	
	if(mDim == 1) glTexImage1D(mTarget, mLevel, mInternalFormat, mSize[0], 0, mFormat, mType, pData );
	else if(mDim == 2) glTexImage2D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], 0, mFormat, mType, pData );
	else if(mDim == 3) glTexImage3D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], mSize[2], 0, mFormat, mType, pData );	

	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
	
		throw(GLException( "ERR_TEXTURE", base::String("failed to create texture"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
	
	mCreated = true;
}

template<typename DataType> 
void 
Texture::update( DataType* pData ) throw (GLException)
{
	if( mCreated == false )
	{
		try 
		{
			create();
			return;
		}
		catch (GLException& e) 
		{
			throw;
		}
	}

	GL::get().clearError();

    glBindTexture(mTarget, mId);   
	
	glTexParameteri(mTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(mTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	

	if(mDim == 1) glTexImage1D(mTarget, mLevel, mInternalFormat, mSize[0], 0, mFormat, mType, pData );
	else if(mDim == 2) glTexImage2D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], 0, mFormat, mType, pData );
	else if(mDim == 3) glTexImage3D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], mSize[2], 0, mFormat, mType, pData );	

	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_TEXTURE", base::String("failed to update texture"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
}

};

};

#endif
