/**	\file iso_math.h
 *	\class iso::math::Math singleton class that provides handy math functions and constants
 *	\brief singleton class that provides handy math functions and constants
 *
 *  Created by Daniel Bisig on 5/17/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef _iso_math_h_
 #define _iso_math_h_
 
 #include "iso_math_types.h"
 #include "iso_math_exception.h"
 #include "iso_math_vector.h"
 #include <iso_base/iso_base_types.h>
 #include <newran/newran.h>
 
 namespace iso
 {
 
 namespace math
 {
 
 class Math
 {
 public:
	/**
	\brief get singleton instance
	\return reference to instance
	*/
	static Math& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();

	/**
	\brief integer power function
	\param pBase base
	\param pExponent exponent
	\return pBase ^ pExponent
	*/
	inline int powi(int pBase, int pExponent) const;
	
	/**
	\brief returns the smaller of the two supplied values
	\param p1 first value
	\param p2 second value 
	*/
	template <class t>
	t min(t p1, t p2)
	{
		return p1<p2 ? p1 : p2;
	};
	
	/**
	\brief returns the larger of the two supplied values
	\param p1 first value
	\param p2 second value 
	*/
	template <class t>
	t max(t p1, t p2)
	{
		return p1>p2 ? p1 : p2;
	};
	
	/**
	\brief returns factorial of an positive integer
	\param pValue positive integer
	\return factorial
	*/	
	unsigned int 
	factorial(unsigned int pValue)
	{
		int value = static_cast<int>(pValue);
		int returnValue = 1;
		
		for(; value > 0; --value) returnValue *= value;
		
		return returnValue;
	}
	
	/**
	\brief checks whether number is power of two
	\return true if number is power of two, false otherwise
	*/
	bool checkPowerOfTwo(unsigned int pValue);
	
	/**
	\brief transforms second input value into a (larger) number that's related to the first input value by a power of two
	\param pFixValue fixed input value
	\param pVarValue variable input value
	*/
	void powerOfTwoRatio(real pFixValue, real& pVarValue);

	/**
	\brief return uniform random number (in between 0 - 1)
	\return uniform random number
	*/
	real random();
	
	/**
	\brief return uniform random number (in between 0 - pUpValue)
	\param pUpValue upper random number limit
	\return uniform random number
	*/
	real random(real pUpValue);
	
	/**
	\brief return uniform random number (in between pLowValue - pUpValue)
	\param pLowValue lower random number limit
	\param pUpValue upper random number limit
	\return uniform random number
	*/
	real random(real pLowValue, real pUpValue);

	/**
	\brief return uniform random number (in between 0 - 1)
	\return uniform random number
	*/
	real uniformRandom();
	
	/**
	\brief return uniform random number (in between 0 - pUpValue)
	\param pUpValue upper random number limit
	\return uniform random number
	*/
	real uniformRandom(real pUpValue);
	
	/**
	\brief return uniform random number (in between pLowValue - pUpValue)
	\param pLowValue lower random number limit
	\param pUpValue upper random number limit
	\return uniform random number
	*/
	real uniformRandom(real pLowValue, real pUpValue);

	/**
	\brief converts degrees to radians
	*/
	real radian(real pValue);

	/**
	\brief converts radians to degrees
	*/
	real degree(real pValue);
	
	/**
	\brief converts degrees to radians
	*/
	real degreeToRadian(real pValue);
	
	/**
	\brief converts radians to degrees
	*/
	real radianToDegree(real pValue);
	
	/**
	\brief transforms complex number into polar angle
	\param aValue aValue
	\param bValue bValue
	\return angle in radians 
	*/
	real complexToPhase(real aValue, real bValue);
	
	/**
	\brief gaussian function
	\param pXValue x value
	\param pMean mean
	\param pStd standard deviation
	/return y value
	*/
	real gaussian(real pXValue, real pMean, real pStd);

	/**
	\brief hamming function
	\param pXValue x value
	/return y value
	
	runs from 0.0 to 1.0
	*/
	real hamming(real pXValue);

	/**
	\brief hann function
	\param pXValue x value
	/return y value
	
	runs from 0.0 to 1.0
	*/
	real hann(real pXValue);

	/**
	\brief hann function
	\param pXValue x value
	/return y value
	
	runs from 0.0 to 1.0
	*/
	real kaiser(real pXValue);
	
	/**
	\brief sigmoid function
	\param pXValue x value
	\param pA a value
	\return y value
	*/
	real sigmoid(real pXValue, real pA);
	
	/**
	\brief normalized sigmoid function
	\param pXValue x value
	\param pA a value
	\return y value
	
	the normal function input range from -6 to +6 has been rescaled and shifted to the range of 0 to 1
	*/
	real nSigmoid(real pXValue, real pA);
	
	/**
	\brief calculate index into n-dimensional array
	\param pSubdivisions array subdivisions
	\param pPosition position into array
	\return index
	\exception MathException dimensions of subdivisons and position don't match or if positition is out of bounds
	*/
	unsigned int index( const Vector<unsigned int>& pSubdivisions, const Vector<unsigned int>& pPosition ) throw (MathException);

	static const real PI; ///\brief PI
 
 protected:
	/**
	\brief default constructor
	*/
	Math();
	
	/**
	\brief destructor
	*/
	~Math();
	
	NEWRAN::Uniform* mUniformRandom; ///\brief random number variable
	NEWRAN::MotherOfAll* mURNG; ///\brief newran generator
	static Math* sMath; ///\brief singleton instance
 };
 
 };
 
 };
 
 #endif

