/**	\file iso_math_exception.h
 *  \class iso::math::MathException class for handling all kinds of math related run time errors
 *	\brief class for handling all kinds of math related run time errors
 *
 *	Defines a variety of run time errors, which are specific to the math classes:
 *  ERR_VALUE_NOT_POWER_OF_TWO : Value is not a Power of Two
 *	ERR_VECTOR_VECTOR_MISMATCH : Vectors Don't Match
 *	ERR_VECTOR_MATRIX_MISMATCH : Vector and Matrix Don't Match
 *	ERR_MATRIX_MATRIX_MISMATCH : Matrix and Matrix Don't Match
 *  ERR_VECTOR_ERROR : Vector ERROR
 *  ERR_MATRIX_ERROR : Matrix ERROR
 *  ERR_DIMENSION_MISMATCH : Dimension Mismatch Error 
 *
 *  Created by Daniel Bisig on 5/17/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_math_exception_h_
#define _iso_math_exception_h_

#include "iso_math_types.h"
#include <iso_base/iso_base_types.h>
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace math
{

class MathException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	MathException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	MathException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief destructor
	*/
	~MathException();
	
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const MathException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
		
protected:	
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();

	/**
	\brief default constructur
	*/
	MathException();	
};

};

};


#endif