/**	\file iso_math_matrixh.h
 *	\class iso::math::MatrixH nxn matrix class
 *	\brief nxn matrix class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

 #ifndef _iso_math_matrixH_h_
 #define _iso_math_matrixH_h_
 
 #include "iso_math_vector.h"
 #include "iso_math_matrix.h"
 
 namespace iso
 {
 
 namespace math
 {
 
 template< class Type> 
 class MatrixH : public Matrix<Type>
 {
 public:
 	/**
	\brief create default zero matrix
	*/
	MatrixH();
	
 	/**
	\brief create zero matrix
	\param pDim dimension of matrix
	*/	
	MatrixH(size_t pDim);

	/**
	\brief create matrix containing identical values
	\param pDim dimension of matrix
	\param pValue value
	*/	
	MatrixH(size_t pDim, Type pValue);

	/**
	\brief create matrix
	\param pDim dimension of matrix
	\param pValues values (must be of correct size!)
	*/	
	MatrixH(size_t pDim, Type* pValues);
 	
	/**
	\brief copy constructor
	\param pMatrix matrix
	*/	
	MatrixH(const MatrixH<Type>& pMatrix);
	
	/**
	\brief copy constructor
	\param pMatrix matrix
	\exception MathException pMatrix not a NxN matrix
	*/	
	MatrixH(const Matrix<Type>& pMatrix) throw (MathException);

	/**
	\brief destructor
	*/
	virtual ~MatrixH();
	
	/**
	\brief set all matrix elements to zer
	*/
	void setZero();
	
	/**
	\brief set matrix elements to those of an identity matrix
	*/
	void setIdentity();
	
	/**
	\brief get inverse of matrix
	\return inverse of matrix
	*/
	virtual Matrix<Type> getInverse() const;

	/**
	\brief calculate inverse of a matrix
	\param pMatrix inverse matrix
	\exception MathException matrix dimension mismatch
	*/
	virtual void getInverse(Matrix<Type>& pMatrix) const throw (MathException);

	/**
	\brief inverse scale of matrix
	\return inverse scale of matrix
	*/
	virtual Matrix<Type> getInverseScale() const;

	/**
	\brief calculate inverse scale of matrix
	\param pMatrix inversed scale matrix
	*/
	virtual void getInverseScale(Matrix<Type>& pMatrix) const;

	/**
	\brief change scale
	\param pScale uniform scale
	*/
	void scale( Type pScale );
	
	/**
	\brief change scale
	\param pScale non-uniform scale values
	\remarks no size check is performed on array
	*/
    void scale( const Type* pScale );
	
	/**
	\brief change scale
	\param pScale non-uniform scale values
	\exception MathException wrong scale vector dimension
	*/
    void scale( const Vector< Type >& pScale ) throw (MathException);
	
	/**
	\brief set scale
	\param pScale uniform scale
	*/	
	void setScale( Type pScale );
	
	/**
	\brief set scale
	\param pScale non-uniform scale values
	\remarks no size check is performed on array
	*/	
    void setScale( const Type* pScale );

	/**
	\brief set scale
	\param pScale non-uniform scale values
	\exception MathException wrong scale vector dimension
	*/
    void setScale( const Vector< Type >& pScale ) throw (MathException);

	/**
	\brief change translation
	\param pTrans translation values
	\remarks no size check is performed on array
	*/	
    void translate( const Type* pTrans );
 
	/**
	\brief change translation
	\param pTrans translation values
	\exception MathException wrong translation vector dimension
	*/	
	void translate( const Vector<Type>& pTrans ) throw (MathException);
	
	/**
	\brief set translation
	\param pTrans translation values
	\remarks no size check is performed on array
	*/		
    void setTranslation( const Type* pTrans );

	/**
	\brief set translation
	\param pTrans translation values
	\exception MathException wrong translation vector dimension
	*/	
    void setTranslation( const Vector<Type>& pTrans ) throw (MathException);
	
	/**
	\brief change rotation
	\param pAngle rotation angle
	\param pAxis rotation axis
	\exception MathException wrong rotation axis vector dimension
	*/		
	void rotate( Type pAngle, const Vector<Type>& pAxis ) throw (MathException);

	/**
	\brief set rotation
	\param pAngle rotation angle
	\param pAxis rotation axis
	\exception MathException wrong rotation axis vector dimension
	*/		
	void setRotation( Type pAngle, const Vector<Type>& pAxis ) throw (MathException);
	
	/**
	\brief multiply vector/point in homogenous coordinates
	\param pVec vector/point in homogenous coordinates
	\return vector in homogenous coordinates
	\exception MathException wrong vector dimension
	*/
	Vector<Type> multiply(const Vector<Type>& pVec) const throw (MathException);
	
	/**
	\brief multiply standard coordinate vector
	\param pVec vector in standard coordinates
	\param pHValue homogenous last coordinate: 0 : vector 1: point
	\return vector in standard coordinates
	\exception MathException wrong vector dimension
	*/
	Vector<Type> multiply(const Vector<Type>& pVec, Type pHValue) const throw (MathException);
 }; 
 
 typedef MatrixH<int>    MatrixHI;
 typedef MatrixH<float>  MatrixHF;
 typedef MatrixH<double> MatrixHD;
 
 
 template<class Type>
 MatrixH<Type>::MatrixH()
	: Matrix<Type>(4, 4)
 {}
 
  template<class Type>
 MatrixH<Type>::MatrixH(size_t pDim)
	: Matrix<Type>(pDim, pDim)
 {}
 
 template<class Type>
 MatrixH<Type>::MatrixH(size_t pDim, Type pValue)
	: Matrix<Type>(pDim, pDim, pValue)
 {}
 
 template<class Type>
 MatrixH<Type>::MatrixH(size_t pDim, Type* pValues)
	: Matrix<Type>(pDim, pDim, pValues)
 {}

 template<class Type>	
 MatrixH<Type>::MatrixH(const MatrixH<Type>& pMatrix)
	: Matrix<Type>(pMatrix)
 {}

 template<class Type>	
 MatrixH<Type>::MatrixH(const Matrix<Type>& pMatrix) throw (MathException)
	: Matrix<Type>(pMatrix)
 {
  	#ifdef _math_debug_
	if( Matrix<Type>::mDim.x != Matrix<Type>::mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("matrix not NxN. rows: %1 columns: %2 ").arg( Matrix<Type>::mDim.x ).arg( Matrix<Type>::mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
 }
 
 template<class Type>
 MatrixH<Type>::~MatrixH()
 {}

 template<class Type> 
 void 
 MatrixH<Type>::setZero()
 {
	for(size_t i=0; i<Matrix<Type>::m_count; ++i) Matrix<Type>::m[i] = 0.0;
 }
 
 template<class Type>
 void 
 MatrixH<Type>::setIdentity()
 {
	for(size_t row = 0, index = 0; row < Matrix<Type>::mDim.x; ++row)
	{
		for(size_t col = 0; col < Matrix<Type>::mDim.y; ++col, ++index)
		{
			if(row == col) Matrix<Type>::m[index] = 1.0;
			else Matrix<Type>::m[index] = 0.0;
		}
	}
 }

 template<class Type>
 Matrix<Type> 
 MatrixH<Type>::getInverse() const
 {
	Matrix<Type> inverseMatrix = *this;
	
	size_t rowCount_1 = Matrix<Type>::mDim.x - 1;
	size_t colCount_1 = Matrix<Type>::mDim.y - 1;
	
	// transpose rotation part
	for(size_t row = 0, index = 0; row < rowCount_1; ++row, ++index)
	{
		for(size_t col = 0; col < colCount_1; ++col, ++index)
		{
			inverseMatrix.m[index] = Matrix<Type>::m[col * Matrix<Type>::mDim.y + row];
		}
	}
	
	//negate translation part
	for(size_t row = 0, index = colCount_1; row < rowCount_1; ++row, index += Matrix<Type>::mDim.y) 	
	{
		inverseMatrix.m[index] = -Matrix<Type>::m[index];
	}
	
	// copy bottom row
	for(size_t col = 0, index = rowCount_1 * Matrix<Type>::mDim.y; col < Matrix<Type>::mDim.y; ++col, ++index)
	{
		inverseMatrix.m[index] = Matrix<Type>::m[index];
	}
	
	return inverseMatrix;
 }
 
 template<class Type>
 void 
 MatrixH<Type>::getInverse(Matrix<Type>& pMatrix) const throw (MathException)
 {
	//std::cout << " MatrixH<Type>::getInverse(Matrix<Type>& pMatrix) start\n";
 
 	#ifdef _math_debug_
	if( Matrix<Type>::mDim != pMatrix.dim() ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected matrix dim: %1 supplied dim %2").arg( Matrix<Type>::mDim.info() ).arg( pMatrix.dim().info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	//std::cout << "pMatrix before\n" << pMatrix << "\n";
	
	size_t rowCount_1 = Matrix<Type>::mDim.x - 1;
	size_t colCount_1 = Matrix<Type>::mDim.y - 1;
	
	// transpose rotation part
	for(size_t row = 0, index = 0; row < rowCount_1; ++row, ++index)
	{
		for(size_t col = 0; col < colCount_1; ++col, ++index)
		{
			pMatrix.m[index] = Matrix<Type>::m[col * Matrix<Type>::mDim.y + row];
		}
	}
	
	//negate translation part
	for(size_t row = 0, index = colCount_1; row < rowCount_1; ++row, index += Matrix<Type>::mDim.y) 	
	{
		pMatrix.m[index] = -Matrix<Type>::m[index];
	}
	
	// copy bottom row
	for(size_t col = 0, index = rowCount_1 * Matrix<Type>::mDim.y; col < Matrix<Type>::mDim.y; ++col, ++index)
	{
		pMatrix.m[index] = Matrix<Type>::m[index];
	}
		
	//std::cout << "pMatrix after\n" << pMatrix << "\n";
	
	//std::cout << " MatrixH<Type>::getInverse(Matrix<Type>& pMatrix) done\n";
 }

 template<class Type>
 Matrix<Type> 
 MatrixH<Type>::getInverseScale() const
 {
	Matrix<Type> inverseMatrix = *this;
 
 	for(size_t row = 0, index = 0; row < Matrix<Type>::mDim.x; ++row)
	{
		for(size_t col = 0; col < Matrix<Type>::mDim.y; ++col, ++index)
		{
			if(row == col) inverseMatrix.m[index] = 1.0 / inverseMatrix.m[index];
		}
	}
	
	return inverseMatrix;
 }

 template<class Type>
 void 
 MatrixH<Type>::getInverseScale(Matrix<Type>& pMatrix) const
 {
  	for(size_t row = 0, index = 0; row < Matrix<Type>::mDim.x; ++row)
	{
		for(size_t col = 0; col < Matrix<Type>::mDim.y; ++col, ++index)
		{
			if(row == col) pMatrix.m[index] = 1.0 / Matrix<Type>::m[index];
		}
	}
 }

 template<class Type>
 void 
 MatrixH<Type>::scale( Type pScale )
 {
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = 0; pos < count_1; ++pos, index += Matrix<Type>::mDim.x + 1)
	{
		Matrix<Type>::m[index] *= pScale;
	}
 }

 template<class Type>
 void 
 MatrixH<Type>::scale( const Type* pScale )
 {
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = 0; pos < count_1; ++pos, index += Matrix<Type>::mDim.x + 1)
	{
		Matrix<Type>::m[index] *= pScale[pos];
	}
 }
  
 template<class Type>
 void 
 MatrixH<Type>::scale( const Vector< Type >& pScale ) throw (MathException)
 {
 	#ifdef _math_debug_
	if( pScale.dim() != static_cast<int>(Matrix<Type>::mDim.y) - 1 ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( static_cast<int>(Matrix<Type>::mDim.y) - 1 ).arg( pScale.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
 
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = 0; pos < count_1; ++pos, index += Matrix<Type>::mDim.x + 1)
	{
		Matrix<Type>::m[index] *= pScale[pos];
	}
 }
 
 template<class Type>
 void 
 MatrixH<Type>::setScale( Type pScale )
 {
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = 0; pos < count_1; ++pos, index += Matrix<Type>::mDim.x + 1)
	{
		Matrix<Type>::m[index] = pScale;
	}
 }
 
 template<class Type>
 void 
 MatrixH<Type>::setScale( const Type* pScale )
 {
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = 0; pos < count_1; ++pos, index += Matrix<Type>::mDim.x + 1)
	{
		Matrix<Type>::m[index] = pScale[pos];
	}
 }
  
 template<class Type>
 void 
 MatrixH<Type>::setScale( const Vector< Type >& pScale ) throw (MathException)
 {
  	#ifdef _math_debug_
	if( pScale.dim() != static_cast<int>(Matrix<Type>::mDim.y) - 1 ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( static_cast<int>(Matrix<Type>::mDim.y) - 1 ).arg( pScale.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
 
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = 0; pos < count_1; ++pos, index += Matrix<Type>::mDim.x + 1)
	{
		Matrix<Type>::m[index] = pScale[pos];
	}
 }
  
 template<class Type>
 void 
 MatrixH<Type>::translate( const Type* pTrans )
 {
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = count_1; pos < count_1; ++pos, index += Matrix<Type>::mDim.x) Matrix<Type>::m[index] += pTrans[pos];
 }
 
 template<class Type>
 void 
 MatrixH<Type>::translate( const Vector<Type>& pTrans ) throw (MathException)
 {
   	#ifdef _math_debug_
	if( pTrans.dim() != static_cast<int>(Matrix<Type>::mDim.x) - 1 ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( static_cast<int>(Matrix<Type>::mDim.x) - 1 ).arg( pTrans.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
 
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = count_1; pos < count_1; ++pos, index += Matrix<Type>::mDim.x) Matrix<Type>::m[index] += pTrans[pos];
 }
 
 template<class Type>
 void 
 MatrixH<Type>::setTranslation( const Type* pTrans )
 {
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = count_1; pos < count_1; ++pos, index += Matrix<Type>::mDim.x) Matrix<Type>::m[index] = pTrans[pos];
 }
 
 template<class Type>
 void 
 MatrixH<Type>::setTranslation( const Vector<Type>& pTrans ) throw (MathException)
 {
   	#ifdef _math_debug_
	if( pTrans.dim() != static_cast<int>(Matrix<Type>::mDim.x) - 1 ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( static_cast<int>(Matrix<Type>::mDim.x) - 1 ).arg( pTrans.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	 
	int count_1 = static_cast<int>( Matrix<Type>::mDim.x ) - 1;
 
	for(size_t pos = 0, index = count_1; pos < count_1; ++pos, index += Matrix<Type>::mDim.x) Matrix<Type>::m[index] = pTrans[pos];
 }
 
 template<class Type>
 void 
 MatrixH<Type>::rotate( Type pAngle, const Vector<Type>& pAxis ) throw (MathException)
 {
	#ifdef _math_debug_
	if( pAxis.dim() != static_cast<int>(Matrix<Type>::mDim.x) - 1 ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( static_cast<int>(Matrix<Type>::mDim.x) - 1 ).arg( pAxis.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	MatrixH<Type> tmpMatrix(Matrix<Type>::mDim.x);
	tmpMatrix.setIdentity();
	tmpMatrix.setRotation(pAngle, pAxis);
	
	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 MatrixH<Type>::setRotation( Type pAngle,  const Vector<Type>& pAxis ) throw (MathException)
 {
	#ifdef _math_debug_
	if( pAxis.dim() != static_cast<int>(Matrix<Type>::mDim.x) - 1 ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( static_cast<int>(Matrix<Type>::mDim.x) - 1 ).arg( pAxis.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
		
	Type* n = pAxis.c;
	Type* m = Matrix<Type>::m;
	Type phi = pAngle;
	
	Type cosphi = cos(phi);
	Type sinphi = sin(phi);
	
	switch(Matrix<Type>::mDim.x)
	{
		case 3 : // 2D
		
			m[0] = cosphi;
			m[1] = -sinphi;
			m[3] = sinphi;
			m[4] = cosphi;			
	
			break;
			
		case 4 : // 3D
		
			m[0] = n[0] * n[0] + (1.0 - n[0] * n[0]) * cosphi;
			m[1] = n[0] * n[1] - n[0] * n[1] * cosphi - n[2] * sinphi;
			m[2] = n[0] * n[2] - n[0] * n[2] * cosphi + n[1] * sinphi;
			m[4] = n[0] * n[1] - n[0] * n[1] * cosphi + n[2] * sinphi; 
			m[5] = n[1] * n[1] + (1.0 - n[1] * n[1]) * cosphi;
			m[6] = n[1] * n[2] - n[1] * n[2] * cosphi - n[0] * sinphi; 
			m[8] = n[0] * n[2] - n[0] * n[2] * cosphi - n[1] * sinphi; 
			m[9] = n[1] * n[2] - n[1] * n[2] * cosphi + n[0] * sinphi;
			m[10] = n[2] * n[2] + (1.0 - n[2] * n[2]) * cosphi;	
			
			break;
			
		case 5 : // 4D
		
			m[0] = n[0] * n[0] + (1 - n[0] * n[0]) * cosphi;
			m[1] = n[0] * n[1] - n[0] * n[1] * cosphi - (n[2] + n[3]) * sinphi; 
			m[2] = n[0] * n[2] - n[0] * n[2] * cosphi - (-n[1] + n[3]) * sinphi; 
			m[3] = n[0] * n[3] - n[0] * n[3] * cosphi - (-n[1] - n[2]) * sinphi;
			m[5] = n[0] * n[1] - n[0] * n[1] * cosphi - (-n[2] - n[3]) * sinphi; 
			m[6] = n[1] * n[1] + (1 - n[1] * n[1]) * cosphi; 
			m[7] = n[1] * n[2] - n[1] * n[2] * cosphi - (n[0] + n[3]) * sinphi; 
			m[8] = n[1] * n[3] - n[1] * n[3] * cosphi - (n[0] - n[2]) * sinphi; 
			m[10] = n[0] * n[2] - n[0] * n[2] * cosphi - (n[1] - n[3]) * sinphi; 
			m[11] = n[1] * n[2] - n[1] * n[2] * cosphi - (-n[0] - n[3]) * sinphi; 
			m[12] = n[2] * n[2] + (1 - n[2] * n[2]) * cosphi; 
			m[13] = n[2] * n[3] - n[2] * n[3] * cosphi - (n[0] + n[1]) * sinphi; 
			m[15] = n[0] * n[3] - n[0] * n[3] * cosphi - (n[1] + n[2]) * sinphi; 
			m[16] = n[1] * n[3] - n[1] * n[3] * cosphi - (-n[0] + n[2]) * sinphi; 
			m[17] = n[2] * n[3] - n[2] * n[3] * cosphi - (-n[0] - n[1]) * sinphi; 
			m[18] = n[3] * n[3] + (1 - n[3] * n[3]) * cosphi;
		
			break;
			
		case 6 : // 5D

			m[0] = n[0] * n[0] + (1 - n[0] * n[0]) * cosphi; 
			m[1] = n[0] * n[1] - n[0] * n[1] * cosphi - (n[2] + n[3] + n[4]) * sinphi; 
			m[2] = n[0] * n[2] - n[0] * n[2] * cosphi - (-n[1] + n[3] + n[4]) * sinphi; 
			m[3] = n[0] * n[3] - n[0] * n[3] * cosphi - (-n[1] - n[2] + n[4]) * sinphi; 
			m[4] = n[0] * n[4] - n[0] * n[4] * cosphi - (-n[1] - n[2] - n[3]) * sinphi; 
			m[6] = n[0] * n[1] - n[0] * n[1] * cosphi - (-n[2] - n[3] - n[4]) * sinphi; 
			m[7] = n[1] * n[1] + (1 - n[1] * n[1]) * cosphi;
			m[8] = n[1] * n[2] - n[1] * n[2] * cosphi - (n[0] + n[3] + n[4]) * sinphi;
			m[9] = n[1] * n[3] - n[1] * n[3] * cosphi - (n[0] - n[2] + n[4]) * sinphi; 
			m[10] = n[1] * n[4] - n[1] * n[4] * cosphi - (n[0] - n[2] - n[3]) * sinphi; 
			m[12] = n[0] * n[2] - n[0] * n[2] * cosphi - (n[1] - n[3] - n[4]) * sinphi; 
			m[13] = n[1] * n[2] - n[1] * n[2] * cosphi - (-n[0] - n[3] - n[4]) * sinphi; 
			m[14] = n[2] * n[2] + (1 - n[2] * n[2]) * cosphi; 
			m[15] = n[2] * n[3] - n[2] * n[3] * cosphi - (n[0] + n[1] + n[4]) * sinphi; 
			m[16] = n[2] * n[4] - n[2] * n[4] * cosphi - (n[0] + n[1] - n[3]) * sinphi; 
			m[18] = n[0] * n[3] - n[0] * n[3] * cosphi - (n[1] + n[2] - n[4]) * sinphi; 
			m[19] = n[1] * n[3] - n[1] * n[3] * cosphi - (-n[0] + n[2] - n[4]) * sinphi; 
			m[20] = n[2] * n[3] - n[2] * n[3] * cosphi - (-n[0] - n[1] - n[4]) * sinphi; 
			m[21] = n[3] * n[3] + (1 - n[3] * n[3]) * cosphi; 
			m[22] = n[3] * n[4] - n[3] * n[4] * cosphi - (n[0] + n[1] + n[2]) * sinphi; 
			m[24] = n[0] * n[4] - n[0] * n[4] * cosphi - (n[1] + n[2] + n[3]) * sinphi; 
			m[25] = n[1] * n[4] - n[1] * n[4] * cosphi - (-n[0] + n[2] + n[3]) * sinphi; 
			m[26] = n[2] * n[4] - n[2] * n[4] * cosphi - (-n[0] - n[1] + n[3]) * sinphi; 
			m[27] = n[3] * n[4] - n[3] * n[4] * cosphi - (-n[0] - n[1] - n[2]) * sinphi; 
			m[28] = n[4] * n[4] + (1 - n[4] * n[4]) * cosphi;
		
			break;
			
		case 7 : // 6D

			m[0] = n[0] * n[0] + (1 - n[0] * n[0]) * cosphi; 
			m[1] = n[0] * n[1] - n[0] * n[1] * cosphi - (n[2] + n[3] + n[4] + n[5]) * sinphi; 
			m[2] = n[0] * n[2] - n[0] * n[2] * cosphi - (-n[1] + n[3] + n[4] + n[5]) * sinphi; 
			m[3] = n[0] * n[3] - n[0] * n[3] * cosphi - (-n[1] - n[2] + n[4] + n[5]) * sinphi; 
			m[4] = n[0] * n[4] - n[0] * n[4] * cosphi - (-n[1] - n[2] - n[3] + n[5]) * sinphi; 
			m[5] = n[0] * n[5] - n[0] * n[5] * cosphi - (-n[1] - n[2] - n[3] - n[4]) * sinphi; 
			m[7] = n[0] * n[1] - n[0] * n[1] * cosphi - (-n[2] - n[3] - n[4] - n[5]) * sinphi; 
			m[8] = n[1] * n[1] + (1 - n[1] * n[1]) * cosphi; 
			m[9] = n[1] * n[2] - n[1] * n[2] * cosphi - (n[0] + n[3] + n[4] + n[5]) * sinphi; 
			m[10] = n[1] * n[3] - n[1] * n[3] * cosphi - (n[0] - n[2] + n[4] + n[5]) * sinphi; 
			m[11] = n[1] * n[4] - n[1] * n[4] * cosphi - (n[0] - n[2] - n[3] + n[5]) * sinphi; 
			m[12] = n[1] * n[5] - n[1] * n[5] * cosphi - (n[0] - n[2] - n[3] - n[4]) * sinphi; 
			m[14] = n[0] * n[2] - n[0] * n[2] * cosphi - (n[1] - n[3] - n[4] - n[5]) * sinphi;
			m[15] = n[1] * n[2] - n[1] * n[2] * cosphi - (-n[0] - n[3] - n[4] - n[5]) * sinphi; 
			m[16] = n[2] * n[2] + (1 - n[2] * n[2]) * cosphi; 
			m[17] = n[2] * n[3] - n[2] * n[3] * cosphi - (n[0] + n[1] + n[4] + n[5]) * sinphi; 
			m[18] = n[2] * n[4] - n[2] * n[4] * cosphi - (n[0] + n[1] - n[3] + n[5]) * sinphi; 
			m[19] = n[2] * n[5] - n[2] * n[5] * cosphi - (n[0] + n[1] - n[3] - n[4]) * sinphi; 
			m[21] = n[0] * n[3] - n[0] * n[3] * cosphi - (n[1] + n[2] - n[4] - n[5]) * sinphi; 
			m[22] = n[1] * n[3] - n[1] * n[3] * cosphi - (-n[0] + n[2] - n[4] - n[5]) * sinphi; 
			m[23] = n[2] * n[3] - n[2] * n[3] * cosphi - (-n[0] - n[1] - n[4] - n[5]) * sinphi; 
			m[24] = n[3] * n[3] + (1 - n[3]*n[3]) * cosphi; 
			m[25] = n[3] * n[4] - n[3] * n[4] * cosphi - (n[0] + n[1] + n[2] + n[5]) * sinphi; 
			m[26] = n[3] * n[5] - n[3] * n[5] * cosphi - (n[0] + n[1] + n[2] - n[4]) * sinphi; 
			m[28] = n[0] * n[4] - n[0] * n[4] * cosphi - (n[1] + n[2] + n[3] - n[5]) * sinphi; 
			m[29] = n[1] * n[4] - n[1] * n[4] * cosphi - (-n[0] + n[2] + n[3] - n[5]) * sinphi; 
			m[30] = n[2] * n[4] - n[2] * n[4] * cosphi - (-n[0] - n[1] + n[3] - n[5]) * sinphi; 
			m[31] = n[3] * n[4] - n[3] * n[4] * cosphi - (-n[0] - n[1] - n[2] - n[5]) * sinphi; 
			m[32] = n[4] * n[4] + (1 - n[4] * n[4]) * cosphi; 
			m[33] = n[4] * n[5] - n[4] * n[5] * cosphi - (n[0] + n[1] + n[2] + n[3]) * sinphi; 
			m[35] = n[0] * n[5] - n[0] * n[5] * cosphi - (n[1] + n[2] + n[3] + n[4]) * sinphi; 
			m[36] = n[1] * n[5] - n[1] * n[5] * cosphi - (-n[0] + n[2] + n[3] + n[4]) * sinphi; 
			m[37] = n[2] * n[5] - n[2] * n[5] * cosphi - (-n[0] - n[1] + n[3] + n[4]) * sinphi; 
			m[38] = n[3] * n[5] - n[3] * n[5] * cosphi - (-n[0] - n[1] - n[2] + n[4]) * sinphi; 
			m[39] = n[4] * n[5] - n[4] * n[5] * cosphi - (-n[0] - n[1] - n[2] - n[3]) * sinphi; 
			m[40] = n[5] * n[5] + (1 - n[5] * n[5]) * cosphi;
		
			break;
			
		default : // wrong D
			#ifdef _math_debug_
			throw MathException( "ERR_DIMENSION_MISMATCH", base::String("orientation not implemented for dimension %1 but only up to dimension %2").arg( static_cast<int>(Matrix<Type>::mDim.x) - 1 ).arg( 6 ), __FILE__, __FUNCTION__, __LINE__ );
			#endif
	}
 }
 
 template<class Type>
 Vector<Type> 
 MatrixH<Type>::multiply(const Vector<Type>& pVec) const throw (MathException)
 {
	try
	{
		return (*this) * pVec;
	}
	catch(MathException& e)
	{
		throw;
	}
 }

 template<class Type>
 Vector<Type> 
 MatrixH<Type>::multiply(const Vector<Type>& pVec, Type pHValue) const throw (MathException)
 {
	unsigned int vecDim = pVec.dim();
	
 	#ifdef _math_debug_
	if( vecDim != static_cast<int>(Matrix<Type>::mDim.y) - 1 ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( static_cast<int>(Matrix<Type>::mDim.y) - 1 ).arg( vecDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif	
	
	Vector<Type> outVector(vecDim);
	
	//top left submatrix
	for(size_t row = 0, mIndex = 0; row < vecDim; ++row, mIndex += 1)
	{
		for(size_t column = 0; column < vecDim; ++column, ++mIndex)
		{
			//std::cout << "row " << row << " col " << column << " index " << mIndex << " m_value " << Matrix<Type>::m[mIndex] << "\n";
		
			outVector[row] += Matrix<Type>::m[mIndex] * pVec[column];
		}
	}

	// right column
	for(size_t row = 0, mIndex = vecDim; row < vecDim; ++row, mIndex += Matrix<Type>::mDim.y)
	{
		//std::cout << "row " << row << " col " << vecDim << " index " << mIndex << " m_value " << Matrix<Type>::m[mIndex] << "\n";
	
		outVector[row] += Matrix<Type>::m[mIndex] * pHValue;
	}
	
	return outVector;
 }
  
 }
 
 }
 
 #endif