/**	\file iso_math_quaternion.h
 *	\class iso::math::Quaternion quaternion class
 *	\brief quaternion class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_math_quaternion_h_
#define _iso_math_quaternion_h_

#include "iso_math_types.h"
#include "iso_math_vector.h"
#include "iso_math_vector3.h"
#include "iso_math_vector4.h"
#include "iso_math_matrix4.h"
#include "iso_math_matrixh.h"

namespace iso
{

namespace math
{

template< class Type >
class Quaternion
{
public:
	static const Quaternion<Type> IDENTITY;

    union
    {
        struct
        {
            Type  w, x, y, z;
        };
        Type c[4];
    };

	/**
	\brief create identity quaternion
	*/
	Quaternion();

	/**
	\brief create quaternion directly from components
	\param pW w
	\param pX x
	\param pY y
	\param pZ z
	*/
	Quaternion(Type pW, Type pX, Type pY, Type pZ);

	/**
	\brief create quaternion directly from array
	\param pValues array	*/
	Quaternion(Type pValues[4]);
	
	/**
	\brief create quaternion directly from vector
	\param pVector vector4
	*/
	Quaternion(Vector4<Type> pVector);

	/**
	\brief create quaternion from axis-of-rotation and rotation angle
	\param pAxis axis of rotation
	\param pAngle rotation angle
	*/
	Quaternion(const Vector3<Type>& pAxis, Type pAngle);
	
	/**
	\brief create quaternion from rotation matrix.
	\param pMatrix rotation matrix
	*/
	Quaternion(const Matrix4<Type>& pMatrix);

	/**
	\brief create quaternion from minimum rotation from vector a into vector b.
	\param pVectorA vector a
	\param pVectorB vector b
	*/
	Quaternion(const Vector3<Type>& pVectorA, const Vector3<Type>& pVectorB);

	/**
	\brief create quaternion from euler orientation.
	\param pYaw yaw
	\param pPitch pitch
	\param pRoll roll
	*/	
	Quaternion(Type pYaw, Type pPitch, Type pRoll);

	/**
	\brief copy constructor
	\param pQuaternion quaternion
	*/	
	Quaternion( const Quaternion<Type>& pQuaternion );
	
	/**
	\brief copy constructor
	\param pQuaternion quaternion
	*/	
	template< class Type2> Quaternion( const Quaternion<Type2>& pQuaternion );	

	/**
	\brief destructor
	*/
	virtual ~Quaternion();

	Type dot( const Quaternion<Type>& pQuat ) const;
	
	static Type dot( const Quaternion<Type>& pQuat1, const Quaternion<Type>& pQuat2 );
	
	/**
	\brief assignment quaternion = quaternion
	\param pQuat quaternion
	\return quaternion
	*/
	const Quaternion<Type>& operator=( const Quaternion<Type>& pQuat );

	/**
	\brief assignment quaternion = quaternion
	\param pQuat quaternion
	\return quaternion
	*/
	template <class Type2> const Quaternion<Type>& operator=( const Quaternion<Type2>& pQuat );
	
	/**
	\brief assignment quaternion = vector
	\param pVector vector4
	\return quaternion
	*/	
	const Quaternion<Type>& operator=( const Vector4<Type>& pVector ); 

	/**
	\brief scale quaternion
	\param pValue value
	*/	
	const Quaternion<Type>& scale( Type pValue );

	/**
	\brief set quaternion directly from components
	\param pW w
	\param pX x
	\param pY y
	\param pZ z
	*/	
	void set( Type pW, Type pX, Type pY, Type pZ );
	
	/**
	\brief set quaternion directly from array
	\param pValues array
	*/		
	void set(const Type pValues[4]);

	/**
	\brief set quaternion directly from vector
	\param pVector vector4
	*/		
	void set(const Vector4<Type> pVector);
	
	/**
	\brief set quaternion from axis-of-rotation and rotation angle
	\param pAxis axis of rotation
	\param pAngle rotation angle
	*/
	void set(const Vector3<Type>& pAxis, Type pAngle);

	/**
	\brief set quaternion from rotation matrix.
	\param pMatrix rotation matrix
	*/
	void set(const Matrix4<Type>& pMatrix);

	/**
	\brief set quaternion from minimum rotation from vector a into vector b.
	\param pVectorA vector a
	\param pVectorB vector b
	*/	
	void set2(const Vector3<Type> & pVectorA, const Vector3<Type>& pVectorB);

	/**
	\brief set quaternion from minimum rotation from vector a into vector b.
	\param pVectorA vector a
	\param pVectorB vector b
	*/	
	void set(const Vector3<Type> & pVectorA, const Vector3<Type>& pVectorB);
	
	/**
	\brief set quaternion from euler orientation.
	\param pYaw yaw
	\param pPitch pitch
	\param pRoll roll
	*/	
	void set(Type pYaw, Type pPitch, Type pRoll);

	/**
	\brief get axis-of-rotation and rotation angle from quaternion
	\param pAxis axis of rotation
	\param pAngle rotation angle
	*/
	void get(Vector3<Type>& pAxis, Type& pAngle) const;
	
	/**
	\brief get rotation matrix from quaternion
	\param pMatrix rotation matrix
	*/
	void get(Matrix4<Type>& pMatrix) const;

	/**
	\brief get rotation matrix from quaternion
	\param pMatrix rotation matrix
	*/
	void get(MatrixH<Type>& pMatrix) const;

	/**
	\brief euler orientation from quaternion
	\param pYaw yaw
	\param pPitch pitch
	\param pRoll roll
	*/	
	void get(Type& pYaw, Type& pPitch, Type& pRoll) const;
	
	/**
	\brief invert
	\return quaternion
	
	original quaternion modified
	*/
	const Quaternion<Type>& invert();
	
	/**
	\brief get inverse
	\return quaternion
	
	original quaternion not modified
	*/
	Quaternion<Type> getInverse() const;

	/**
	\brief conjugate
	\return quaternion
	
	original quaternion modified
	*/
	const Quaternion<Type>& conjugate();

	/**
	\brief get conjugate
	\return quaternion
	
	original quaternion not modified
	*/
	Quaternion<Type> getConjugate() const;

	/**
	\brief quaternion += quaternion
	\param pQuat quaternion
	\return quaternion
	*/	
	const Quaternion<Type>& operator += (const Quaternion<Type>& pQuat);
	  
	/**
	\brief quaternion -= quaternion
	\param pQuat quaternion
	\return quaternion
	*/	
	const Quaternion<Type>& operator -= (const Quaternion<Type>& pQuat);

	/**
	\brief quaternion *= scalar
	\param pValue value
	\return quaternion
	
	scale quaternion
	*/	
	const Quaternion<Type>& operator*= (Type pValue);

	/**
	\brief quaternion *= quaternion
	\param pQuat quaternion
	\return quaternion
	*/
	const Quaternion<Type>& operator*= (const Quaternion<Type>& pQuat);

	/**
	\brief quaternion /= quaternion
	\param pQuat quaternion
	\return quaternion
	*/
	const Quaternion<Type>& operator/= (const Quaternion<Type>& pQuat);

	/**
	\brief quaternion * scalar
	\param pValue value
	\return quaternion
	*/	
	Quaternion<Type> operator* (const Type& pValue) const;

	/**
	\brief quaternion + quaternion
	\param pQuat quaternion
	\return quaternion
	*/	
	Quaternion<Type> operator + (const Quaternion<Type>& pQuat) const;

	/**
	\brief quaternion - quaternion
	\param pQuat quaternion
	\return quaternion
	*/		  
	Quaternion<Type> operator - (const Quaternion<Type>& pQuat) const;

	/**
	\brief quaternion * quaternion
	\param pQuat quaternion
	\return quaternion
	*/	
	Quaternion<Type> operator* (const Quaternion<Type>& pQuat) const;

	/**
	\brief quaternion / quaternion
	\param pQuat quaternion
	\return quaternion
	*/	
	Quaternion<Type> operator / (Quaternion<Type>& pQuat) const;

	/**
	\brief vector rotation
	\param pVector vector3
	\return vector3
	*/	
	Vector3<Type> operator* (const Vector3<Type>& pVector) const;
	
	/**
	\brief vector rotation
	\param pVector vector3
	\return vector3
	*/
	Vector3<Type> rotVec(const Vector3<Type>& pVector) const;
		
	/**
	\brief interpolate between two orientations
	\param pRot0 first quaternion based orientiation
	\param pRot1 second quaternion based orientiation
	\param pT interpolation ( 0.0 - 1.0 )
	\return quaternion
	*/
	static Quaternion<Type> slerp(const Quaternion<Type>& pRot0, const Quaternion<Type>& pRot1, Type pT);

	/**
	\brief quaternion == quaternion
	\param pQuat quaternion
	\return bool
	*/	
	bool operator == (const Quaternion<Type>& pQuat) const;  
	
	/**
	\brief quaternion != quaternion
	\param pQuat quaternion
	\return bool
	*/	
	bool operator != (const Quaternion<Type>& pQuat) const;

	/**
	\brief return norm
	\return norm
	
	sqrt(w^2 + x^2 + y^2 + z^2);
	*/
	Type norm() const;

	/**
	\brief normalise
	\return quaternion
	*/
	const Quaternion<Type>& normalise();
	
	friend std::ostream& operator << ( std::ostream& os, const Quaternion<Type> pQuat )
    {
        const std::ios::fmtflags flags = os.flags();
        const int prec  = os.precision();

        os.setf( std::ios::right, std::ios::adjustfield );
        os.precision( 5 );
		os << "[ " << std::setw(10) << pQuat.w << std::setw(10) << pQuat.x << std::setw(10) << pQuat.y << std::setw(10) << pQuat.z << "]";
        os.precision( prec );
        os.setf( flags );
        return os;
    };  	
};

typedef Quaternion<float>  QuaternionF;
typedef Quaternion<double> QuaternionD;

template< class Type >
const Quaternion<Type>
Quaternion<Type>::IDENTITY(static_cast<Type>(1.0), static_cast<Type>(0.0), static_cast<Type>(0.0), static_cast<Type>(0.0));

template< class Type >
Quaternion<Type>::Quaternion()
	: w( IDENTITY.w )
	, x( IDENTITY.x )
	, y( IDENTITY.y )
	, z( IDENTITY.z )
{}

template< class Type >
Quaternion<Type>::Quaternion(Type pW, Type pX, Type pY, Type pZ)
	: w( pW )
	, x( pX )
	, y( pY )
	, z( pZ )
{}

template< class Type >
Quaternion<Type>::Quaternion(Type pValues[4])
	: w( pValues[0] )
	, x( pValues[1] )
	, y( pValues[2] )
	, z( pValues[3] )
{}

template< class Type >	
Quaternion<Type>::Quaternion(Vector4<Type> pVector)
	: w( pVector[0] )
	, x( pVector[1] )
	, y( pVector[2] )
	, z( pVector[3] )
{}

template< class Type >
Quaternion<Type>::Quaternion(const Vector3<Type>& pAxis, Type pAngle)
{
	this->set(pAxis, pAngle);
}

template< class Type >
Quaternion<Type>::Quaternion(const Matrix4<Type>& pMatrix)
{
	this->set(pMatrix);
}

template< class Type >
Quaternion<Type>::Quaternion(const Vector3<Type>& pVectorA, const Vector3<Type>& pVectorB)
{
	this->set(pVectorA, pVectorB);
}

template< class Type >
Quaternion<Type>::Quaternion(Type pYaw, Type pPitch, Type pRoll)
{
	set(pYaw, pPitch, pRoll);
}


template< class Type >
Quaternion<Type>::Quaternion( const Quaternion<Type>& pQuaternion )
	: w(pQuaternion.w)
	, x(pQuaternion.x)
	, y(pQuaternion.y)
	, z(pQuaternion.z)
{}

template< class Type >	
template< class Type2 > 
Quaternion<Type>::Quaternion( const Quaternion<Type2>& pQuaternion )
	: w( static_cast<Type>( pQuaternion.w) )
	, x( static_cast<Type>( pQuaternion.x) )
	, y( static_cast<Type>( pQuaternion.y) )
	, z( static_cast<Type>( pQuaternion.z) )
{}

template< class Type >
Quaternion<Type>::~Quaternion()
{}

template< class Type >
Type 
Quaternion<Type>::dot( const Quaternion<Type>& pQuat ) const
{
	return x * pQuat.x + y * pQuat.y + z * pQuat.z + w * pQuat.w;
}

template< class Type >	
Type 
Quaternion<Type>::dot( const Quaternion<Type>& pQuat1, const Quaternion<Type>& pQuat2 )
{
	return pQuat1.x * pQuat2.x + pQuat1.y * pQuat2.y + pQuat1.z * pQuat2.z + pQuat1.w * pQuat2.w;
}


template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::operator=( const Quaternion<Type>& pQuat )
{
	w = pQuat.w;
	x = pQuat.x;
	y = pQuat.y;
	z = pQuat.z;
	
	return *this;
}

template< class Type >
template< class Type2 > 
const Quaternion<Type>& 
Quaternion<Type>::operator=( const Quaternion<Type2>& pQuat )
{
	w = static_cast<Type>( pQuat.w );
	x = static_cast<Type>( pQuat.x );
	y = static_cast<Type>( pQuat.y );
	z = static_cast<Type>( pQuat.z );
	
	return *this;
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::operator=( const Vector4<Type>& pVector )
{
	w = pVector[0];
	x = pVector[1];
	y = pVector[2];
	z = pVector[3];
	
	return *this;
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::scale( Type pValue )
{
	(*this) *= pValue;
	
	return *this;
}

template< class Type >
void 
Quaternion<Type>::set(Type pW, Type pX, Type pY, Type pZ)
{
	w = pW;
	x = pX;
	y = pY;
	z = pZ;
}

template< class Type >
void 
Quaternion<Type>::set(const Type pValues[4])
{
	w = pValues[0];
	x = pValues[1];
	y = pValues[2];
	z = pValues[3];	
}

template< class Type >
void 
Quaternion<Type>::set(const Vector4<Type> pVector)
{
	w = pVector[0];
	x = pVector[1];
	y = pVector[2];
	z = pVector[3];
}

template< class Type >
void 
Quaternion<Type>::set(const Vector3<Type>& pAxis, Type pAngle)
{
	Type tmp1,tmp2;
	tmp1 = pAxis.length();
	
	assert(tmp1 != 0 && "Quaternion Axis is zero!");
	
	tmp2 = sin(pAngle * 0.5) / tmp1;
	
	x = pAxis.x * tmp2;
	y = pAxis.y * tmp2;
	z = pAxis.z * tmp2;
	w = cos( pAngle * 0.5 );
}

template< class Type >
void
Quaternion<Type>::set(const Matrix4<Type>& pMatrix)
{
	Type scalerow = pMatrix.m[0] + pMatrix.m[5] + pMatrix.m[10];

	if(scalerow > 0.0)
	{
		Type s = sqrt(scalerow + pMatrix.m[15]);

		w = s * 0.5;

		s = 0.5 / s;

		x = (pMatrix.m[6] - pMatrix.m[9]) * s;
		y = (pMatrix.m[8] - pMatrix.m[2]) * s;
		z = (pMatrix.m[1] - pMatrix.m[4]) * s;
	}
	else
	{
		int i = 0;
		if (pMatrix.m[5] > pMatrix.m[0]) i = 1;
		if (pMatrix.m[10] > pMatrix.m[5 * i]) i = 2;

		int j = (i+1)%3;
		int k = (j+1)%3;

		
		Type s = sqrt((pMatrix.m[5 * i] - (pMatrix.m[5 * j] + pMatrix.m[5 * k])) + pMatrix.m[15]);

		c[i] = s * 0.5f;

		s = 0.5 / s;

		int i4 = i * 4;
		int j4 = j * 4;
		int k4 = k * 4;

		c[3] = (pMatrix.m[j4 + k] - pMatrix.m[k4 + j]) * s;
		c[j] = (pMatrix.m[i4 + j] + pMatrix.m[j4 + i]) * s;
		c[k] = (pMatrix.m[i4 + k] + pMatrix.m[k4 + i]) * s;
	}

	if (pMatrix(3, 3) != 1.0)
	{
		operator*=(1.0 / std::sqrt(pMatrix.m[15]));
	}
}

template< class Type >
void 
Quaternion<Type>::set(const Vector3<Type> & pVectorA, const Vector3<Type>& pVectorB)
{
	Vector3<Type> vectorA(pVectorA);
	Vector3<Type> vectorB(pVectorB);

	vectorA.normalise();
	vectorB.normalise();
	
	Vector3<Type> vecProd = Vector3<Type>::cross(vectorA, vectorB);
	
	Type dotProd = Vector3<Type>::dot(vectorA, vectorB);
	
	Type s = sqrt((1.0 + dotProd) * 2.0);
	
	if(s < 0.01)
	{
		w = 1.0;
		x = 0.0;
		y = 0.0;
		z = 0.0;
	}
	else
	{
		w = s / 2.0;
		s = 1.0 / s;
		
		x = vecProd.x * s;
		y = vecProd.y * s;
		z = vecProd.z * s;
	}
}

template< class Type >
void 
Quaternion<Type>::set2(const Vector3<Type> & pVectorA, const Vector3<Type>& pVectorB)
{
	Vector3<Type> vectorA(pVectorA);
	Vector3<Type> vectorB(pVectorB);

	vectorA.normalise();
	vectorB.normalise();
	
	Vector3<Type> vecProd = Vector3<Type>::cross(vectorA, vectorB);
	
	if(vecProd.length() < 0.01)
	{
		if( pVectorA.z >= pVectorA.x && pVectorA.z >= pVectorA.y ) vecProd.set( 1.0, 0.0, 0.0 );
		else vecProd.set( 0.0, 0.0, 1.0 );
	}
	else
	{
		vecProd.normalise();
	}
	
	Type dotProd = Vector3<Type>::dot(vectorA, vectorB);
	
	set(vecProd, dotProd);
}

template< class Type >
void 
Quaternion<Type>::set(Type pYaw, Type pPitch, Type pRoll)
{
	// Gainer, 1972
	// calculate trig identities
	pYaw *= 0.5;
	pPitch *= 0.5;
	pRoll *= 0.5;

	Type c1 = cos(pRoll);
	Type c2 = cos(pYaw);
	Type c3 = cos(pPitch);

	Type s1 = sin(pRoll);
	Type s2 = sin(pYaw);
	Type s3 = sin(pPitch);

	Type c2c1 = c2 * c1;
	Type s2s1 = s2 * s1;

	x = s3 * c2c1 - c3 * s2s1;
	y = c3 * s2 * c1 + s3 * c2 * s1;
	z = c3 * c2 * s1 - s3 * s2 * c1;
	w = c3 * c2c1 + s3 * s2s1;
}

template< class Type >
void 
Quaternion<Type>::get(Vector3<Type>& pAxis, Type& pAngle) const
{
	if((w >= -1.0) && (w <= 1.0))
	{
		pAngle = acos(w) * 2.0;

		Type scale = sin(pAngle * 0.5);

		if(scale != 0.0f)
		{
			scale = 1.0 / scale;
		
			pAxis.x = x * scale;
			pAxis.y = y * scale;
			pAxis.z = z * scale;

			return;
		}
	}

	// Quat can't be converted to axis and rotation angle, so we just
	// set up values to indicate this.
	pAxis.set(0.0, 0.0, 1.0);
	pAngle = 0.0;
}

template< class Type >
void 
Quaternion<Type>::get(Matrix4<Type>& pMatrix) const
{
	// calculate coefficients
	Type xx2 = x * x * 2.0; 
	Type yy2 = y * y * 2.0; 
	Type zz2 = z * z * 2.0;
	Type xy2 = x * y * 2.0;
	Type xz2 = x * z * 2.0;
	Type xw2 = x * w * 2.0;
	Type yz2 = y * z * 2.0;
	Type yw2 = y * w * 2.0;
	Type zw2 = z * w * 2.0;
	
	// First row
	pMatrix.m[ 0] = 1.0 - yy2 - zz2;
	pMatrix.m[ 4] = xy2 + zw2;
	pMatrix.m[ 8] = xz2 - yw2;
	pMatrix.m[ 12] = 0.0;
	
	// Second row
	pMatrix.m[ 1] = xy2 - zw2;
	pMatrix.m[ 5] = 1.0 - xx2 - zz2;
	pMatrix.m[ 9] = yz2 + xw2;
	pMatrix.m[ 13] = 0.0;

	// Third row
	pMatrix.m[ 2] = xz2 + yw2;
	pMatrix.m[ 6] = yz2 - xw2;
	pMatrix.m[10] = 1.0 - xx2 - yy2;
	pMatrix.m[14] = 0.0;

	// Fourth row
	/*
	pMatrix.m[3] = 0;
	pMatrix.m[7] = 0;
	pMatrix.m[11] = 0;
	pMatrix.m[15] = 1.0;
	*/
}

template< class Type >
void 
Quaternion<Type>::get(MatrixH<Type>& pMatrix) const
{
	// calculate coefficients
	Type xx2 = x * x * 2.0; 
	Type yy2 = y * y * 2.0; 
	Type zz2 = z * z * 2.0;
	Type xy2 = x * y * 2.0;
	Type xz2 = x * z * 2.0;
	Type xw2 = x * w * 2.0;
	Type yz2 = y * z * 2.0;
	Type yw2 = y * w * 2.0;
	Type zw2 = z * w * 2.0;
	
	// First row
	pMatrix.m[ 0] = 1.0 - yy2 - zz2;
	pMatrix.m[ 4] = xy2 + zw2;
	pMatrix.m[ 8] = xz2 - yw2;
	pMatrix.m[ 12] = 0.0;
	
	// Second row
	pMatrix.m[ 1] = xy2 - zw2;
	pMatrix.m[ 5] = 1.0 - xx2 - zz2;
	pMatrix.m[ 9] = yz2 + xw2;
	pMatrix.m[ 13] = 0.0;

	// Third row
	pMatrix.m[ 2] = xz2 + yw2;
	pMatrix.m[ 6] = yz2 - xw2;
	pMatrix.m[10] = 1.0 - xx2 - yy2;
	pMatrix.m[14] = 0.0;

	// Fourth row
	/*
	pMatrix.m[3] = 0;
	pMatrix.m[7] = 0;
	pMatrix.m[11] = 0;
	pMatrix.m[15] = 1.0;
	*/
}

template< class Type >
void 
Quaternion<Type>::get(Type& pYaw, Type& pPitch, Type& pRoll) const
{
//	Type xx = x*x;
//	Type yy = y*y;
//	Type zz = z*z;
//	Type ww = w*w;

//	pYaw   = asin( -2.0 * ( x * z-w * y ) );
//	pPitch = atan( 2.0 * ( w * x + y * z ) / ( ww - xx - yy+ zz ) );
//	pRoll  = atan( 2.0 * ( x * y + w * z ) / ( ww + xx - yy- zz ) );
	
	// x: q0
	// y: q1
	// z: q2
	// w: q3
	
	pRoll = atan2( 2.0 * ( x * y + z * w  ), 1.0 - 2.0 * ( y * y + z * z) );
	pPitch = asin( 2.0 * ( x * z - w * y ) );
	pYaw = atan2( 2.0 * ( x * w + y * z ), 1.0 - 2.0 * ( z * z + w * w ) );
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::invert()
{
	Type inv = 1.0 / Vector<Type>::length();

	w =  w * inv;
	x = -x * inv;
	y = -y * inv;
	z = -z * inv;

	return *this;
}

template< class Type >
Quaternion<Type> 
Quaternion<Type>::getInverse() const
{
	Type inv = 1.0 / (w * w + x * x + y * y + z * z);
	
	return Quaternion( w * inv, -x * inv, -y * inv, -z * inv );
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::conjugate()
{
	x = -x;
	y = -y;
	z = -z;

	return *this;
}

template< class Type >
Quaternion<Type> 
Quaternion<Type>::getConjugate() const
{
	return Quaternion(w, -x, -y, -z);
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::operator*= (Type pValue)
{
	w *= pValue;	
	x *= pValue;
	y *= pValue;
	z *= pValue;

	return *this;
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::operator+= (const Quaternion<Type>& pQuat)
{
	w += pQuat.w;
	x += pQuat.x;
	y += pQuat.y;
	z += pQuat.z;

	return (*this);
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::operator-= (const Quaternion<Type>& pQuat)
{
	w -= pQuat.w;
	x -= pQuat.x;
	y -= pQuat.y;
	z -= pQuat.z;

	return (*this);
}

template< class Type >
const Quaternion<Type>& 
Quaternion<Type>::operator*= (const Quaternion<Type>& pQuat)
{
	Type nw = w * pQuat.w - x * pQuat.x - y * pQuat.y - z * pQuat.z;
	Type nx = w * pQuat.x + x * pQuat.w + y * pQuat.z - z * pQuat.y;
	Type ny = w * pQuat.y - x * pQuat.z + y * pQuat.w + z * pQuat.x;
	Type nz = w * pQuat.z + x * pQuat.y - y * pQuat.x + z * pQuat.w;

	w = nw;	
	x = nx;
	y = ny;
	z = nz;

	return *this;
}

template<class Type> 
const Quaternion<Type>& 
Quaternion<Type>::operator /= (const Quaternion<Type>& pQuat)
{
  (*this) = (*this)*pQuat.getInverse();
  return (*this);
}

template< class Type >
Quaternion<Type> 
Quaternion<Type>::operator* (const Type& pValue) const
{
	return Quaternion<Type>( w * pValue, x * pValue, y * pValue, z * pValue );
}

template< class Type >
Quaternion<Type> 
Quaternion<Type>::operator + (const Quaternion<Type>& pQuat) const
{
	return Quaternion( w + pQuat.w, x + pQuat.x, y + pQuat.y, z + pQuat.z );
}

template< class Type >
Quaternion<Type> 
Quaternion<Type>::operator - (const Quaternion<Type>& pQuat) const
{
	return Quaternion( w - pQuat.w, x - pQuat.x, y - pQuat.y, z - pQuat.z );
}

template< class Type >
Quaternion<Type> 
Quaternion<Type>::operator* (const Quaternion<Type>& pQuat) const
{
	Quaternion<Type> quat;
	
	quat.w = w * pQuat.w - x * pQuat.x - y * pQuat.y - z * pQuat.z;
	quat.x = w * pQuat.x + x * pQuat.w + y * pQuat.z - z * pQuat.y;
	quat.y = w * pQuat.y - x * pQuat.z + y * pQuat.w + z * pQuat.x;
	quat.z = w * pQuat.z + x * pQuat.y - y * pQuat.x + z * pQuat.w;
	
	return quat;
}

template< class Type >
Quaternion<Type> 
Quaternion<Type>::operator / (Quaternion<Type>& pQuat) const
{
	return ( (*this) * ( pQuat.getInverse() ) );
}

template< class Type >
Vector3<Type> 
Quaternion<Type>::operator* (const Vector3<Type>& pVector) const
{
	Vector3<Type> uv, uuv;
	Vector3<Type> qvec(x, y, z);
	uv = qvec.cross(pVector);
	uuv = qvec.cross(uv);
	uv *= (2.0f * w);
	uuv *= 2.0f;

	return pVector + uv + uuv;

/*
	Vector3<Type> vector;

	Type qwx = w * x; 
	Type qwy = w * y; 
	Type qwz = w * z;
	Type qxy = x * y; 
	Type qxz = x * z; 
	Type qyz = y * z;

	vector.x = 2.0 * ( pVector.y * ( -qwz + qxy ) + pVector.y * ( qwy + qxz ) );
	vector.y = 2.0 * ( pVector.x * ( qwz + qxy ) + pVector.z * ( -qwx + qyz ) );
	vector.z = 2.0 * ( pVector.x * ( -qwy + qxz ) + pVector.y * ( qwx + qyz ) );

	Type qww = w * w; 
	Type qxx = x * x; 
	Type qyy = y * y; 
	Type qzz = z * z;

	vector.x += pVector.x * ( qww + qxx - qyy - qzz );
	vector.y += pVector.y * ( qww - qxx + qyy - qzz );
	vector.z += pVector.z * ( qww - qxx - qyy + qzz );

	return vector;
*/
}

template< class Type >
Vector3<Type>
Quaternion<Type>::rotVec(const Vector3<Type>& pVector) const
{
	Vector3<Type> vector;

	Type qwx = w * x; 
	Type qwy = w * y; 
	Type qwz = w * z;
	Type qxy = x * y; 
	Type qxz = x * z; 
	Type qyz = y * z;

	vector.x = 2.0 * ( pVector.y * ( -qwz + qxy ) + pVector.y * ( qwy + qxz ) );
	vector.y = 2.0 * ( pVector.x * ( qwz + qxy ) + pVector.z * ( -qwx + qyz ) );
	vector.z = 2.0 * ( pVector.x * ( -qwy + qxz ) + pVector.y * ( qwx + qyz ) );

	Type qww = w * w; 
	Type qxx = x * x; 
	Type qyy = y * y; 
	Type qzz = z * z;

	vector.x += pVector.x * ( qww + qxx - qyy - qzz );
	vector.y += pVector.y * ( qww - qxx + qyy - qzz );
	vector.z += pVector.z * ( qww - qxx - qyy + qzz );

	return vector;
}

//! Interpolates along the shortest path between the two rotation positions (from \a rot0 to \a rot1, 0<= t <= 1).
template< class Type >
Quaternion<Type> 
Quaternion<Type>::slerp(const Quaternion<Type>& pRot0, const Quaternion<Type>& pRot1, Type pT)
{
	Quaternion<Type> rot = pRot1;
	Type dotVal = pRot0.dot(rot);

	// Find the correct direction of the interpolation.
	if(dotVal < 0.0f)
	{
		dotVal = -dotVal;
		rot *= 1.0;
	}

	// fallback to linear interpolation, in case we run out of floating
	// point precision
	Type scale0 = 1.0 - pT;
	Type scale1 = pT;

	if ((1.0 - dotVal) > 1E-3)
	{
		Type angle = acos(dotVal);
		Type sinangle = sin(angle);
		if (sinangle > 1E-3)
		{
			// calculate spherical interpolation
			scale0 = sin((1.0 - pT) * angle) / sinangle;
			scale1 = sin(pT * angle) / sinangle;
		}
	}

	return pRot0 * scale0 + rot * scale1;
}

template< class Type >
bool
Quaternion<Type>::operator== (const Quaternion<Type>& pQuat) const
{
	return ( w == pQuat.w && x == pQuat.x && y == pQuat.y && z == pQuat.z) ? true : false;
}
	
template< class Type >
bool
Quaternion<Type>::operator!= (const Quaternion<Type>& pQuat) const
{
	return ( w != pQuat.w || x != pQuat.x || y != pQuat.y || z != pQuat.z) ? true : false;
}

template< class Type >
Type 
Quaternion<Type>::norm() const
{
	return sqrt( w*w + x*x + y*y + z*z );
}

template< class Type >
const Quaternion<Type>&
Quaternion<Type>::normalise()
{
	Type length = norm();
	
	if(length == 0.0) (*this) = IDENTITY;
	else scale(1.0 / length);
	
	return *this;
}

}

}

#endif



	
