/**	\file iso_math_vector2.h
 *	\class iso::math::Vector2 2D vector class
 *	\brief 2D vector class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_math_vector2_h_
#define _iso_math_vector2_h_

#include "iso_math_vector.h"

namespace iso
{

namespace math
{

template< class Type > 
class Vector2 : public Vector<Type>
{
public:
 	/**
	\brief zero vector
	*/
	static const Vector2<Type> ZERO;

	/**
	\brief one vector
	*/
	static const Vector2<Type> ONE;

	/**
	\brief references to vector components
	*/
    Type &x, &y;

	/**
	\brief create zero vector
	*/
    Vector2();
	
	/**
	\brief create vector with identical vector components
	\param pValue value
	*/
    Vector2( Type pValue );
	
	/**
	\brief create vector from parameters
	\param p0 0. value
	\param p1 1. value
	*/
	Vector2( Type p0, Type p1 );
	
	/**
	\brief create vector from array of values
	\param pValues array of values (must be of correct size)
	*/
	Vector2( Type* pValues );
	
	/**
	\brief create vector from vector
	\param pVector vector of size 2
	*/
	Vector2( const Vector2<Type>& pVector );

	/**
	\brief create vector from vector
	\param pVector vector of size 2
	*/
	template< class Type2 > Vector2( const Vector2<Type2>& pVector );
	
	/**
	\brief create vector from vector
	\param pVector vector
	\exception MathException pVector dimension not 2
	*/
	Vector2( const Vector<Type>& pVector ) throw (MathException);

	/**
	\brief create vector from vector
	\param pVector vector
	\exception MathException pVector dimension not 2
	*/
	template< class Type2 >Vector2( const Vector<Type2>& pVector ) throw (MathException);

	/**
	\brief destructor
	*/
    virtual ~Vector2();

	/**
	\brief set dimension of vector
	\param pDim new vector dimension
	\exception MathException illegal function
	*/
	virtual void setDim(unsigned int pDim) throw (MathException);
	
	/**
	\brief set vector components from parameters
	\param p0 0. value
	\param p1 1. value
	*/
	void set( Type p0, Type p1 );
	
	/**
	\brief set vector components from vector
	\param pVector of size 2
	*/
	const Vector2<Type>& operator=( const Vector2<Type>& pVector ); 
	
	/**
	\brief set vector components from vector
	\param pVector
	\exception MathException pVector dimension not 2
	*/
	const Vector2<Type>& operator=( const Vector<Type>& pVector ) throw (MathException); 

	/**
	\brief vector cross product
	\param pVector second vector
	\return cross product
	
	2d version according to Geometric Tools for Computer Graphics, Schneider & Eberly, 2003
	*/	
    Type cross( const Vector2& pVector ) const;
	
	/**
	\brief vector cross product
	\param pVector1 first vector
	\param pVector2 second vector
	\return cross product
	
	2d version according to Geometric Tools for Computer Graphics, Schneider & Eberly, 2003
	*/		
    static Type cross( const Vector2<Type> &pVector1, const Vector2<Type> &pVector2);
	
	/**
	\brief rotate vector around angle
	\param pTheta angle (in radians)
	*/
	void rotate( Type pTheta ); 
	
	/**
	\brief print vector information
	\param pOstream output stream
	\param pVector vector
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const Vector2<Type>& pVector)
    {
		pOstream << pVector.info().toStdString();
		return pOstream;
    }; 
};

typedef Vector2<int>    Vector2I;
typedef Vector2<float>  Vector2F;
typedef Vector2<double> Vector2D;

 template<class Type>
 const Vector2<Type> 
 Vector2<Type>::ZERO(0.0, 0.0);

 template<class Type>
 const Vector2<Type> 
 Vector2<Type>::ONE(1.0, 1.0);

template< class Type > 
Vector2<Type>::Vector2()
	: Vector<Type>( 2 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{}

template< class Type > 
Vector2<Type>::Vector2( Type pValue )
	: Vector<Type>( 2, pValue )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{}

template< class Type > 
Vector2<Type>::Vector2( Type p0, Type p1 )
	: Vector<Type>( 2 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{
	x = p0;
	y = p1;
}

template< class Type > 
Vector2<Type>::Vector2( Type* pValues )
	: Vector<Type>( 2, pValues )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{}

template< class Type > 
Vector2<Type>::Vector2( const Vector2<Type>& pVector )
	: Vector<Type>( pVector )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{}

template< class Type > 
template< class Type2 > 
Vector2<Type>::Vector2( const Vector2<Type2>& pVector )
	: Vector<Type>( pVector )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{}

template< class Type > 
Vector2<Type>::Vector2( const Vector<Type>& pVector ) throw (MathException)
	: Vector<Type>( 2 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	x = pVector.c[0];
	y = pVector.c[1];
}

template< class Type >
template< class Type2 > 
Vector2<Type>::Vector2( const Vector<Type2>& pVector ) throw (MathException)
	: Vector<Type>( 2 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	x = static_cast<Type>( pVector.c[0] );
	y = static_cast<Type>( pVector.c[1] );
}

template< class Type > 
Vector2<Type>::~Vector2()
{}

template < class Type > 	
void 
Vector2< Type >::setDim(unsigned int pDim) throw (MathException)
{
	#ifdef _math_debug_
	throw MathException( "ERR_VECTOR_ERROR", base::String("illegal vector function"), __FILE__, __FUNCTION__, __LINE__ );
	#endif
}

template< class Type > 
void 
Vector2<Type>::set( Type p0, Type p1 )
{
	x = p0;
	y = p1;
}

template< class Type > 
const Vector2<Type>& 
Vector2<Type>::operator=( const Vector2<Type>& pVector )
{
	x = pVector.x;
	y = pVector.y;
	
	return (*this);
}

template< class Type > 
const Vector2<Type>& 
Vector2<Type>::operator=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	x = pVector.c[0];
	y = pVector.c[1];
	
	return (*this);
}

template< class Type >
Type 
Vector2<Type>::cross( const Vector2& pVector ) const
{
	return x * pVector.y - pVector.x * y;
}
		
template< class Type >
Type 
Vector2<Type>::cross( const Vector2<Type> &pVector1, const Vector2<Type> &pVector2)
{
	return pVector1.x * pVector2.y - pVector2.x * pVector1.y;
}

template< class Type >
void 
Vector2<Type>::rotate( Type pTheta )
{
	Type tx = x;
	Type ty = y;
	Type cosTheta = cos( pTheta );
	Type sinTheta = sin( pTheta );
	
	x = tx * cosTheta - ty * sinTheta;
	y = ty * cosTheta + tx * sinTheta;
}

}

}

#endif