/** \file iso_math_vector_field.h
 * \class iso::math::VectorField template vector field
 * \brief template vector field
 *
 *  Created by Daniel Bisig on 4/27/07.
 */

#ifndef _iso_math_vector_field_h_
#define _iso_math_vector_field_h_

#include "iso_math_types.h"
#include "iso_math_exception.h"
#include "iso_math_vector.h"

namespace iso
{

namespace math
{

template< class Type > 
class VectorField
{
public:
	/**
	\brief create vector field
	\param pSize size of vector field (i.e. number of vectors)
	\param pVectorDim dimension of vectors
	*/
	VectorField(const Vector<unsigned int>& pSize, unsigned int pVectorDim);

	/**
	\brief create vector field
	\param pVectorField vector field
	*/
	VectorField(const VectorField<Type>& pVectorField);
	
	/**
	\brief destructor
	*/
	~VectorField();
	
	/**
	\brief copy operator
	\param pVectorField vectorfield to copy
	\return vectorfield
	\exception MathException if vector fields are incompatible
	*/
	const VectorField<Type>& operator= (const VectorField<Type>& pVectorField) throw (MathException);

	/**
	\brief return field dimension
	\return field dimension
	*/
	unsigned int fieldDim() const;
	
	/**
	\brief return vector dimension
	\return vector dimension
	*/
	unsigned int vectorDim() const;
	
	/**
	\brief return size of vector field
	\return size of vector field
	*/
	const Vector<unsigned int>& size() const;
	
	/**
	\brief return number of vectors
	\return number of vectors
	*/
	unsigned int vectorCount() const;

	/**
	 \brief return index offset
	 \return index offset
	 */
	const Vector<unsigned int> indexOffset() const;

	/**
	\brief return vectors
	\return vectors
	*/
	Vector<Type>** vectors();

	/**
	\brief return vectors
	\return vectors
	*/
	const Vector<Type>** vectors() const;

	/**
	\brief return sum of all vectors
	\return sum of all vectors
	*/
	Vector<Type> sum() const;

	/**
	\brief retrieve subfield
	\param pIndex position of subfield (top left)
	\param pSubField subfield
	\exception MathException dimension mismatch or out of bounds error
	*/
	void subField( const Vector<unsigned int>& pIndex, VectorField<Type>& pSubField ) throw (MathException);
	
	/**
	\brief return vector
	\param pIndex scalar index
	\return vector
	\exception MathException if index is out of bounds
	*/
	inline const Vector<Type>& operator[] (unsigned int pIndex) const throw (MathException);

	/**
	\brief return vector
	\param pIndex scalar index
	\return vector
	\exception MathException fails if index is out of bounds
	*/
	inline const Vector<Type>& get(unsigned int pIndex) const throw (MathException);
	
	/**
	\brief return vector
	\param pIndex nD index
	\return vector
	\exception MathException if dimension mismatch or index is out of bounds
	*/
	const Vector<Type>& get(const Vector<unsigned int>& pIndex) const throw (MathException);

	/**
	\brief return interpolated vector
	\param pIndex nD index
	\return vector
	\exception MathException if dimension mismatch or index is out of bounds
	
	perform linear interpolation
	*/
	Vector<Type> get(const Vector<real>& pIndex) const throw (MathException);

	/**
	\brief return interpolated vector
	\param pIndex nD index
	\param pResult interpolated vector
	\exception MathException if dimension mismatch or index is out of bounds
	
	perform linear interpolation
	*/
	void get(const Vector<real>& pIndex, Vector<Type>& pResult) const throw (MathException);
	
	/**
	\brief set vector
	\param pIndex scalar index
	\param pValue vector value
	\exception MathException Vector dimension mismatch or out of bounds error
	*/
	void set(unsigned int pIndex, const Vector<Type>& pValue) throw (MathException);
	
	/**
	\brief set vector
	\param pIndex vector index
	\param pValue vector value
	\exception MathException vector dimension mismatch or field dimension mismatch or index out of bounds
	*/
	void set(const Vector<unsigned int>& pIndex, const Vector<Type>& pValue) throw (MathException);

	/**
	\brief set vector (interpolated mode)
	\param pIndex vector index
	\param pValue vector value
	\exception MathException vector dimension mismatch or field dimension mismatch or index out of bounds
	*/
	void set(const Vector<real>& pIndex, const Vector<Type>& pValue) throw (MathException);

	/**
	\brief set vectors
	\param pStartIndex start vector index
	\param pEndIndex end vector index
	\param pValue vector value
	\exception MathException vector dimension mismatch or field dimension mismatch or index out of bounds or start index larger than end index
	*/
	void set(const Vector<unsigned int>& pStartIndex, const Vector<unsigned int>& pEndIndex, const Vector<Type>& pValue) throw (MathException);	

	/**
	\brief set all vector values
	\param pValue vector value
	\exception MathException vector dimension mismatch
	*/
	void set(const Vector<Type>& pValue) throw (MathException);

	/**
	\brief add vector
	\param pIndex scalar index
	\param pValue vector value
	\exception MathException Vector dimension mismatch or out of bounds error
	*/
	void add(unsigned int pIndex, const Vector<Type>& pValue) throw (MathException);
	
	/**
	\brief add vector
	\param pIndex vector index
	\param pValue vector value
	\exception MathException field dimension mismatch or index out of bounds or vector dimension mismatch
	*/
	void add(const Vector<unsigned int>& pIndex, const Vector<Type>& pValue) throw (MathException);

	/**
	\brief add vector (interpolated mode)
	\param pIndex vector index
	\param pValue vector value
	\exception MathException vector dimension mismatch or field dimension mismatch or index out of bounds
	*/
	void add(const Vector<real>& pIndex, const Vector<Type>& pValue) throw (MathException);

	/**
	\brief add vector
	\param pStartIndex start vector index
	\param pEndIndex end vector index
	\param pValue vector value
	\exception MathException vector dimension mismatch or field dimension mismatch or index out of bounds or start index larger than end index
	*/
	void add(const Vector<unsigned int>& pStartIndex, const Vector<unsigned int>& pEndIndex, const Vector<Type>& pValue) throw (MathException);	

	/**
	\brief add all vector values
	\param pValue vector value
	\exception MathException vector dimension mismatch
	*/
	void add(const Vector<Type>& pValue) throw (MathException);

	/**
	\brief vectorfield += vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/
    inline const VectorField<Type>& operator+=( const Vector<Type>& pVector ) throw (MathException);
 
	/**
	\brief vectorfield -= vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/	
	inline const VectorField<Type>& operator-=( const Vector<Type>& pVector ) throw (MathException); 

	/**
	\brief vectorfield *= vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/	
    inline const VectorField<Type>& operator*=( const Vector<Type>& pVector ) throw (MathException);

	/**
	\brief vectorfield /= vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/	
    inline const VectorField<Type>& operator/=( const Vector<Type>& pVector ) throw (MathException);
	
	/**
	\brief vectorfield + vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/	 
	inline VectorField<Type> operator+( const Vector<Type>& pVector ) const throw (MathException);
	
	/**
	\brief vectorfield - vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/
    inline VectorField<Type> operator-( const Vector<Type>& pVector ) const throw (MathException);
	
	/**
	\brief vectorfield * vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/
    inline VectorField<Type> operator*( const Vector<Type>& pVector ) const throw (MathException);
	
	/**
	\brief vectorfield / vector
	\param pVector vector
	\return vectorfield
	\exception MathException vector dimension mismatch
	*/
    inline VectorField<Type> operator/( const Vector<Type>& pVector ) const throw (MathException);

	/**
	\brief vectorfield += vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/
    inline const VectorField<Type>& operator+=( const VectorField<Type>& pVectorField ) throw (MathException); 
	
	/**
	\brief vectorfield -= vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/
    inline const VectorField<Type>& operator-=( const VectorField<Type>& pVectorField ) throw (MathException); 
	
	/**
	\brief vectorfield *= vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/
    virtual const VectorField<Type>& operator*=( const VectorField<Type>& pVectorField ) throw (MathException); 
	
	/**
	\brief vectorfield /= vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/
    inline const VectorField<Type>& operator/=( const VectorField<Type>& pVectorField ) throw (MathException); 

	/**
	\brief vectorfield + vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/
	inline VectorField<Type> operator+( const VectorField<Type>& pVectorField ) const throw (MathException);
	
	/**
	\brief vectorfield - vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/ 
    inline VectorField<Type> operator-( const VectorField<Type>& pVectorField ) const throw (MathException);
	
	/**
	\brief vectorfield * vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/
    inline VectorField<Type> operator*( const VectorField<Type>& pVectorField ) const throw (MathException); 
	
	/**
	\brief vectorfield / vectorfield
	\param pVectorField vectorfield
	\return vectorfield
	\exception MathException vector dimension or field dimension or field size mismatch 
	*/
    inline VectorField<Type> operator/( const VectorField<Type>& pVectorField ) const throw (MathException); 

	/**
	\brief calculate index
	\param pIndex nD index
	\return 1D index
	\remarks exception MathException dimension mismatch or index out of bounds
	
	calculates scalar index from an n-dimensional index
	*/
	inline unsigned int calcIndex(const Vector<unsigned int>& pIndex) const throw (MathException);

	/**
	 \brief calculate index
	 \param pIndex index
	 \return nD index
	 
	 calculates n-dimensional index from scalar index
	 */
	inline Vector<unsigned int> calcIndex(unsigned int pIndex) const;	

	/**
	 \brief calculate index
	 \param pIndex scalar index
	 \param pVecIndex n-dimensional index
	 
	 calculates n-dimensional index from scalar index
	 */
	inline void calcIndex(unsigned int pIndex, math::Vector<unsigned int>& pVecIndex) const;	
	
	/**
	\brief convolve vector field with supplied kernel
	\param pKernel convolution kernel
	\exception MathException kernel vector dimension or kernel field dimension mismatch or kernel size error (size not odd)
	*/
	void convolve( const VectorField<Type>& pKernel ) throw (MathException);

	/**
	\brief output textual vector field information
	\param os output stream
	\param pVectorField vector field
	\return output stream
	*/
    friend std::ostream& operator << ( std::ostream& os, const VectorField<Type> pVectorField )
    {
		os << "fieldDim " << pVectorField.fieldDim() << "\n";
		os << "vectorDim " << pVectorField.vectorDim() << "\n";
		os << "size " << pVectorField.size() << "\n";
			
		for(unsigned int i=0; i<pVectorField.mVectorCount; ++i) std::cout << i << " " << *(pVectorField.mVectors[i]) << "\n";
	
        return os;
    }; 

protected:
	/**
	\brief number of vectors
	*/
	unsigned int mVectorCount;

	/**
	\brief number of vectors in field
	*/
	Vector<unsigned int> mSize;
	
	/**
	\brief offsets for converting a relative index into an absolute index
	*/
	Vector<unsigned int> mIndexOffset;
	Vector<real> mInvIndexOffset;

	/**
	\brief vectors
	*/
	Vector<Type>** mVectors;
	
	unsigned int mInterpolVectorCount; ///\brief number of vectors to test for linear interpolation
};

};

};

using namespace iso;
using namespace iso::math;

template < class Type > 
VectorField< Type >::VectorField(const Vector<unsigned int>& pSize, unsigned int pVectorDim) 
	: mSize(pSize)
	, mIndexOffset( pSize.dim() )
	, mInvIndexOffset( pSize.dim() )
{
	mVectorCount = 1;
	for(unsigned int i=0; i<mSize.dim(); ++i) 
	{
		mVectorCount *= mSize[i];
		mIndexOffset[i] = 1;
		for(unsigned int j=0; j<i; ++j) mIndexOffset[i] *= mSize[j];
		
		mInvIndexOffset[i] = 1.0 / static_cast<real>( mIndexOffset[i] );
	}
	
	mVectors = new Vector<Type>*[mVectorCount];
	for(unsigned int i=0; i<mVectorCount; ++i) mVectors[i] = new Vector<Type>(pVectorDim);
	
	mInterpolVectorCount = static_cast<unsigned int>( pow( 2.0f, static_cast<float>( mSize.dim() ) ) );
}

template < class Type > 
VectorField< Type >::VectorField(const VectorField<Type>& pVectorField)
	: mSize(pVectorField.mSize)
	, mVectorCount(pVectorField.mVectorCount)
	, mIndexOffset( pVectorField.mIndexOffset )
	, mInvIndexOffset( pVectorField.mInvIndexOffset )	
{
	mVectors = new Vector<Type>*[mVectorCount];
	for(unsigned int i=0; i<mVectorCount; ++i) mVectors[i] = new Vector<Type>( *( pVectorField.mVectors[i] ) );
	mInterpolVectorCount = pVectorField.mInterpolVectorCount;
}

template < class Type > 
VectorField< Type >::~VectorField() 
{
	for(unsigned int i=0; i<mVectorCount; ++i) delete mVectors[i];
	
	delete [] mVectors;
}

template < class Type > 
const VectorField<Type>& 
VectorField< Type >::operator= (const VectorField<Type>& pVectorField) throw (MathException)
{
	#ifdef _math_debug_
	if(fieldDim() != pVectorField.fieldDim() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field dimension mismatch: expected %1 supplied %2").arg( fieldDim() ).arg( pVectorField.fieldDim() ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(mSize != pVectorField.mSize ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field size mismatch: expected %1 supplied %2").arg( mSize.info() ).arg( pVectorField.mSize.info() ), __FILE__, __FUNCTION__, __LINE__ );
	if(vectorDim() != pVectorField.vectorDim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: expected %1 supplied %2").arg( vectorDim() ).arg( pVectorField.vectorDim() ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	for(unsigned int i=0; i<mVectorCount; ++i) mVectors[i]->set( *( pVectorField.mVectors[i] ) );
	
	return pVectorField;
}

template < class Type > 
unsigned int 
VectorField< Type >::fieldDim() const
{
	return mSize.dim();
}

template < class Type > 
unsigned int 
VectorField< Type >::vectorDim() const
{
	return mVectors[0]->dim();
}

template < class Type > 
const Vector<unsigned int>& 
VectorField< Type >::size() const
{
	return mSize;
}

template < class Type > 
inline unsigned int 
VectorField< Type >::calcIndex(const Vector<unsigned int>& pIndex) const throw (MathException)
{
	#ifdef _math_debug_
	if(pIndex.dim() != mSize.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("index dim %1 does not match field dim %2").arg( pIndex.dim() ).arg( mSize.dim() ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pIndex >= mSize) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index %1 exceeds field size %2").arg( pIndex.info() ).arg( mSize.info() ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif

	unsigned int dim = pIndex.dim();
	unsigned int index = 0;
	for(int d = 0; d < dim; ++d) index += pIndex[d] * mIndexOffset[d];
	return index;	
}

template < class Type > 
inline Vector<unsigned int> 
VectorField< Type >::calcIndex(unsigned int pIndex) const
{
	unsigned int dim = mSize.dim();
	math::Vector<unsigned int> index( dim );
	
	for(int d = dim - 1; d >= 0; --d) 
	{
		index.c[d] = pIndex * mInvIndexOffset.c[d];
		pIndex -= index.c[d] * mIndexOffset.c[d];
	}
	
	return index;
}

template < class Type > 
inline void 
VectorField< Type >::calcIndex(unsigned int pIndex, math::Vector<unsigned int>& pVecIndex) const
{
	unsigned int dim = mSize.dim();
	if( pVecIndex.dim() != dim ) pVecIndex.setDim( dim );
	
	for(int d = dim - 1; d >= 0; --d) 
	{
		pVecIndex.c[d] = pIndex * mInvIndexOffset.c[d];
		pIndex -= pVecIndex.c[d] * mIndexOffset.c[d];
	}
}

template < class Type > 
unsigned int 
VectorField< Type >::vectorCount() const
{
	return mVectorCount;
}

template < class Type > 
const Vector<unsigned int>
VectorField< Type >::indexOffset() const
{
	return mIndexOffset;
}

template < class Type > 
Vector<Type>** 
VectorField< Type >::vectors()
{
	return mVectors;
}

template < class Type > 
const Vector<Type>** 
VectorField< Type >::vectors() const
{
	return const_cast< const Vector<Type>** >( mVectors );
}

template < class Type > 
Vector< Type > 
VectorField< Type >::sum() const
{
	Vector<Type> tmp( vectorDim() );
	for(unsigned int vI=0; vI<mVectorCount; ++vI) tmp += (*mVectors[vI]);
	return tmp;
}

template < class Type > 
void 
VectorField< Type >::subField( const Vector<unsigned int>& pIndex, VectorField<Type>& pSubField ) throw (MathException)
{
	try 
	{
		unsigned int fDim = mSize.dim();
		unsigned int vDim = mVectors[0]->dim();
		const math::Vector<unsigned int>& sfSize = pSubField.size(); // subfield size
		Vector<Type>** sfVectors = pSubField.vectors(); // subfield vectors
		const Vector<unsigned int>& sfIndexOffset = pSubField.mIndexOffset;

		#ifdef _math_debug_
		if(pIndex.dim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("index dim %1 does not match field dim %2").arg( pIndex.dim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
		if(pSubField.fieldDim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("subfield dim %1 does not match field dim %2").arg( pIndex.dim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ );
		if(pSubField.vectorDim() != vDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("subfield value dim %1 does not match field vector dim %2").arg( pSubField.vectorDim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
		for(unsigned int d=0; d<fDim; ++d) if(sfSize.c[d] > mSize.c[d]) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("at dim %1 subfield size: %2 is larger than vector field size %3").arg( d ).arg( sfSize.c[d] ).arg( mSize.c[d] ), __FILE__, __FUNCTION__, __LINE__ );
		#endif

		const math::Vector<unsigned int>& fStartIndex = pIndex; // relative field start index
		math::Vector<unsigned int> fEndIndex = fStartIndex + sfSize - 1; // relative field end index
		int fAbsStartIndex = calcIndex( fStartIndex ); // absolute field start index
		int fAbsEndIndex = calcIndex( fEndIndex ); // absolute subfield start index
		math::Vector<unsigned int> fCurRelIndex(fStartIndex); // running relative field index 
		math::Vector<unsigned int> sfCurRelIndex( pIndex.dim() ); // running relative subfield index
		
		int fAbsIndex = fAbsStartIndex; // absolute field index 
		int sfAbsIndex = 0; // absolute subfield index
		
		while( fAbsIndex < fAbsEndIndex )
		{
			// first dimension
			for(fCurRelIndex.c[0] = fStartIndex.c[0], sfCurRelIndex.c[0] = 0; fCurRelIndex[0] <= fEndIndex.c[0]; fCurRelIndex[0] += 1, sfCurRelIndex[0] += 1, fAbsIndex++, sfAbsIndex++)
			{
				//std::cout << "fCurRelIndex: " << fCurRelIndex << " sfCurRelIndex " << sfCurRelIndex << " fAbsIndex " << fAbsIndex << " sfAbsIndex " << sfAbsIndex << "\n";
		
				(*sfVectors[sfAbsIndex]) = *(mVectors[fAbsIndex]);
			}
			
			if(fAbsIndex >= fAbsEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, fCurRelIndex[d] > fEndIndex.c[d]; ++d)
			{
				fCurRelIndex[d] = fStartIndex.c[d];
				fCurRelIndex[d + 1] += 1;	
				fAbsIndex -= mIndexOffset[d] * ( fEndIndex.c[d] - fStartIndex.c[d] + 1);
				fAbsIndex += mIndexOffset[d + 1];
				
				sfCurRelIndex[d] = 0;
				sfCurRelIndex[d + 1] += 1;
				sfAbsIndex -= sfIndexOffset[d] * ( sfSize.c[d] );
				sfAbsIndex += sfIndexOffset[d + 1];
			}
		}
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type > 
const Vector<Type>&
VectorField< Type >::operator[] (unsigned int pIndex) const throw (MathException)
{
	#ifdef _math_debug_
	if(pIndex >= mVectorCount ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index out of bounds: max index %1 supplied index %2").arg( mVectorCount - 1 ).arg( pIndex ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif

	return *(mVectors[pIndex]);
}

template < class Type > 
const Vector<Type>& 
VectorField< Type >::get(unsigned int pIndex) const throw (MathException)
{
	#ifdef _math_debug_
	if(pIndex >= mVectorCount ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index out of bounds: max index %1 supplied index %2").arg( mVectorCount - 1 ).arg( pIndex ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif

	return *(mVectors[pIndex]);	
}

template < class Type > 
const Vector<Type>&
VectorField< Type >::get(const Vector<unsigned int>& pIndex) const throw (MathException)
{
	try
	{
		return *(mVectors[calcIndex(pIndex)]);
	}
	catch( MathException& e)
	{
		throw;
	}
}

template < class Type > 
Vector<Type> 
VectorField< Type >::get(const Vector<real>& pIndex) const throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pIndex.dim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("index dim %1 does not match field dim %2").arg( pIndex.dim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(mSize <= pIndex + 1.0) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index exceeds field size: fieldsize %1 supplied %2").arg( mSize.info() ).arg( pIndex.info() ), __FILE__, __FUNCTION__, __LINE__ );
	if( pIndex < 0.0 ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index smaller than zero" ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	try
	{
		math::Vector<unsigned int> startIndex( fDim );
		unsigned int localIndex[fDim];
		real ratios[2 * fDim ];
		math::Vector<real> result( vDim );
		float ratio; 

		for(unsigned int d=0; d<fDim; ++d) 
		{
			startIndex.c[d] = static_cast<unsigned int>( pIndex.c[d] );
			ratios[d] = 1.0 - ( pIndex[d] - floor( pIndex[d] ) );
			if(ratios[d] < 0) ratios[d] += 1.0;
			ratios[d + fDim] = 1.0 - ratios[d];
			localIndex[d] = 0;
		}
		
		int absStartIndex = calcIndex( startIndex );
		int absEndIndex = calcIndex( startIndex + 1 );
		
		if(absEndIndex >= mVectorCount) return result;
		
		int absIndex = absStartIndex;
		
		while( absIndex < absEndIndex )
		{			
			// first dimension
			for(localIndex[0] = 0; localIndex[0] < 2; localIndex[0] += 1, absIndex++)
			{		
				ratio = 1.0;
				for(unsigned int d=0; d<fDim; ++d) ratio *= ratios[ d + localIndex[d] * fDim ];				
				result += *(mVectors[absIndex]) * ratio;
			}
				
			if(absIndex >= absEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, localIndex[d] > 1; ++d)
			{
				localIndex[d] = 0;
				localIndex[d + 1] += 1;		
				absIndex -= 2 * mIndexOffset[d];
				absIndex += mIndexOffset[d + 1];
			}
		}
		
		return result;
	}
	catch(MathException& e)
	{
		throw;
	}
}


template < class Type > 
void
VectorField< Type >::get(const Vector<real>& pIndex, Vector<Type>& pResult) const throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pIndex.dim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("index dim %1 does not match field dim %2").arg( pIndex.dim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(mSize <= pIndex + 1.0) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index exceeds field size: fieldsize %1 supplied %2").arg( mSize.info() ).arg( pIndex.info() ), __FILE__, __FUNCTION__, __LINE__ );
	if( pIndex < 0.0 ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index smaller than zero" ), __FILE__, __FUNCTION__, __LINE__ );
	if( pResult.dim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("result dim %1 does not vector dim %2").arg( pResult.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		pResult.reset();
	
		math::Vector<unsigned int> startIndex( fDim );
		unsigned int localIndex[fDim];
		real ratios[2 * fDim ];
		float ratio; 

		for(unsigned int d=0; d<fDim; ++d) 
		{
			startIndex.c[d] = static_cast<unsigned int>( pIndex.c[d] );
			ratios[d] = 1.0 - ( pIndex.c[d] - floor( pIndex.c[d] ) );
			if(ratios[d] < 0) ratios[d] += 1.0;
			ratios[d + fDim] = 1.0 - ratios[d];
			localIndex[d] = 0;
		}
		
		int absStartIndex = calcIndex( startIndex );
		int absEndIndex = calcIndex( startIndex + 1 );
		
		if(absEndIndex >= mVectorCount) return;
		
		int absIndex = absStartIndex;
		
		while( absIndex < absEndIndex )
		{			
			// first dimension
			for(localIndex[0] = 0; localIndex[0] < 2; localIndex[0] += 1, ++absIndex)
			{		
				ratio = 1.0;
				for(unsigned int d=0; d<fDim; ++d) ratio *= ratios[ d + localIndex[d] * fDim ];				
				pResult += *(mVectors[absIndex]) * ratio;
			}
				
			if(absIndex >= absEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, localIndex[d] > 1; ++d)
			{
				localIndex[d] = 0;
				localIndex[d + 1] += 1;		
				absIndex -= 2 * mIndexOffset[d];
				absIndex += mIndexOffset[d + 1];
			}
		}
		
		return;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type > 
void 
VectorField< Type >::set(unsigned int pIndex, const Vector<Type>& pValue) throw (MathException)
{
	#ifdef _math_debug_
	if(pValue.dim() != mVectors[0]->dim() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: expected %1 supplied %2").arg( mVectors[0]->dim() ).arg( pValue.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	if(pIndex >= mVectorCount ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index exceeds vector count: vector count %1 index %2").arg( mVectorCount ).arg( pIndex ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	*(mVectors[pIndex]) = pValue;
}

template < class Type > 
void 
VectorField< Type >::set(const Vector<unsigned int>& pIndex, const Vector<Type>& pValue) throw (MathException)
{
	#ifdef _math_debug_
	if(pValue.dim() != mVectors[0]->dim() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: expected %1 supplied %2").arg( mVectors[0]->dim() ).arg( pValue.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	try
	{
		*(mVectors[calcIndex(pIndex)]) = pValue;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type > 
void 
VectorField< Type >::set(const Vector<real>& pIndex, const Vector<Type>& pValue) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pIndex.dim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("index dim %1 does not match field dim %2").arg( pIndex.dim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pValue.dim() != vDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("value dim %1 does not match field vector dim %2").arg( pValue.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(mSize <= pIndex ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index exceeds field size: fieldsize %1 supplied %2").arg( mSize.info() ).arg( pIndex.info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	try
	{
		math::Vector<unsigned int> startIndex( fDim );
		math::Vector<unsigned int> localIndex( fDim );
		math::Vector<real> ratios( 2 * fDim );
		float ratio;

		for(unsigned int d=0; d<fDim; ++d) 
		{
			startIndex.c[d] = static_cast<unsigned int>( pIndex.c[d] );
			ratios.c[d] = 1.0 - ( pIndex.c[d] - floor( pIndex.c[d] ) );
			if(ratios.c[d] < 0) ratios.c[d] += 1.0;
			ratios.c[d + fDim] = 1.0 - ratios.c[d];
		}
		
		int absStartIndex = calcIndex( startIndex );
		int absEndIndex = calcIndex( startIndex + 1 );
		
		if(absEndIndex >= mVectorCount) return;
		
		int absIndex = absStartIndex;

		real ratioSum = 0;
		
		while( absIndex < absEndIndex )
		{			
			// first dimension
			for(localIndex.c[0] = 0; localIndex.c[0] < 2; localIndex.c[0] += 1, absIndex++)
			{		
				ratio = 1.0;
				for(unsigned int d=0; d<fDim; ++d) ratio *= ratios.c[ d + localIndex.c[d] * fDim ];
				
				std::cout << "absIndex " << absIndex << " localIndex " << localIndex << " ratio " << ratio << "\n";
				
				*(mVectors[absIndex]) = pValue * ratio;
				
				ratioSum += ratio;
			}
				
			if(absIndex >= absEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, localIndex.c[d] > 1; ++d)
			{
				localIndex[d] = 0;
				localIndex[d + 1] += 1;		
				absIndex -= 2 * mIndexOffset[d];
				absIndex += mIndexOffset[d + 1];
			}
		}
		
		std::cout << ">> ratioSum " << ratioSum << "\n";

	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type > 
void 
VectorField< Type >::set(const Vector<unsigned int>& pStartIndex, const Vector<unsigned int>& pEndIndex, const Vector<Type>& pValue) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pStartIndex.dim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("start index dim %1 does not match field dim %2").arg( pStartIndex.dim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pEndIndex.dim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("end index dim %1 does not match field dim %2").arg( pEndIndex.dim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pValue.dim() != vDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("value dim %1 does not match field vector dim %2").arg( pValue.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pStartIndex > pEndIndex) 
	{
		std::cout << "start index " << pStartIndex << " larger than end index " << pEndIndex << "\n";
		throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("start index is larger than end index"), __FILE__, __FUNCTION__, __LINE__ ); 
	}
	if(pEndIndex >= mSize) 
	{
		std::cout << "end index " << pEndIndex << " larger than field size " << mSize << "\n";
		throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("end index is larger than field size"), __FILE__, __FUNCTION__, __LINE__ ); 
	}
	#endif

	try
	{
		int absStartIndex = calcIndex( pStartIndex );
		int absEndIndex = calcIndex( pEndIndex );
		math::Vector<unsigned int> curRelIndex(pStartIndex);
		
		int absIndex = absStartIndex;
		
		while( absIndex < absEndIndex )
		{
			// first dimension
			for(curRelIndex.c[0] = pStartIndex.c[0]; curRelIndex[0] <= pEndIndex.c[0]; curRelIndex[0] += 1, absIndex++)
			{
				std::cout << "curRelIndex: [";
				for(int i=0; i<fDim; ++i) std::cout << curRelIndex[i] << " ";
				std::cout << " ] ";
				std::cout << "absIndex: " << absIndex << "\n";

				*(mVectors[absIndex]) = pValue;
			}
			
			if(absIndex >= absEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, curRelIndex[d] > pEndIndex.c[d]; ++d)
			{
				curRelIndex[d] = pStartIndex.c[d];
				curRelIndex[d + 1] += 1;			
				absIndex -= mIndexOffset[d] * ( pEndIndex.c[d] - pStartIndex.c[d] + 1);
				absIndex += mIndexOffset[d + 1];
			}
		}
	}
	catch(MathException& e)
	{
		throw;
	}
}	

template < class Type > 
void 
VectorField< Type >::set(const Vector<Type>& pValue) throw (MathException)
{
	#ifdef _math_debug_
	if(pValue.dim() != mVectors[0]->dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("value dim %1 does not match field vector dim %2").arg( pValue.dim() ).arg( mVectors[0]->dim() ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif

	for(unsigned int i=0; i<mVectorCount; ++i) *(mVectors[i]) = pValue;
}

template < class Type > 
void 
VectorField< Type >::add(unsigned int pIndex, const Vector<Type>& pValue) throw (MathException)
{
	#ifdef _math_debug_
	if(pIndex >= mVectorCount ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index out of bounds: max index %1 supplied index %2").arg( mVectorCount - 1 ).arg( pIndex ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pValue.dim() != mVectors[0]->dim() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: expected %1 supplied %2").arg( mVectors[0]->dim() ).arg( pValue.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	*(mVectors[pIndex]) += pValue;
}

template < class Type > 
void 
VectorField< Type >::add(const Vector<unsigned int>& pIndex, const Vector<Type>& pValue) throw (MathException)
{
	#ifdef _math_debug_
	if(pValue.dim() != mVectors[0]->dim() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: expected %1 supplied %2").arg( mVectors[0]->dim() ).arg( pValue.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	try
	{
		*(mVectors[calcIndex(pIndex)]) += pValue;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type > 
void 
VectorField< Type >::add(const Vector<real>& pIndex, const Vector<Type>& pValue) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pValue.dim() != vDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("value dim %1 does not match field vector dim %2").arg( pValue.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		math::Vector<unsigned int> startIndex( fDim );
		math::Vector<unsigned int> localIndex( fDim );
		math::Vector<real> ratios( 2 * fDim );
		float ratio;
		
		for(unsigned int d=0; d<fDim; ++d) 
		{
			startIndex.c[d] = static_cast<unsigned int>( pIndex.c[d] );
			ratios.c[d] = 1.0 - ( pIndex.c[d] - floor( pIndex.c[d] ) );
			if(ratios.c[d] < 0) ratios.c[d] += 1.0;
			ratios.c[d + fDim] = 1.0 - ratios.c[d];
		}
		
		int absStartIndex = calcIndex( startIndex );
		int absEndIndex = calcIndex( startIndex + 1 );
		
		if(absEndIndex >= mVectorCount) return;
		
		int absIndex = absStartIndex;

		
		while( absIndex < absEndIndex )
		{			
			// first dimension
			for(localIndex.c[0] = 0; localIndex.c[0] < 2; localIndex.c[0] += 1, absIndex++)
			{		
				ratio = 1.0;
				for(unsigned int d=0; d<fDim; ++d) ratio *= ratios.c[ d + localIndex.c[d] * fDim ];
				*(mVectors[absIndex]) += pValue * ratio;
			}
				
			if(absIndex >= absEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, localIndex.c[d] > 1; ++d)
			{
				localIndex[d] = 0;
				localIndex[d + 1] += 1;		
				absIndex -= 2 * mIndexOffset[d];
				absIndex += mIndexOffset[d + 1];
			}
		}
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type > 
void 
VectorField< Type >::add(const Vector<unsigned int>& pStartIndex, const Vector<unsigned int>& pEndIndex, const Vector<Type>& pValue) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pValue.dim() != vDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("value dim %1 does not match field vector dim %2").arg( pValue.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pStartIndex > pEndIndex) 
	{
		std::cout << "start index " << pStartIndex << " larger than end index " << pEndIndex << "\n";
		throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("start index is larger than end index"), __FILE__, __FUNCTION__, __LINE__ ); 
	}

	#endif

	try
	{
		int absStartIndex = calcIndex( pStartIndex );
		int absEndIndex = calcIndex( pEndIndex );
		math::Vector<unsigned int> curRelIndex(pStartIndex);
		
		int absIndex = absStartIndex;
		
		while( absIndex < absEndIndex )
		{
			// first dimension
			for(curRelIndex.c[0] = pStartIndex.c[0]; curRelIndex[0] <= pEndIndex.c[0]; curRelIndex[0] += 1, absIndex++)
			{
				*(mVectors[absIndex]) += pValue;
			}
			
			if(absIndex >= absEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, curRelIndex[d] > pEndIndex.c[d]; ++d)
			{
				curRelIndex[d] = pStartIndex.c[d];
				curRelIndex[d + 1] += 1;			
				absIndex -= mIndexOffset[d] * ( pEndIndex.c[d] - pStartIndex.c[d] + 1);
				absIndex += mIndexOffset[d + 1];
			}
		}
	}
	catch(MathException& e)
	{
		throw;
	}
}	

template < class Type > 
void 
VectorField< Type >::add(const Vector<Type>& pValue) throw (MathException)
{
	#ifdef _math_debug_
	if(pValue.dim() != mVectors[0]->dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("value dim %1 does not match field vector dim %2").arg( pValue.dim() ).arg( mVectors[0]->dim() ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif

	for(unsigned int i=0; i<mVectorCount; ++i) *(mVectors[i]) += pValue;
}

template < class Type >
const VectorField< Type >& 
VectorField< Type >::operator+=( const Vector<Type>& pVector ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pVector.dim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVector.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) += pVector;
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >
const VectorField< Type >& 
VectorField< Type >::operator-=( const Vector<Type>& pVector ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pVector.dim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVector.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) -= pVector;
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >
const VectorField< Type >& 
VectorField< Type >::operator*=( const Vector<Type>& pVector ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pVector.dim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVector.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) *= pVector;
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >
const VectorField< Type >& 
VectorField< Type >::operator/=( const Vector<Type>& pVector ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();

	#ifdef _math_debug_
	if(pVector.dim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVector.dim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) /= pVector;
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}	
}

template < class Type >
VectorField< Type > 
VectorField< Type >::operator+( const Vector<Type>& pVector ) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field += pVector;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >
VectorField< Type >
VectorField< Type >::operator-( const Vector<Type>& pVector ) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field -= pVector;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >
VectorField< Type > 
VectorField< Type >::operator*( const Vector<Type>& pVector) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field *= pVector;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >
VectorField< Type >
VectorField< Type >::operator/( const Vector<Type>& pVector ) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field /= pVector;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >
const VectorField< Type >& 
VectorField< Type >::operator+=( const VectorField<Type>& pVectorField ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();
	Vector<Type>** pVectors = pVectorField.vectors();

	#ifdef _math_debug_
	if(pVectorField.fieldDim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field dimension mismatch: provided %1 expected %2").arg( pVectorField.fieldDim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pVectorField.vectorDim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVectorField.vectorDim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if( mSize != pVectorField.size() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field size mismatch: provided %1 expected %2"), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) += (*pVectors[vI]);
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}	
}

template < class Type >
const VectorField< Type >& 
VectorField< Type >::operator-=( const VectorField<Type>& pVectorField ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();
	Vector<Type>** pVectors = pVectorField.vectors();

	#ifdef _math_debug_
	if(pVectorField.fieldDim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field dimension mismatch: provided %1 expected %2").arg( pVectorField.fieldDim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pVectorField.vectorDim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVectorField.vectorDim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if( mSize != pVectorField.size() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field size mismatch: provided %1 expected %2"), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) -= (*pVectors[vI]);
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}	
}

template < class Type >
const VectorField< Type >& 
VectorField< Type >::operator*=( const VectorField<Type>& pVectorField ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();
	const Vector<Type>** pVectors = pVectorField.vectors();

	#ifdef _math_debug_
	if(pVectorField.fieldDim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field dimension mismatch: provided %1 expected %2").arg( pVectorField.fieldDim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pVectorField.vectorDim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVectorField.vectorDim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if( mSize != pVectorField.size() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field size mismatch: provided %1 expected %2"), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) *= (*pVectors[vI]);
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}	
}

template < class Type >  
const VectorField< Type >& 
VectorField< Type >::operator/=( const VectorField<Type>& pVectorField ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();
	Vector<Type>** pVectors = pVectorField.vectors();

	#ifdef _math_debug_
	if(pVectorField.fieldDim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field dimension mismatch: provided %1 expected %2").arg( pVectorField.fieldDim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pVectorField.vectorDim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("vector dimension mismatch: provided %1 expected %2").arg( pVectorField.vectorDim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if( mSize != pVectorField.size() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("field size mismatch: provided %1 expected %2"), __FILE__, __FUNCTION__, __LINE__ ); 
	#endif
	
	try
	{
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) /= (*pVectors[vI]);
		return(*this);
	}
	catch(MathException& e)
	{
		throw;
	}	
}

template < class Type >  
VectorField< Type > 
VectorField< Type >::operator+( const VectorField<Type>& pVectorField ) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field += pVectorField;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
VectorField< Type > 
VectorField< Type >::operator-( const VectorField<Type>& pVectorField ) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field -= pVectorField;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
VectorField< Type > 
VectorField< Type >::operator*( const VectorField<Type>& pVectorField ) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field *= pVectorField;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
VectorField< Type > 
VectorField< Type >::operator/( const VectorField<Type>& pVectorField ) const throw (MathException)
{
	try
	{
		VectorField<Type> field(*this);
		field /= pVectorField;
		return field;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type > 
void 
VectorField< Type >::convolve( const VectorField<Type>& pKernel ) throw (MathException)
{
	unsigned int fDim = mSize.dim();
	unsigned int vDim = mVectors[0]->dim();
	const math::Vector<unsigned int>& kernelSize = pKernel.size();

	#ifdef _math_debug_
	if(pKernel.fieldDim() != fDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("kernel field dim dim %1 does not vector match field dim %2").arg( pKernel.fieldDim() ).arg( fDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	if(pKernel.vectorDim() != vDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("kernel value dim %1 does not match vector value dim %2").arg( pKernel.vectorDim() ).arg( vDim ), __FILE__, __FUNCTION__, __LINE__ ); 
	for(unsigned int d=0; d<fDim; ++d) if(kernelSize.c[d] % 2 != 1) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("kernel size in dim %1 not odd").arg( d ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	try
	{
		math::Vector<unsigned int> kernelCenter = kernelSize / 2; // center position of kernel
		VectorField<Type> backupField( mSize, vDim ); // backup of vector field
		VectorField<Type> movingField( pKernel.size(), vDim ); // field section that movies together with kernel
		Vector<Type>** backupVectors = backupField.vectors();
		math::Vector<unsigned int> startIndex( fDim );
		math::Vector<unsigned int> endIndex(fDim);	
		int absIndexKernelCenterOffset = 0;
		for(unsigned int d=0; d<fDim; ++d) 
		{
			endIndex.c[d] = static_cast<unsigned int>( static_cast<int>( mSize.c[d] ) - kernelSize.c[d] );
			absIndexKernelCenterOffset += mIndexOffset.c[d] * kernelCenter.c[d];
		}
		
		int absStartIndex = calcIndex( startIndex );
		int absEndIndex = calcIndex( endIndex );
		math::Vector<unsigned int> curRelIndex(startIndex);
		math::Vector<unsigned int> curRelCenterIndex(startIndex);	
		int absIndex = absStartIndex;
		
	//	std::cout << "kernelSize " << kernelSize << "\n";
	//	std::cout << "kernelCenter " << kernelCenter << "\n";
	//	std::cout << "startIndex " << startIndex << "\n";
	//	std::cout << "endIndex " << endIndex << "\n";
	//	std::cout << "absIndex " << absIndex << "\n";
	//	std::cout << "absIndexKernelCenterOffset " << absIndexKernelCenterOffset << "\n";
		
		while( absIndex < absEndIndex )
		{
			// first dimension
			for(curRelIndex.c[0] = startIndex.c[0]; curRelIndex[0] <= endIndex.c[0]; curRelIndex[0] += 1, absIndex++)
			{
				//std::cout << "curRelIndex: " << curRelIndex << " absIndex: " << absIndex << " absCenterIndex " << absIndex + absIndexKernelCenterOffset << "\n";
				
				subField( curRelIndex, movingField );
				
				//std::cout << "movingField " << movingField << "\n";
				
				movingField *= pKernel;
				
				//std::cout << "movingField * Kernel " << movingField << "\n";
				
				(*backupVectors[absIndex + absIndexKernelCenterOffset]) = movingField.sum();
				
				//std::cout << "sum " << movingField.sum() << "\n";
			}
			
			if(absIndex >= absEndIndex) break;
			
			// successive dimensions
			for(unsigned int d = 0; d < fDim, curRelIndex[d] > endIndex.c[d]; ++d)
			{
				curRelIndex[d] = startIndex.c[d];
				curRelIndex[d + 1] += 1;			
				absIndex -= mIndexOffset[d] * ( endIndex.c[d] - startIndex.c[d] + 1);
				absIndex += mIndexOffset[d + 1];
			}
		}
		
		for(unsigned int vI=0; vI<mVectorCount; ++vI) (*mVectors[vI]) = (*backupVectors[vI]);
	}
	catch(MathException& e)
	{
		throw;
	}
}

#endif
