/**	\file iso_serialize_data.h
 *  \class iso::serialize::SerializeData serialization class
 *	\brief serialization class
 *
 *  The SerializeData class is a helper class for object serialization. 
 *  It stores all values as strings. It contains two hash tables, one for string / value pairs and one for string / SerializeData pairs.
 *  String / value pairs are used to store basic data types such as ints or arrays of ints.
 *  String / SerializeData pairs store member variables, which are instances of objects.
 *  Contrary to values, SerializeData objects can be stored with the same SerializeData name. Whenever there are several SerializeData objects stored under the same key, they can all
 *  be retrieved as list of SerializeData objects. Individual SerializeData objects within such a list can be retrieved by supplying both the key name and the SerializeData object name
 *  to the appropriate get method.
 *	Every class in iso, which can be serialized knows how to create a SerializeData object for itself. It also knows how to construct itself from a SerializeData object.
 *  SerializeData objects can't store addresses. So serializable objects which need to know the address of other objects need to chose an indirect approach when constructed via SerializeData objects. 
 *  The somewhat ugly solution is to store objects addresses and names temporarilly in the SerializeManager class and then look the addresses up baed on the object names which are stored in the SerializeData object.
 *
 *  Created by Daniel Bisig on 8/20/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef iso_serialize_data_h_
 #define iso_serialize_data_h_
 
 #include "iso_serialize_types.h"
 #include "iso_serialize_exception.h"
 #include <iso_data/iso_data_values.h>
 #include <QHash>
 #include <QVector>
 #include <iostream>
 
 namespace iso
 {
 
 namespace serialize
 {
 
 class SerializeData
 {
 public:
	/**
	\brief default constructor
	*/
	SerializeData();
	
	/**
	\brief copy constructor
	\param pSerial SerializeData object to copy properties from
	*/
	SerializeData(const SerializeData& pSerial);
	
	/**
	\brief destructor
	*/
	~SerializeData();
	
	/**
	\brief assignment operator
	\param pSerial SerializeData object to copy properties from
	*/
	const SerializeData& operator= (const SerializeData& pSerial);
	
	/**
	\brief check whether config is empty
	\return true: config is empty, false otherwise
	
	empty means: both the values and config hashtables contain no elements
	*/
	bool empty() const;
	
	/**
	\brief return number of Serialize Objects
	\return number of configs
	*/
	unsigned int serializeCount() const;
	
	/**
	\brief returns value hashtable
	\return values hashtable
	*/
	const QHash<base::String, base::String>& values() const;
	
	/**
	\brief returns SerializeData hashtable
	\return SerializeData hashtable 
	*/
	const QHash<base::String, SerializeData*>& serials() const;
	 
	 /**
	  \brief check if SerializeData contains value/serial name
	  \return true if SerializeData contains value/serial name, false otherwise
	  */	 
	bool contains( const base::String& pName ) const;

	 /**
	  \brief check if SerializeData contains value name
	  \return true if SerializeData contains value name, false otherwise
	  */	 
	 bool containsValue( const base::String& pName ) const;	 

	 /**
	  \brief check if SerializeData contains serial name
	  \return true if SerializeData contains serial name, false otherwise
	  */	 
	 bool containsSerial( const base::String& pName ) const;		 
	 
	/**
	\brief return bool value
	\param pValueName name of value
	\return bool
	\exception Exception value name not found
	*/
	bool boolValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return int value
	\param pValueName name of value
	\return int
	\exception Exception value name not found
	*/
	int intValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return unsigned int value
	\param pValueName name of value
	\return unsigned int
	\exception Exception value name not found
	*/
	unsigned int uintValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return long value
	\param pValueName name of value
	\return long
	\exception Exception value name not found
	*/
	long longValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return float value
	\param pValueName name of value
	\return float
	\exception Exception value name not found
	*/
	float floatValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return double value
	\param pValueName name of value
	\return double
	\exception Exception value name not found
	*/
	double doubleValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return real value
	\param pValueName name of value
	\return double
	\exception Exception value name not found
	*/
	real realValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return string value
	\param pValueName name of value
	\return string
	\exception Exception value name not found
	*/
	base::String stringValue(const base::String& pValueName) const throw (SerializeException);

	/**
	\brief return array of bool values
	\param pValueName name of values
	\param pValueCount will hold the size of the array
	\return array of bools
	\exception Exception value name not found
	*/
	bool* boolValues(const base::String& pValueName, unsigned int& pValueCount) const throw (SerializeException);

	/**
	\brief return array of int values
	\param pValueName name of values
	\param pValueCount will hold the size of the array
	\return array of ints
	\exception Exception value name not found
	*/
	int* intValues(const base::String& pValueName, unsigned int& pValueCount) const throw (SerializeException);

	/**
	\brief return array of unsigned int values
	\param pValueName name of values
	\param pValueCount will hold the size of the array
	\return array of unsigned ints
	\exception Exception value name not found
	*/
	unsigned int* uintValues(const base::String& pValueName, unsigned int& pValueCount) const throw (SerializeException);

	/**
	\brief return array of float values
	\param pValueName name of values
	\param pValueCount will hold the size of the array
	\return array of floats
	\exception Exception value name not found
	*/
	float* floatValues(const base::String& pValueName, unsigned int& pValueCount) const throw (SerializeException);

	/**
	\brief return array of double values
	\param pValueName name of values
	\param pValueCount will hold the size of the array
	\return array of doubles
	\exception Exception value name not found
	*/
	double* doubleValues(const base::String& pValueName, unsigned int& pValueCount) const throw (SerializeException);

	/**
	\brief return array of real values
	\param pValueName name of values
	\param pValueCount will hold the size of the array
	\return array of reals
	\exception Exception value name not found
	*/
	real* realValues(const base::String& pValueName, unsigned int& pValueCount) const throw (SerializeException);

	/**
	\brief return array of strings
	\param pValueName name of values
	\param pValueCount will hold the size of the array
	\return array of Strings
	\exception Exception value name not found
	*/
	base::String* stringValues(const base::String& pValueName, unsigned int& pValueCount) const throw (SerializeException);

	/**
	\brief return SerializeData object
	\param pSerialName name of SerializeData object
	\return SerializeData object
	\exception Exception SerliazeData name not found
	*/	
	const SerializeData& serial(const base::String& pSerialName) const throw (SerializeException);
	
	/**
	\brief return serializeData object from a list of serializeData objects
	\param pSerialName name of SerializeData object
	\param pName name of serializeData objects within list
	\return serializeData object
	\exception Exception pSerialName or pName not found
	
	pSerialName is the key value under which the SerializeData objects are stored in the hashtable
	pName is the internal SerializeData name, which serves to select a SerializeData object from within the list
	not all SerializeData objects possess a name value, therefore the applicability of this method is somewhat restricted
	*/		
	const SerializeData& serial(const base::String& pSerialName, const base::String& pName) const throw (SerializeException);
	
	/**
	\brief return list of SerializeData objects
	\param pSerialName name of SerializeData objects
	\return list of SerializeData objects
	\exception Exception SerliazeData name not found
	*/
	QList<SerializeData*> serials(const base::String& pSerialName) const throw (SerializeException);
	
	/**
	\brief add bool value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, bool pValue) throw (SerializeException);

	/**
	\brief add int value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, int pValue) throw (SerializeException);

	/**
	\brief add unsigned int value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValue) throw (SerializeException);

	/**
	\brief add long value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, long pValue) throw (SerializeException);

	/**
	\brief add float value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, float pValue) throw (SerializeException);

	/**
	\brief add double value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, double pValue) throw (SerializeException);

	/**
	\brief add string value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, const base::String& pValue) throw (SerializeException);

	/**
	\brief add string value
	\param pValueName value name
	\param pValue value
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, const char* pValue) throw (SerializeException);

	/**
	\brief add bool values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues array of values
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValueCount, const bool pValues[]) throw (SerializeException);

	/**
	\brief add int values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues array of values
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValueCount, const int pValues[]) throw (SerializeException);

	/**
	\brief add unsigned int values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues array of values
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValueCount, const unsigned int pValues[]) throw (SerializeException);

	/**
	\brief add long values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues array of values
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValueCount, const long pValues[]) throw (SerializeException);

	/**
	\brief add float values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues array of values
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValueCount, const float pValues[]) throw (SerializeException);

	/**
	\brief add double values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues array of values
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValueCount, const double pValues[]) throw (SerializeException);

	/**
	\brief add long values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues array of Strings
	\exception Exception duplicate pValueName
	*/
	void add(const base::String& pValueName, unsigned int pValueCount, const base::String pValues[]) throw (SerializeException);
	
	/**
	\brief add config
	\param pSerialName SerializeData name
	\param pSerial SerializeData
	*/
	void add(const base::String& pSerialName, const SerializeData& pSerial);	

	/**
	\brief replace bool value
	\param pValueName value name
	\param pValue new value
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, bool pValue) throw (SerializeException);

	/**
	\brief replace int value
	\param pValueName value name
	\param pValue new value
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, int pValue) throw (SerializeException);

	/**
	\brief replace unsigend int value
	\param pValueName value name
	\param pValue new value
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, unsigned int pValue) throw (SerializeException);

	/**
	\brief replace long value
	\param pValueName value name
	\param pValue new value
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, long pValue) throw (SerializeException);

	/**
	\brief replace float value
	\param pValueName value name
	\param pValue new value
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, float pValue) throw (SerializeException);

	/**
	\brief replace double value
	\param pValueName value name
	\param pValue new value
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, double pValue) throw (SerializeException);

	/**
	\brief replace string value
	\param pValueName value name
	\param pValue new value
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, const base::String& pValue) throw (SerializeException);

	/**
	\brief replace bool values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues new values
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, unsigned int pValueCount, bool pValues[]) throw (SerializeException);

	/**
	\brief replace int values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues new values
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, unsigned int pValueCount, int pValues[]) throw (SerializeException);

	/**
	\brief replace unsigned int values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues new values
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, unsigned int pValueCount, unsigned int pValues[]) throw (SerializeException);

	/**
	\brief replace long values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues new values
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, unsigned int pValueCount, long pValues[]) throw (SerializeException);

	/**
	\brief replace float values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues new values
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, unsigned int pValueCount, float pValues[]) throw (SerializeException);

	/**
	\brief replace double values
	\param pValueName value name
	\param pValueCount number of values
	\param pValues new values
	\exception Exception pValueName not found
	*/
	void replace(const base::String& pValueName, unsigned int pValueCount, double pValues[]) throw (SerializeException);

	/**
	\brief replace SerializeData
	\param pSerialName SerializeData name
	\param pSerial SerializeData
	\exception Exception pSerialName not found
	*/
	void replace(const base::String& pSerialName, const SerializeData& pSerial) throw (SerializeException);	

	//void replace(const Config& pConfig);

	/**
	\brief remove value
	\param pValueName value name
	\exception Exception value name not found
	*/	
	void remove(const base::String& pValueName) throw (SerializeException);

	/**
	\brief add SerializeData
	\param pSerial SerializeData

	extend SerializeData objects with all values / serials in pSerial
	*/		
	void add(const SerializeData& pSerial);
	
	/**
	\brief print array information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const SerializeData& pSerial)
	{
		QHashIterator<base::String, base::String> valueIter(pSerial.mValues);
		
		while (valueIter.hasNext()) 
		{
			valueIter.next();
			
			pOstream << valueIter.key().toStdString() << " : " << valueIter.value().toStdString() << "\n";
		}
		
		QHashIterator<base::String, SerializeData*> serialIter(pSerial.mSerials);
		
		while (serialIter.hasNext()) 
		{
			serialIter.next();
			
			pOstream << ">> " << serialIter.key().toStdString() << " :\n" << *(serialIter.value()) << "\n";
		}
		
		return pOstream;
	};
 
 protected:
	/**
	\brief remove all values and configs
	*/
	void clear();
 
	/**
	\brief return value as string
	\param pValueName name of value
	\return string
	\exception Exception value name not found
	*/
	base::String value(const base::String& pValueName) const throw (SerializeException);
 
	/**
	\brief hashtable of string / SerializeData pairs
	*/
	QHash<base::String, SerializeData*> mSerials;
	
	/**
	\brief hashtable of string / value pairs
	*/
	QHash<base::String, base::String> mValues;
 };
 
 };
 
 };
 
 #endif