/**	\file iso_serialize_serializable.h
 *  \class iso::serialize::Serializable abstract serializable object class
 *	\brief abstract serializable object class
 *
 *  abstract base class for all objects which are serializable
 *  
 *  in order for an object to be serializable is must have the following properties:
 *  a static classname : that's the name by which the class is identified within the xml document (if not overwritten)
 *  a method to create a serialize_data object which contains all the necessary information to build a class instance from it
 *  a constructor which accepts a serialize_data object in order to create a class instance
 *  a static method to create a class instance from a serialize_data object
 *  every serializable class needs to register itself automatically in the serialize_manager: i.e. const String Event::sClassName = SerializeManager::get().registerSerializable<Event>("Event");
 * 
 *
 *  Created by Daniel Bisig on 8/20/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef _iso_serializable_h_
 #define _iso_serializable_h_
 
 #include "iso_serialize_types.h"
 #include "iso_serialize_data.h"
 #include "iso_serialize_exception.h"
 
namespace iso
{

namespace serialize
{
 
class Serializable
{
public:
	/**
	\brief create SerializeData object
	\param pSerialName overwrites default serial name
	\return SerializeData
	*/
	virtual SerializeData serialize(const base::String& pSerialName = sClassName) const = 0;
	
	/**
	\brief create object from SerializeData
	\param pSerializeData SerializeData
	\return Serializable
	\exception SerializeException incompatible SerializeData
	*/
	static Serializable* deSerialize(const SerializeData& pSerializeData) throw (SerializeException)
	{ 
		return NULL;
	};

	/**
	\brief destructor
	*/
	virtual ~Serializable();
	
	/**
	\brief class name
	*/
	static const base::String sClassName;
	
	/**
	\brief check if class is serializable
	\return true
	
	this really is just a horrible hack in order to distinguish between 
	classes that derive from serializable and others that dont from the point of view a template classes or template functions.
	Whenever somebody is trying to use a non-serializable class as type in such a template class, a compiler error is generated
	*/
	//static bool isSerializable();
};
 
};
 
};
 
#endif