/**	\file iso_space.h
 *	\class iso::space::Space space
 *	\brief space
 *
 *
 *  Created by Daniel Bisig on 3/29/07.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

 #ifndef _iso_space_h_
 #define _iso_space_h_
 
 #include "iso_space_types.h"
 #include "iso_space_exception.h"
 #include "iso_space_object.h"
 #include "iso_space_proxy_object.h"
 #include "iso_space_neighbor_group.h"
 #include <iso_math/iso_math_vector.h>
 #include <QVector>
 #include <QMutex>
 
 namespace iso
 {
 
 namespace space
 {
 
 class SpaceAlg;
 
 class Space
 {
 
 public:
	static const base::String sClassName;
 
 	/**
	\brief create space
	\param pName space name
	\param pSpaceAlg space alorithm
	*/
	Space( const base::String& pName, SpaceAlg* pSpaceAlg );
	
	/**
	\brief destructor
	*/
	~Space();

	/**
	\brief return space name
	\return space name
	*/
	const base::String& name() const;

	/**
	\brief return space dimension
	\return space dimension
	*/
	unsigned int dim() const;
	
	/**
	\brief return space minimum position
	\return space minimum position
	*/
	const math::Vector<real>& minPos() const;

	/**
	\brief return space maximum position
	\return space maximum position
	*/
	const math::Vector<real>& maxPos() const;	
	
	/**
	\brief return space alg
	\return spaceAlg
	*/
	SpaceAlg* spaceAlg();

	/**
	\brief check if space contains space object
	\param pSpaceObject space object
	\return true if space contains space object, false otherwise
	*/
	bool checkObject(SpaceObject* pSpaceObject) const;
	
	/**
	\brief return space proxy object for space object
	\param pSpaceObject space object
	\return space proxy object
	\exception SpaceException fails if no space proxy object is found
	*/
	SpaceProxyObject* object(SpaceObject* pSpaceObject) throw (SpaceException);

	/**
	\brief return space all space objects
	\return all space objects
	*/
	QVector<SpaceProxyObject*>& objects();

	/**
	\brief add space object to space
	\param pObject space object
	\param pVisible visibility in neighbor space
	\param pNeighborGroupAlg neighbor group algorithm
	\exception SpaceException if object dimension of space object and space don't match
	
	automatically creates or updates the space objects's neighborGroup for this space
	*/
	virtual void addObject(SpaceObject* pObject, bool pVisible, NeighborGroupAlg* pNeighborGroupAlg = NULL) throw (SpaceException);

	 /**
	  \brief update space object
	  \param pObject space object
	  \param pVisible visibility in neighbor space
	  \param pNeighborGroupAlg neighbor group algorithm
	  \exception SpaceException object not stored in space
	  
	  automatically creates or updates the space objects's neighborGroup for this space
	  */
	 virtual void setObject(SpaceObject* pObject, bool pVisible, NeighborGroupAlg* pNeighborGroupAlg = NULL) throw (SpaceException);	 
	 
	/**
	\brief remove single space object handle object
	\param pObject single space object handle to remove
	\exception SpaceException fails if object is not stored in this space 
	*/
	virtual void removeObject(SpaceObject* pObject) throw (SpaceException);
	
	/**
	\brief update space
	\exception SpaceException failed to update space
	*/
	void update() throw (SpaceException);
	
  	/**
	\brief retrieve textual space info
	\param pOstream output text stream
	\param pSpace space
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const Space& pSpace )
    {
		pOstream << pSpace.info().toStdString(); 
					
        return pOstream;
    }; 
 
	/**
	\brief print space information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	base::String info(int pPropagationLevel = 0) const;
		
 protected:
	void updateObjects() throw(SpaceException);
 
	base::String mName; ///\brief name of space
 
	SpaceAlg* mSpaceAlg; ///\brief space algorithm
	QVector<SpaceProxyObject*> mObjects; ///\brief all space proxy objects in this space	
	QVector< SpaceProxyObject* > mVisibleObjects; ///\brief visible space proxy objects
	QVector< SpaceProxyObject* > mNeighborObjects; ///\brief space proxy objects that can possess neighbors
	
	QMutex mLock;
 };
 
 };
 
 };
 
 #endif