/** \file iso_space_exception.h
 *
 *	\brief class for handling all kinds of run time errors
 *
 *	Defines a variety of run time errors, which are specific to the space library:
 *  ERR_SPACE_SIZE : Space Size Error
 *  ERR_SPACE_DIM : Space Dimension Error
 *  ERR_UNKNOWN_SPACE : Space Not Found Error
 *  ERR_DUPLICATE_SPACE : Space Already Exists Error 
 *  ERR_SPACE_OBJECT_STORE : Space Object Store Error 
 *  ERR_SPACE : Generic Space Error
 *
 *  Created by Daniel Bisig on 10/4/07.
 */

#ifndef _iso_space_exception_h_
#define _iso_space_exception_h_

#include "iso_space_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace space
{

class SpaceException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	SpaceException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	SpaceException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);

	/**
	\brief copy constructor
	\param pException exception
	*/
	SpaceException( const Exception& pException );
	
	/**
	\brief destructor
	*/
	~SpaceException();
		
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const SpaceException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
			
protected:
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();
	
	/**
	\brief default constructur
	*/
	SpaceException();
};
	
};

};

#endif