/** \file iso_space_grid.h
 *
 *  Created by Daniel Bisig on 5/21/07.
 */

#ifndef _iso_space_grid_h_
#define _iso_space_grid_h_

#include <iso_math/iso_math_vector_field.h>
#include <iso_serialize/iso_serialize_includes.h>

namespace iso
{

namespace space
{

enum GridValueSetMode 
{
	Nearest,
	Interpol
};

class SpaceGrid : public serialize::Serializable
{
public:
	/**
	\brief default name
	*/
	static const base::String sClassName;

	/**
	\brief create space grid
	\param pValueDim dimension of values stored at each grid location
	\param pSubdivisionCount number of gread subdivisions in each dimension
	\param pMinPos minimum position of grid
	\param pMaxPos maximum position of grid
	*/
	SpaceGrid(unsigned int pValueDim, const Vector<unsigned int>& pSubdivisionCount, const Vector<real>& pMinPos, const Vector<real>& pMaxPos);

	/**
	\brief copy constructor
	\param pSpaceGrid space grid
	*/
	SpaceGrid(const SpaceGrid& pSpaceGrid);

	/**
	\brief serialization based constructor
	\param pSerializeData serialization data
	\exception SerializeException failed to create buffer
	*/
	SpaceGrid(const serialize::SerializeData& pSerializeData) throw (serialize::SerializeException);
	
	/**
	\brief destructor
	*/
	~SpaceGrid();

	/**
	\brief create serializeData for space grid
	\return serializeData
	*/
	virtual serialize::SerializeData serialize(const base::String& pSerialName = sClassName) const;

	/**
	\brief create space grid from SerializeData
	\param pSerializeData SerializeData
	\return Serializable
	\exception SerializeException incompatible SerializeData
	*/
	static serialize::Serializable* deSerialize(const serialize::SerializeData& pSerializeData) throw (serialize::SerializeException);
	
	/**
	\brief assignment operator
	\param pSpaceGrid space grid to copy
	\return SpaceGrid
	\remarks fails if space grids are not compatible
	*/
	const SpaceGrid& operator= ( const SpaceGrid& pSpaceGrid );

	/**
	\brief return dimension of space grid
	\return dimension of space grid
	*/
	unsigned int dim() const;
	
	/**
	\brief return dimension of space grid
	\return dimension of space grid
	*/
	unsigned int gridDim() const;

	/**
	\brief return dimension of values
	\return dimension of values
	*/
	unsigned int valueDim() const;
	
	/**
	\brief return subdivision count
	\return subdivision count
	*/
	const math::Vector<unsigned int>& subdivisionCount() const;
	
	/**
	\brief return vector field
	\return vector field
	*/
	VectorField<real>& vectorField();
	
	/**
	\brief return minimum position
	\return minimum position
	*/
	const Vector<real>& minPos() const;

	void setMinPos( const math::Vector<float>& pMinPos );
	
	/**
	\brief return maximum position
	\return maximum position
	*/
	const Vector<real>& maxPos() const;
	
	void setMaxPos( const math::Vector<float>& pMaxPos );
	
	/**
	\brief return grid index
	\param pPosition position
	*/
	math::Vector<unsigned int> position2index( const math::Vector<real>& pPosition ) const;

	/**
	\brief calculate grid index
	\param pPosition position
	\param pIndex resulting index
	*/
	void position2index( const math::Vector<real>& pPosition, unsigned int& pIndex ) const;

	/**
	\brief calculate grid index
	\param pPosition position
	\param pIndex resulting index
	*/
	void position2index( const math::Vector<real>& pPosition, math::Vector<unsigned int>& pIndex ) const;

	/**
	\brief calculate grid index
	\param pPosition position
	\param pIndex resulting index
	*/
	void position2index( const math::Vector<real>& pPosition, math::Vector<real>& pIndex ) const;
	
	math::Vector<real> index2position( unsigned int pIndex ) const;
	
	math::Vector<real> index2position( const math::Vector<unsigned int>& pIndex ) const;
	
	math::Vector<real> index2position( const math::Vector<real>& pIndex ) const;

	/**
	\brief return vector field
	\return vector field
	*/
	const VectorField<real>& vectorField() const;

	/**
	\brief get value at grid position
	\param pIndex grid index
	\return pGridValue grid value
	*/
	const math::Vector<real>& value(unsigned int pIndex) const;

	/**
	\brief get value at grid position
	\param pGridPosition grid position (with regard to subdivision count)
	\return pGridValue grid value
	*/
	const math::Vector<real>& value(const math::Vector<unsigned int>& pGridPosition) const;
	
	/**
	\brief return interpolated value
	\param pPosition position (with regard to grid size)
	\return interpolated value
	*/
	math::Vector<real> value(const math::Vector<real>& pPosition) const;

	/**
	\brief return interpolated value
	\param pPosition position (with regard to grid size)
	\param pValue interpolated value (written)
	*/
	void value(const math::Vector<real>& pPosition, math::Vector<real>& pValue) const;

	/**
	\brief get value at grid position
	\param pIndex grid index
	\return pGridValue grid value
	*/
	const math::Vector<real>& gridValue(unsigned int pIndex) const;

	/**
	\brief get value at grid position
	\param pGridPosition grid position (with regard to subdivision count)
	\return pGridValue grid value
	*/
	const math::Vector<real>& gridValue(const math::Vector<unsigned int>& pGridPosition) const;

	/**
	\brief set values
	\param pVectorField vector field
	\remarks fails if vector field doesn't match grid settings
	*/
	void setValues( const math::VectorField<real>& pVectorField );

	/**
	\brief set values
	\param pValue
	
	set all grid values
	*/
	void setValues(const math::Vector<real>& pValue);

	/**
	\brief change values
	\param pValue
	
	change all grid values
	*/
	void changeValues(const math::Vector<real>& pValue );

	/**
	\brief set value
	\param pPosition position
	\param pValue
	\param pSetMode value set mode
	*/
	void setValue(const math::Vector<real>& pPosition, const math::Vector<real>& pValue, GridValueSetMode pSetMode = Interpol );

	/**
	\brief change value
	\param pPosition position
	\param pValue
	\param pSetMode value set mode
	*/
	void changeValue(const math::Vector<real>& pPosition, const math::Vector<real>& pValue, GridValueSetMode pSetMode = Interpol );

	/**
	\brief set value at grid location
	\param pIndex grid index
	\param pGridValue grid value
	*/
	void setGridValue(unsigned int pIndex, const math::Vector<real>& pGridValue);
	
	/**
	\brief set value at grid location
	\param pGridPosition grid position
	\param pGridValue grid value
	*/
	void setGridValue(const math::Vector<unsigned int>& pGridPosition, const math::Vector<real>& pGridValue);

  	/**
	\brief retrieve textual space grid info
	\param pOstream output text stream
	\param pGrid space grid
	*/ 
	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceGrid& pGrid )
    {
		pOstream << pGrid.info().toStdString(); 
					
        return pOstream;
    }; 

	/**
	\brief print grid information
	*/
	virtual base::String info() const;

protected:
	/**
	\brief default constructor
	*/
	SpaceGrid();
	
	/**
	\brief dimension of space grid
	*/
	unsigned int mGridDim;

	/**
	\brief dimension of values
	*/
	unsigned int mValueDim;
	
	/**
	\brief minimum position of grid
	*/
	math::Vector<real> mMinPos;
	
	/**
	\brief maximum position of grid
	*/
	math::Vector<real> mMaxPos;
	
	/**
	\brief n dimensional field of vector values
	*/
	math::VectorField<real> mVectorField;

	/**
	\brief input position to grid position scale
	*/
	math::Vector<real> mPositionScale;
};

};

};

#endif