/**	\file iso_space_manager.h
 *	\class iso::space::SpaceManager space manager
 *	\brief space manager
 *
 *  handles all agent parameters that need neighborhood calculation
 *
 *  Created by Daniel Bisig on 3/29/07.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef _iso_space_manager_h_
 #define _iso_space_manager_h_
 
 #include "iso_space_types.h"
 #include "iso_space.h"
 #include <iso_data/iso_data_indexhash.h>
 
 namespace iso
 {
 
 namespace space
 {
 
 class SpaceObject;
 class SpaceShape;
 
 class SpaceManager
 {
 public:
 
	static const base::String sClassName;
 
	/**
	\brief get singleton instance
	\return space manager
	*/
	static SpaceManager& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy(); 
	 
	 /**
	  \brief return space names
	  \return space names
	  */
	 QList< base::String > spaceNames() const;
 
	/**
	\brief return space
	\param pSpaceName space nme
	\exception SpaceException space doesn't exist
	*/
	Space& space(const base::String& pSpaceName) throw (SpaceException);
 
 	/**
	\brief check if space exists
	\param pSpaceName space name
	\return true if space exists, false otherwise
	*/
	bool checkSpace(const base::String& pSpaceName);
	
	/**
	\brief add space to space manager
	\param pSpace space
	\exception SpaceException space already exists	
	*/
	void addSpace(Space* pSpace) throw (SpaceException);
 	
	/**
	\brief remove space
	\param pSpaceName name of space
	\exception SpaceException space does not exist
	*/
	void removeSpace(const base::String& pSpaceName) throw (SpaceException);
 
	/**
	 \brief remove all spaces
	 */
	 void removeSpaces();
	 
	/**
	\brief add space object to space
	\param pSpaceName name of space to add space object to
	\param pObject space object
	\param pVisible visibility in neighbor space
	\param pNeighborGroupAlg neighbor group algorithm
	\exception SpaceException space not found or space dimension and space object dimension don't match or space is of wrong type
	*/ 
 	virtual void addObject(const base::String& pSpaceName, SpaceObject* pObject, bool pVisible, NeighborGroupAlg* pNeighborGroupAlg = NULL) throw (SpaceException);

	/**
	\brief remove single space object handle object
	\param pSpaceName name of space to add space object to
	\param pObject single space object handle to remove
	\exception SpaceException space and or space object not found or space of wrong type
	*/
	virtual void removeObject(const base::String& pSpaceName, SpaceObject* pObject) throw (SpaceException);
 
	/**
	\brief update all parameters
	
	update parameter spaces
	*/
	void update();
 
  	/**
	\brief retrieve textual space manager info
	\param pOstream output text stream
	\param pSpaceManager space manager
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceManager& pSpaceManager )
    {
		pOstream << pSpaceManager.info().toStdString(); 
					
        return pOstream;
    }; 
 
	/**
	\brief print space manager information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;
 
 protected:
	/**
	\brief default constructor
	*/
	SpaceManager();
	
	/**
	\brief destructor
	*/
	~SpaceManager();
 
	/**
	\brief singleton instance
	*/
	static SpaceManager* sSpaceManager;
	
	/**
	\brief spaces
	*/
	data::IndexHash<base::String, Space> mSpaces;
	
 };
 
 };
 
 };
 
 #endif
 
