/** \file iso_space_neighbor_group.h
 *
 *  Created by Daniel Bisig on 4/6/07.
 */
 
#ifndef _iso_space_neighbor_group_h_
#define _iso_space_neighbor_group_h_
 
#include "iso_space_types.h"
#include <iso_math/iso_math_vector.h>
#include <QVector>

namespace iso
{

namespace space
{

class Space;
class SpaceObject;
class SpaceNeighborRelation;
class NeighborGroupAlg;

class NeighborGroup
{

friend class SpaceProxyObject;
friend class NeighborGroupAlg;

public:
	/**
	\brief class name
	*/
	static const base::String sClassName;

	/**
	\brief create neighbor collection
	\param pSpaceObject space object this neighborlist is part of
	\param pSpace space this neighborgroup is based on
	\param pVisible visiblity of space object
	*/
	NeighborGroup(SpaceObject* pSpaceObject, Space* pSpace, bool pVisible);
	
	/**
	\brief create neighbor collection
	\param pSpaceObject space object this neighbor list is part of
	\param pSpace space this neighborgroup is based on
	\param pVisible visiblity of space object
	\param pNeighborGroupAlg neighbor group algorithm
	*/
	NeighborGroup(SpaceObject* pSpaceObject, Space* pSpace, bool pVisible, NeighborGroupAlg* pNeighborGroupAlg);
	
	/**
	\brief destructor
	*/
	virtual ~NeighborGroup();

	/**
	\brief return space object this neighborlist is associated with
	\return space object 
	*/
	SpaceObject* spaceObject();

	/**
	\brief return space object this neighborlist is associated with
	\return space object 
	*/
	const SpaceObject* spaceObject() const;

	/**
	\brief return space name
	\return space name
	*/
	const base::String& name() const; 

	/**
	\brief return space this neighborlist is based on
	\return space
	*/
	Space* space();

	/**
	\brief check whether the space object is visible or not
	\return true: object is visible, false: otherwise
	
	a visible object can be a neighbor for another object in the same space, an invisible can't 
	*/
	bool visible() const;

	/**
	\brief set space object visibility
	\param pVisible object visibility
	*/
	void setVisible(bool pVisible);

	/**
	\brief check whether this neighbor group can have neighbors
	\return true: neighbor group can have neighbors, false: object can't have neighbors
	*/
	bool canHaveNeighbors() const;

	/**
	\brief return neighborhood algorithm
	\return neighborhood algorithm
	*/
	NeighborGroupAlg* neighborGroupAlg();
	
	/**
	\brief set neigbhorhood algorithm
	\param pNeighborGroupAlg neighborhood algorithm
	*/
	void setNeighborGroupAlg(NeighborGroupAlg* pNeighborGroupAlg);
	
	/**
	\brief return neighbor radius
	\return neighbor radius
	\remark fails if object can't have neighbors
	*/	
	real neighborRadius() const;

	/**
	\brief return number of neighbors
	\return number of neighbors
	*/
	unsigned int neighborCount() const;
	
	/**
	\brief return maximum number of neighbors
	\return maximum number of neighbors
	*/
	unsigned int maxNeighborCount() const;
	
	bool replaceNeighborMode() const;
	
	/**
	\brief returns true if no more neighbors can be added to neighbor list
	*/
	bool full() const;

	/**
	\brief set neighbor radius
	\param pNeighborRadius neighbor radius
	\remark fails if object can't have neighbors
	*/
	void setNeighborRadius(real pNeighborRadius);

	/**
	\brief return neighboring space object
	\param pIndex neighbor index
	\return neighboring space object
	\remarks fails if index is out of bounds
	*/
	SpaceObject* neighbor(unsigned int pIndex);

	/**
	\brief return neighboring space object
	\param pIndex neighbor index
	\return neighboring space object
	\remarks fails if index is out of bounds
	*/
	const SpaceObject* neighbor(unsigned int pIndex) const;

	/**
	\brief return value of neighbor relation
	\param pIndex neighbor index
	\returns value
	\remarks fails if index is out of bounds
	*/
	const math::Vector<real>& value(unsigned int pIndex) const;

	/**
	\brief return direction of neighbor relation
	\param pIndex neighbor index
	\returns direction
	\remarks fails if index is out of bounds
	*/
	const math::Vector<real>& direction(unsigned int pIndex) const;
	
	/**
	\brief return distance of neighbor relation
	\param pIndex neighbor index
	\return distance
	\remarks fails if index is out of bounds
	*/
	real distance(unsigned int pIndex) const;

	/**
	\brief return neighbor relations
	\return neighbor relations
	*/	
	QVector<SpaceNeighborRelation*>& neighborRelations();

	/**
	\brief return neighbor relation
	\param pIndex neighbor index
	\return neighbor relation
	\remarks fails if index is out of bounds
	*/
	SpaceNeighborRelation* neighborRelation(unsigned int pIndex);

	/**
	\brief return neighbor relation
	\param pIndex neighbor index
	\return neighbor relation
	\remarks fails if index is out of bounds
	*/
	const SpaceNeighborRelation* neighborRelation(unsigned int pIndex) const;

	/**
	\brief return neighbor relation
	\param pIndex neighbor index
	\return neighbor relation
	\remarks fails if index is out of bounds
	*/
	SpaceNeighborRelation* operator[](unsigned int pIndex);
	
	/**
	\brief add neighbor object to neighbor list
	\param pNeighborObject neighbor space object
	\return true, if neighbor object could be added, false otherwise
	
	creates new neighbor relation
	*/
	bool addNeighbor(SpaceObject* pNeighborObject);

	/**
	\brief add neighbor object to neighbor list
	\param pNeighborObject neighbor space object
	\param pDistance distance
	\param pDirection direction
	\return true, if neighbor object could be added, false otherwise
	
	creates new neighbor relation
	*/
	bool addNeighbor(SpaceObject* pNeighborObject, real pDistance, const math::Vector<real>& pDirection);

	/**
	\brief remove neighbor
	\param pNeighborObject neighbor space object
	*/
	void removeNeighbor(SpaceObject* pNeighborObject);

	/**
	\brief remove neighbor
	\param pNeighborIndex index of neighbor space object
	*/
	void removeNeighbor(unsigned int pNeighborIndex);
	
	/**
	\brief remove neighbors
	*/
	void removeNeighbors();

 	/**
	\brief retrieve textual neighbor group info
	\param pOstream output text stream
	\param pNeighborGroup neighbor group
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const NeighborGroup& pNeighborGroup )
    {
		pOstream << pNeighborGroup.info().toStdString(); 
					
        return pOstream;
    }; 
 
	/**
	\brief print neighbor collection information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;

protected:
	/**
	\brief default constructor
	*/
	NeighborGroup();

	/**
	\brief space object
	*/
	SpaceObject* mSpaceObject;

	/**
	\brief space this neighborlist is based on
	*/
	Space* mSpace;
	
	/**
	\brief visibility of space object in this space
	*/
	bool mVisible;
	
	/**
	\brief neighborhood algorithm
	*/
	NeighborGroupAlg* mNeighborGroupAlg;	

	/**
	\brief list of neighbor relations
	*/
	QVector<SpaceNeighborRelation*> mNeighborRelations;
};

};

};

#endif