/** \file  iso_space_neighbor_group_alg.h
 *
 *  Created by Daniel Bisig on 4/6/07.
 */

#ifndef _iso_space_neighbor_group_alg_h_
#define _iso_space_neighbor_group_alg_h_

#include "iso_space_types.h"
#include <iso_math/iso_math_vector.h>
#include <QVector>

namespace iso
{

namespace space
{

class SpaceObject;
class SpaceNeighborRelation;
class NeighborGroup;

class NeighborGroupAlg
{
public:
	/**
	\brief class name
	*/
	static const base::String sClassName;

	/**
	\brief create neighbor algorithm
	*/
	NeighborGroupAlg();

	/**
	\brief create neighbor algorithm
	\param pNeighborRadius radius within which neighbors are searched
	\param pMaxNeighborCount maximum number of neighbors in neighbor list if >= 0 (-1: no limit)
	\param pReplaceNeighborMode replace more distant neighbors with closer neighbors if true
	*/
	NeighborGroupAlg(real pNeighborRadius, int pMaxNeighborCount, bool pReplaceNeighborMode);

	/**
	\brief copy constructor
	\param pNeighborGroupAlg neighbor algorithm to copy
	*/
	NeighborGroupAlg(NeighborGroupAlg& pNeighborGroupAlg);
	
	/**
	\brief destructor
	*/
	virtual ~NeighborGroupAlg();
	
	/**
	\brief pass pointer to neigbhbor group
	\param pNeighborGroup neighbor group
	*/
	void setNeighborGroup(NeighborGroup* pNeighborGroup);
		
	/**
	\brief returns neighbor search radius
	\return neighbor search radius
	*/
	real neighborRadius() const;

	/**
	\brief return maximum number of neighbors
	\return maximum number of neighbors
	*/
	int maxNeighborCount() const;
	
	/**
	\brief return neighbor replacement mode
	\return neighbor replacement mode
	*/
	bool replaceNeighborMode() const;
		
	/**
	\brief check whether more neighbors can be stored
	*/
	virtual bool full() const;

	/**
	\brief set neighbor radius
	\param pNeighborRadius neighbor radius
	*/
	void setNeighborRadius(real pNeighborRadius);

	/**
	\brief add Neighbor
	\param pObject1 the space object a neighbor will be added to
	\param pObject2 the neighbor space object
	\return whether neighbor has been created or not
	*/
	virtual bool createNeighbor(SpaceObject* pObject1, SpaceObject* pObject2);

	/**
	\brief add Neighbor
	\param pObject1 the space object a neighbor will be added to
	\param pObject2 the neighbor space object
	\param pDistance distance
	\param pDirection direction
	\return whether neighbor has been created or not
	*/
	virtual bool createNeighbor(SpaceObject* pObject1, SpaceObject* pObject2, real pDistance, const math::Vector<real>& pDirection);

	/**
	\brief add Neighbor
	\param pObject1 the space object a neighbor will be added to
	\param pObject2 the neighbor space object
	\param pValue value
	\param pDirection direction
	\param pDistance distance
	\return whether neighbor has been created or not
	*/
	virtual bool createNeighbor(SpaceObject* pObject1, SpaceObject* pObject2, const math::Vector<real>& pValue, const math::Vector<real>& pDirection, real pDistance);

	/**
	\brief remove neighbor
	\param pNeighborObject neighbor space object
	*/
	virtual void removeNeighbor(SpaceObject* pNeighborObject);

	/**
	\brief remove neighbor
	\param pNeighborIndex index of neighbor space object
	*/
	virtual void removeNeighbor(unsigned int pNeighborIndex);
	
	/**
	\brief removes all neighbors from list
	*/
	virtual void removeNeighbors();

	/**
	\brief retrieve textual neighbor algorithm info
	\param pOstream output text stream
	\param pNeighborGroupAlg neighbor group algorithm
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const NeighborGroupAlg& pNeighborGroupAlg )
    {
		pOstream << pNeighborGroupAlg.info().toStdString(); 
					
        return pOstream;
    }; 
 
	/**
	\brief print neighbor algorithm information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;

protected:
	/**
	\brief default search radius for finding neighbors
	*/
	static real sNeighborRadius;
	
	/**
	\brief default maximum number of neighbors in neighbor list
	*/
	static int sMaxNeighborCount;
	
	/**
	\brief default replace more distant neighbors with closer neighbors
	*/
	static bool sReplaceNeighborMode;

	/**
	\brief search radius for finding neighbors
	*/
	real mNeighborRadius;
	
	/**
	\brief maximum number of neighbors in neighbor list
	*/
	int mMaxNeighborCount;
	
	/**
	\brief replace more distant neighbors with closer neighbors
	*/
	bool mReplaceNeighborMode;
	
	/**
	\brief pointer to space neighbors
	*/
	NeighborGroup* mNeighborGroup;

	math::Vector<real> mNeighborDirection; ///\brief neighbor direction helper variable
};

};

};

#endif