/**	\file iso_space_neighbors.h
 *	\class iso::space::Neighbors container class for space neighbor groups
 *	\brief container class for space neighbor groups
 *
 *  Created by Daniel Bisig on 10/6/07.
 */

#ifndef _iso_space_neighbors_h_
#define _iso_space_neighbors_h_

#include "iso_space_neighbor_group.h";
#include <iso_data/iso_data_indexhash.h>
#include <iso_math/iso_math_includes.h>
#include <QVector>

namespace iso
{

namespace space
{

class SpaceObject;
class SpaceNeighborRelation;
class NeighborGroupAlg;

class Neighbors
{

public:
	/**
	\brief create neighbor collection
	\param pSpaceObject space object this neighbor collection is part of
	*/
	Neighbors(SpaceObject* pSpaceObject);
	
	/**
	\brief destructor
	*/
	~Neighbors();
	
	/**
	\brief return number of neighbor groups
	\return number of neighbor groups
	*/
	unsigned int neighborGroupCount() const;

	/**
	\brief check whether neighbor collection for space exists
	\param pSpaceName space name
	\return true, if it exists, false otherwise
	*/
	bool checkNeighborGroup(const base::String& pSpaceName);

	/**
	\brief return neighbor group index
	\param pSpaceName space name
	\return neighbor group index
	\remarks fails if space object is not in particular space or can have neighbors in that particular space
	*/
	unsigned int neighborGroupIndex(const base::String& pSpaceName);

	/**
	\brief return neighbor collection based on index
	\param pIndex index neighbor collection
	\return neighbor collection
	\remarks fails if index is out of bounds
	*/
	NeighborGroup& neighborGroup(unsigned int pIndex);

	/**
	\brief return neighbor collection for a particular space
	\param pSpaceName space name
	\return neighbor collection
	\remarks fails if space object is not in particular space
	*/
	NeighborGroup& neighborGroup(const base::String& pSpaceName);

	/**
	\brief add new neighbor group
	\param pNeighborGroup new neighbor group
	\remarks fails if neighbor group for the associated space already exists
	*/
	void addNeighborGroup(NeighborGroup* pNeighborGroup);

	/**
	\brief remove neighbor group
	\param pNeighborGroup  neighbor group to remove
	\remarks fails if neighbor group for the associated space already exists
	*/
	void removeNeighborGroup(NeighborGroup* pNeighborGroup);

	/**
	\brief check whether this object can have neighbors in a particular space
	\param pSpaceName space name
	\return true: object can have neighbors, false: object can't have neighbors
	\remarks fails if object is not in particular space
	*/
	bool canHaveNeighbors(const base::String& pSpaceName) const;
	
	/**
	\brief check whether this object is visible or not
	\param pSpaceName name of space
	\return true: object is visible, false: otherwise
	\remark fails if object is not in particular space
	
	a visible object can be a neighbor for another object in the same space, an invisible can't 
	*/
	bool visible(const base::String& pSpaceName) const;

	/**
	\brief set object visibility in all spaces
	\param pVisible object visibility
	*/
	void setVisible(bool pVisible);

	/**
	\brief set object visibility in particular space
	\param pSpaceName name of space
	\param pVisible object visibility
	\remarks fails if object is not in particular space
	*/
	void setVisible(const base::String& pSpaceName, bool pVisible);

	/**
	\brief return neighbor radius
	\return neighbor radius
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	*/
	real neighborRadius(const base::String& pSpaceName) const;

	bool replaceNeighborMode(const base::String& pSpaceName) const;
	
	/**
	\brief return maximum number of neighbors
	\param pSpaceName space name
	\return maximum number of neighbors
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	*/
	unsigned int maxNeighborCount(const base::String& pSpaceName) const;

	/**
	\brief return number of neighbors
	\param pSpaceName space name
	\return number of neighbors
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	*/
	unsigned int neighborCount(const base::String& pSpaceName) const;

	/**
	\brief returns true if no more neighbors can be added to neighbor list
	\param pSpaceName space name
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	*/
	bool full(const base::String& pSpaceName) const;

	/**
	\brief return neighboring space object
	\param pSpaceName space name
	\param pIndex neighbor index
	\return neighboring space object
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space or if index is out of bounds
	*/
	SpaceObject* neighbor(const base::String& pSpaceName, unsigned int pIndex);

	/**
	\brief return direction vector pointing to neighboring space object
	\param pSpaceName space name
	\param pIndex neighbor index
	\returns direction
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space or if index is out of bounds
	*/
	const math::Vector<real>& direction(const base::String& pSpaceName, unsigned int pIndex) const;
	
	/**
	\brief return distance to neighboring space object
	\param pSpaceName space name
	\param pIndex neighbor index
	\return distance
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space or if index is out of bounds
	*/
	real distance(const base::String& pSpaceName, unsigned int pIndex) const;

	/**
	\brief return neighbor relations
	\param pSpaceName space name
	\return neighbor relations
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	*/	
	QVector<SpaceNeighborRelation*>& neighborRelations(const base::String& pSpaceName);

	/**
	\brief return neighbor relation
	\param pSpaceName space name
	\param pIndex neighbor index
	\return neighbor relation
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space or if index is out of bounds
	*/
	SpaceNeighborRelation* neighborRelation(const base::String& pSpaceName, unsigned int pIndex);

	/**
	\brief add neighbor object to neighbor list
	\param pSpaceName space name
	\param pNeighborObject neighbor space object
	\return true, if neighbor object could be added, false otherwise
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	
	creates new neighbor relation
	*/
	bool addNeighbor(const base::String& pSpaceName, SpaceObject* pNeighborObject);

	/**
	\brief add neighbor object to neighbor list
	\param pSpaceName space name
	\param pNeighborObject neighbor space object
	\param pDistance distance
	\param pDirection direction
	\return true, if neighbor object could be added, false otherwise
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	
	creates new neighbor relation
	*/
	bool addNeighbor(const base::String& pSpaceName, SpaceObject* pNeighborObject, real pDistance, const math::Vector<real>& pDirection);

	/**
	\brief remove all neighbors from all spaces
	*/
	void removeNeighbors();
	
	/**
	\brief remove all neighbors from a single space
	\remarks fails if space object is not in particular space or cannot have neighbors in that particular space
	*/
	void removeNeighbors(const base::String& pSpaceName);

 	/**
	\brief retrieve textual neighbor group info
	\param pOstream output text stream
	\param pNeighborGroup neighbor group
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const Neighbors& pNeighborGroup )
    {
		pOstream << pNeighborGroup.info().toStdString(); 
					
        return pOstream;
    }; 
 
	/**
	\brief print neighbor group information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;
	
protected:
	/**
	\brief default constructor
	*/
	Neighbors();

	/**
	\brief space object this neighborlists is part of
	*/
	SpaceObject* mSpaceObject;

	/**
	\brief storage for single space neighbor collections
	*/
	data::IndexHash<base::String, NeighborGroup> mNeighborGroups;
};

};

};

#endif