/**	\file iso_space_object.h
 *	\class iso::space::SpaceObject an object in n-dimensional space
 *	\brief an object in n-dimensional space
 *
 *  class representing an object in n-dimensional space.\n
 *  the class can possess neigboring space objects that are organized as neighbor relations in neighbor groups.\n
 *  
 *
 *  Created by Daniel Bisig on 3/23/07.
 */

#ifndef _iso_space_object_h_
#define _iso_space_object_h_

#include "iso_space_types.h"
#include "iso_space_neighbors.h"
#include <iso_math/iso_math_vector.h>
#include <QVector>

namespace iso
{

namespace space
{

class Neighbors;
class NeighborGroupAlg;
class SpaceNeighborRelation;

class SpaceObject
{

friend class SpaceProxyObject;

public:
	static const base::String sClassName; ///\brief class name

	/**
	\brief create space object
	\param pDim dimension of parameter (> 0)
	*/
	SpaceObject(unsigned int pDim);

	/**
	\brief create space object
	\param pPosition position of object
	*/
	SpaceObject(const math::Vector<real>& pPosition);

	/**
	\brief copy constructor
	\param pSpaceObject space object to copy
	*/
	SpaceObject(SpaceObject& pSpaceObject);
	
	/**
	\brief destructor
	*/
	virtual ~SpaceObject();
	
	/**
	\brief return space object id
	\return space object id
	*/
	unsigned int id() const;

	/**
	\brief return space object dimension
	\return space object dimension
	*/
	unsigned int dim() const;
	
	/**
	\brief return space object position
	\return space object position
	*/
	math::Vector<real>& position();

	/**
	\brief return space object position
	\return space object position
	*/
	const math::Vector<real>& position() const;
	
	/**
	\brief set space object position
	\param pPosition space object position
	\remarks fails if new position has wrong dimension
	*/
	virtual void setPosition(const math::Vector<real>& pPosition);

	/**
	\brief change space object position
	\param pPosition space object position offset
	\remarks fails if object is not in particular space
	*/
	virtual void changePosition(const math::Vector<real>& pPosition);

	/**
	\brief check whether space object has neighor group for particular space
	\param pSpaceName space name
	\return true if it has neighbor group for particular space, false otherwise
	*/
	bool checkNeighborGroup(const base::String& pSpaceName) const;

	/**
	\brief add new neighbor group
	\param pNeighborGroup new neighbor group
	\remarks fails if neighbor group for the associated space already exists
	*/
	void addNeighborGroup(NeighborGroup* pNeighborGroup);

	/**
	\brief remove neighbor group
	\param pNeighborGroup  neighbor group to remove
	\remarks fails if neighbor group for the associated space already exists
	*/
	void removeNeighborGroup(NeighborGroup* pNeighborGroup);

	/**
	\brief check whether this object is visible in a particular space or not
	\param pSpaceName name of space
	\return true: object is visible, false: otherwise
	\remark fails if object is not in particular space
	
	a visible object can be a neighbor for another object in the same space, an invisible can't 
	*/
	bool visible(const base::String& pSpaceName) const;

	/**
	\brief set object visibility in all spaces
	\param pVisible object visibility
	*/
	void setVisible(bool pVisible);

	/**
	\brief set object visibility in a particular space
	\param pSpaceName name of space
	\param pVisible object visibility
	\remarks fails if object is not in particular space
	*/
	void setVisible(const base::String& pSpaceName, bool pVisible);

	/**
	\brief return neighbors
	\return neighbors
	*/
	Neighbors& neighbors();		

	/**
	\brief return neighbor group index
	\param pSpaceName space name
	\return neighbor group index
	\remarks fails if space object is not in particular space or can have neighbors in that particular space
	*/
	unsigned int neighborGroupIndex(const base::String& pSpaceName);

	/**
	\brief return neighbor group based on index
	\param pIndex neighbor group index
	\return neighbor group
	\remarks fails if index is out of bounds
	*/
	NeighborGroup& neighborGroup(unsigned int pIndex);

	/**
	\brief return neighbor group
	\param pSpaceName space name
	\return neighbor group
	\remarks fails if space object is not in particular space or can have neighbors in that particular space
	*/
	NeighborGroup& neighborGroup(const base::String& pSpaceName);	

	/**
	\brief return neighbor space
	\param pSpaceName space name
	\return neighbor space
	\remarks fails if space object is not in particular space
	*/
	Space& neighborSpace(const base::String& pSpaceName);	

	/**
	\brief check whether this object can have neighbors in a particular space
	\param pSpaceName space name
	\return true: object can have neighbors, false: object can't have neighbors
	\remarks fails if object is not in particular space
	*/
	bool canHaveNeighbors(const base::String& pSpaceName) const;

	/**
	\brief return number maximum of neighbors
	\param pSpaceName space name
	\return maximum number of neighbors
	\remarks fails if object is not in particular space
	*/
	unsigned int maxNeighborCount(const base::String& pSpaceName) const;

	/**
	\brief return number of neighbors
	\param pSpaceName space name
	\return number of neighbors
	\remarks fails if object is not in particular space
	*/
	unsigned int neighborCount(const base::String& pSpaceName) const;

	/**
	\brief return neighbor radius
	\param pSpaceName space name
	\return neighbor radius
	\remarks fails if object is not in particular space
	*/
	real neighborRadius(const base::String& pSpaceName) const;

	bool replaceNeighborMode(const base::String& pSpaceName) const;
	
	/**
	\brief return neighboring object
	\param pSpaceName space name
	\param pIndex neighboring object index
	\return object
	\remarks fails space object is not in particular space or if this object cannot have any neighbors or if pIndex is out of bounds
	*/
	virtual SpaceObject* neighbor(const base::String& pSpaceName, unsigned int pIndex);

	/**
	\brief return neighbor relation
	\param pSpaceName space name
	\param pIndex neighbor index
	\return neighbor relation
	\remarks fails space object is not in particular space or if this object cannot have any neighbors or if pIndex is out of bounds
	*/
	SpaceNeighborRelation* neighborRelation(const base::String& pSpaceName, unsigned int pIndex);
	
	/**
	\brief returns true if no more neighbors can be added to neighbor list
	\param pSpaceName space name
	\remarks fails space object is not in particular space
	*/
	bool neighborListFull(const base::String& pSpaceName) const;
	
	/**
	\brief add neighbor object to neighbor list
	\param pSpaceName space name
	\param pNeighborObject neighbor object
	\return true, if neighbor object could be added, false otherwise
	\remarks fails if space object is not in particular space or can have neighbors in that particular space
	*/
	bool addNeighbor(const base::String& pSpaceName, SpaceObject* pNeighborObject);

	/**
	\brief add neighbor object to neighbor list
	\param pSpaceName space name
	\param pNeighborObject neighbor object
	\param pDistance distance
	\param pDirection direction
	\return true, if neighbor object could be added, false otherwise
	\remarks fails if space object is not in particular space or can have neighbors in that particular space
	*/
	bool addNeighbor(const base::String& pSpaceName, SpaceObject* pNeighborObject, real pDistance, const math::Vector<real>& pDirection);
	
	/**
	\brief remove all neighbors in all spaces
	*/
	void removeNeighbors();

	/**
	\brief remove all neighbors in particular space
	\param pSpaceName space name
	\remarks fails if space object is not in particular space
	*/
	void removeNeighbors(const base::String& pSpaceName);

 	/**
	\brief obtain textual space object information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;
 
	/**
	\brief retrieve textual space object info
	\param pOstream output text stream
	\param pSpaceObject space object
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceObject& pSpaceObject )
    {
		pOstream << pSpaceObject.info().toStdString(); 
					
        return pOstream;
    }; 
	
protected:
	static unsigned int sObjectCount; ///\brief space object count 

	/**
	\brief default constructor
	*/
	SpaceObject();

	unsigned int mId; /// \brief unique id
 	unsigned int mDim; ///\brief dimension
 	math::Vector<real> mPosition; ///\brief position
	Neighbors* mNeighbors; ///\brief neighbors
};

};

};

#endif