/** \file iso_space_proxy_object.h
 *
 *  proxy object class that mimics a space object containing a single neighbor group\n
 *  serves as an intermediate object for spaces to handle space objects
 * 
 *  Created by Daniel Bisig on 10/7/07.
 *
 */

#ifndef _iso_space_proxy_object_h_
#define _iso_space_proxy_object_h_

#include "iso_space_object.h"
#include "iso_space_neighbor_group.h"
#include "iso_space_neighbor_group_alg.h"
#include <iso_math/iso_math_includes.h>

namespace iso
{

namespace space
{

class SpaceObject;
class NeighborGroup;

class SpaceProxyObject
{
public:
	SpaceProxyObject(SpaceObject* pSpaceObject, NeighborGroup* pNeighborGroup);
	~SpaceProxyObject();
	
	inline SpaceObject* spaceObject();
	inline const SpaceObject* spaceObject() const;
	inline NeighborGroup* neighborGroup();
	inline const NeighborGroup* neighborGroup() const;

	/**
	\brief return space object dimension
	\return space object dimension
	*/
	inline unsigned int dim() const;

	/**
	\brief return space object position
	\return space object position
	*/
	inline math::Vector<real>& position();

	/**
	\brief check whether the space object is visible or not
	\return true: object is visible, false: otherwise
	
	a visible object can be a neighbor for another object in the same space, an invisible can't 
	*/
	inline bool visible() const;

	/**
	\brief check whether this object can have neighbors
	\return true: object can have neighbors, false: object can't have neighbors
	*/
	inline bool canHaveNeighbors() const;

	/**
	\brief returns true if no more neighbors can be added to neighbor list
	*/
	inline bool neighborListFull() const;

	/**
	\brief return neighbor radius
	\return neighbor radius
	\remark fails if object can't have neighbors
	*/	
	inline real neighborRadius() const;

	/**
	\brief return maxmimum number of neighbors
	\return return maxmimum number of neighbors
	\remark fails if object can't have neighbors
	*/	
	inline int maxNeighborCount() const;

	/**
	\brief remove neighbors
	*/
	inline void removeNeighbors();

	/**
	\brief add neighbor object to neighbor list
	\param pNeighborObject neighbor space object
	\return true, if neighbor object could be added, false otherwise
	
	creates new neighbor relation
	*/
	inline bool addNeighbor(SpaceObject* pNeighborObject);

	/**
	\brief add neighbor object to neighbor list
	\param pNeighborObject neighbor space object
	\param pDistance distance
	\param pDirection direction
	\return true, if neighbor object could be added, false otherwise
	
	creates new neighbor relation
	*/
	inline bool addNeighbor(SpaceObject* pNeighborObject, real pDistance, const math::Vector<real>& pDirection);

	/**
	\brief add neighbor object to neighbor list
	\param pNeighborObject neighbor space object
	\param pValue value
	\param pDirection direction
	\param pDistance distance
	\return true, if neighbor object could be added, false otherwise
	
	creates new neighbor relation
	*/
	inline bool addNeighbor(SpaceObject* pNeighborObject, const math::Vector<real>& pValue, const math::Vector<real>& pDirection, real pDistance);

 	/**
	\brief obtain textual space proxy object information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;
 
	/**
	\brief retrieve textual space proxy object info
	\param pOstream output text stream
	\param pSpaceProxyObject space proxy object
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceProxyObject& pSpaceProxyObject )
    {
		pOstream << pSpaceProxyObject.info().toStdString(); 
					
        return pOstream;
    }; 

protected:
	SpaceProxyObject();

	SpaceObject* mSpaceObject;
	NeighborGroup* mNeighborGroup; 

};

SpaceObject* 
SpaceProxyObject::spaceObject()
{
	return mSpaceObject;
}

const SpaceObject* 
SpaceProxyObject::spaceObject() const
{
	return mSpaceObject;
}

NeighborGroup* 
SpaceProxyObject::neighborGroup()
{
	return mNeighborGroup;
}

const NeighborGroup* 
SpaceProxyObject::neighborGroup() const
{
	return mNeighborGroup;
}

unsigned int 
SpaceProxyObject::dim() const
{
	return mSpaceObject->mDim;
}

math::Vector<space::real>& 
SpaceProxyObject::position()
{
	return mSpaceObject->mPosition;
}

bool 
SpaceProxyObject::visible() const
{
	return mNeighborGroup->mVisible;
}

bool 
SpaceProxyObject::canHaveNeighbors() const
{
	return ( mNeighborGroup->mNeighborGroupAlg != NULL && mNeighborGroup->mNeighborGroupAlg->maxNeighborCount() ) ? true : false;
}

bool 
SpaceProxyObject::neighborListFull() const
{
	#ifdef _space_debug_
	assert(mNeighborGroup->mNeighborGroupAlg != NULL && "neighbor collection alg is NULL");
	#endif

	return mNeighborGroup->mNeighborGroupAlg->full();
}

space::real
SpaceProxyObject::neighborRadius() const
{
	#ifdef _space_debug_
	assert(mNeighborGroup->mNeighborGroupAlg != NULL && "neighbor collection alg is NULL");
	#endif

	return mNeighborGroup->mNeighborGroupAlg->neighborRadius();
}

int
SpaceProxyObject::maxNeighborCount() const
{
	#ifdef _space_debug_
	assert(mNeighborGroup->mNeighborGroupAlg != NULL && "neighbor collection alg is NULL");
	#endif

	return mNeighborGroup->mNeighborGroupAlg->maxNeighborCount();
}

void 
SpaceProxyObject::removeNeighbors()
{
	if(mNeighborGroup->mNeighborGroupAlg != NULL) mNeighborGroup->mNeighborGroupAlg->removeNeighbors();
}

bool 
SpaceProxyObject::addNeighbor(SpaceObject* pNeighborObject)
{
	#ifdef _space_debug_
	assert(mNeighborGroup->mNeighborGroupAlg != NULL && "neighbor collection alg is NULL");
	#endif

	return mNeighborGroup->mNeighborGroupAlg->createNeighbor(mSpaceObject, pNeighborObject);
}

bool 
SpaceProxyObject::addNeighbor(SpaceObject* pNeighborObject, real pDistance, const math::Vector<real>& pDirection)
{
	#ifdef _space_debug_
	assert(mNeighborGroup->mNeighborGroupAlg != NULL && "neighbor collection alg is NULL");
	#endif

	return mNeighborGroup->mNeighborGroupAlg->createNeighbor(mSpaceObject, pNeighborObject, pDistance, pDirection);	
}

bool 
SpaceProxyObject::addNeighbor(SpaceObject* pNeighborObject, const math::Vector<real>& pValue, const math::Vector<real>& pDirection, real pDistance)
{
	#ifdef _space_debug_
	assert(mNeighborGroup->mNeighborGroupAlg != NULL && "neighbor collection alg is NULL");
	#endif

	return mNeighborGroup->mNeighborGroupAlg->createNeighbor(mSpaceObject, pNeighborObject, pValue, pDirection, pDistance);	
}

};

};

#endif
